' ###########################################################
' #								      #
' # 	         E-money Growth Forecasting Programme		      #
' #								      #
' ###########################################################



' CREATION OF WORKFILE AND READING OF DATA
' ***************************************************************

wfcreate(wf=emoney, page=monthly) m 1997m09 2013m05
read(a2, s=sheet1) "c:\Progs\ch3\dataset\emoney.xls" 17


' DECLARATION OF WILDCARDS
' ***************************************

!n_mod = 28	' no. component models
!t = 129		' sample length
!h = 60		' forecast hoirzon
!tt = !t + !h	' sample length + forecast horizon


' UNIT ROOT TESTING
' *************************

output(t) "C:\Progs\ch3\forecast\adf\adf"

' including constant and constant and trend for level, just constant for first difference
do cpi.adf(t,p)
	do cpi.adf(dif=1,p)
do cur_ki.adf(t,p)
	do cur_ki.adf(dif=1,p)
do cur_m2.adf(t,p)
	do cur_m2.adf(dif=1,p)
do dep_ki.adf(t,p)
	do dep_ki.adf(dif=1,p)
do dep_m2.adf(t,p)
	do dep_m2.adf(dif=1,p)
do ip_ki.adf(t,p)
	do ip_ki.adf(dif=1,p)
do m2_ki.adf(t,p)
	do m2_ki.adf(dif=1,p)
do tec_i.adf(t,p)
	do tec_i.adf(dif=1,p)
do tot_ki.adf(t,p)
	do tot_ki.adf(dif=1,p)
do tot_m2.adf(t,p)
	do tot_m2.adf(dif=1,p)

' including just constant for both level and first difference
do cur_m2.adf(p)
	do cur_m2.adf(dif=1,p)
do ndq_ki.adf(p)
	do ndq_ki.adf(dif=1,p)
do r.adf(p)
	do r.adf(dif=1,p)


' HP FILTERING
' *****************

smpl @all
tot_m2.hpf(lambda=14400) tot_m2_hp
smpl 1997m9 2008m5
graph hpf_gr.line tot_m2 tot_m2_hp
graph hpf_error_gr.line (tot_m2-tot_m2_hp)
	hpf_error_gr.draw(line,left) 0 0



' ESTIMATION OF COMPONENT MODELS
' *************************************************

' Benchmark geometric random walk model

smpl 1997:9 2008:5
equation grwd.ls log(tot_m2_hp) c ar(1)
smpl 2008:6 2013:5
grwd.forecast grwd_f grwd_se
smpl @all
graph grwd_gr.line tot_m2_hp grwd_f grwd_f+2*grwd_se grwd_f-2*grwd_se
	grwd_gr.draw(line, bottom) 2008:5
series grwd_growth = 1200*log(grwd_f/grwd_f(-1))
graph grwd_growth_gr.line grwd_growth
	grwd_growth_gr.draw(line,bottom) 2008:5 2008:5


' AR(p) models

' AR(2)
smpl 1997:9 2008:5
equation ar2.ls log(tot_m2_hp) c ar(1) ar(2)
smpl 2008:6 2013:5
ar2.forecast ar2_f ar2_se

' AR(3)
smpl 1997:9 2008:5
equation ar3.ls log(tot_m2_hp) c ar(1) ar(2) ar(3)
smpl 2008:6 2013:5
ar3.forecast ar3_f ar3_se

' AR(4)
smpl 1997:9 2008:5
equation ar4.ls log(tot_m2_hp) c ar(1) ar(2) ar(3) ar(4)
smpl 2008:6 2013:5
ar4.forecast ar4_f ar4_se


' MA(q) models

' MA(1)
smpl 1997:9 2008:5
equation ma1.ls log(tot_m2_hp) c ma(1)
smpl 2008:6 2013:5
ma1.forecast ma1_f ma1_se

' MA(2)
smpl 1997:9 2008:5
equation ma2.ls log(tot_m2_hp) c ma(1) ma(2)
smpl 2008:6 2013:5
ma2.forecast ma2_f ma2_se

' MA(3)
smpl 1997:9 2008:5
equation ma3.ls log(tot_m2_hp) c ma(1) ma(2) ma(3)
smpl 2008:6 2013:5
ma3.forecast ma3_f ma3_se

' MA(4)
smpl 1997:9 2008:5
equation ma4.ls log(tot_m2_hp) c ma(1) ma(2) ma(3) ma(4)
smpl 2008:6 2013:5
ma4.forecast ma4_f ma4_se


' ARIMA(p,d,q) models

' ARIMA(1,1,1)
smpl 1997:9 2008:5
equation arima111.ls d(log(tot_m2_hp)) c ar(1) ma(1)
smpl 2008:6 2013:5
arima111.forecast arima111_f arima111_se

' ARIMA(1,1,2)
smpl 1997:9 2008:5
equation arima112.ls d(log(tot_m2_hp)) c ar(1) ma(1) ma(2)
smpl 2008:6 2013:5
arima112.forecast arima112_f arima112_se

' ARIMA(1,1,3)
smpl 1997:9 2008:5
equation arima113.ls d(log(tot_m2_hp)) c ar(1) ma(1) ma(2) ma(3)
smpl 2008:6 2013:5
arima113.forecast arima113_f arima113_se

' ARIMA(1,1,4)
smpl 1997:9 2008:5
equation arima114.ls d(log(tot_m2_hp)) c ar(1) ma(1) ma(2) ma(3) ma(4)
smpl 2008:6 2013:5
arima114.forecast arima114_f arima114_se

' ARIMA(2,1,1)
smpl 1997:9 2008:5
equation arima211.ls d(log(tot_m2_hp)) c ar(1) ar(2) ma(1)
smpl 2008:6 2013:5
arima211.forecast arima211_f arima211_se

' ARIMA(2,1,2)
smpl 1997:9 2008:5
equation arima212.ls d(log(tot_m2_hp)) c ar(1) ar(2) ma(1) ma(2)
smpl 2008:6 2013:5
arima212.forecast arima212_f arima212_se

' ARIMA(2,1,3)
smpl 1997:9 2008:5
equation arima213.ls d(log(tot_m2_hp)) c ar(1) ar(2) ma(1) ma(2) ma(3)
smpl 2008:6 2013:5
arima213.forecast arima213_f arima213_se

' ARIMA(2,1,4)
smpl 1997:9 2008:5
equation arima214.ls d(log(tot_m2_hp)) c ar(1) ar(2) ma(1) ma(2) ma(3) ma(4)
smpl 2008:6 2013:5
arima214.forecast arima214_f arima214_se

' ARIMA(3,1,1)
smpl 1997:9 2008:5
equation arima311.ls d(log(tot_m2_hp)) c ar(1) ar(2) ar(3) ma(1)
smpl 2008:6 2013:5
arima311.forecast arima311_f arima311_se

' ARIMA(3,1,2)
smpl 1997:9 2008:5
equation arima312.ls d(log(tot_m2_hp)) c ar(1) ar(2) ar(3) ma(1) ma(2)
smpl 2008:6 2013:5
arima312.forecast arima312_f arima312_se

' ARIMA(3,1,3)
smpl 1997:9 2008:5
equation arima313.ls d(log(tot_m2_hp)) c ar(1) ar(2) ar(3) ma(1) ma(2) ma(3)
smpl 2008:6 2013:5
arima313.forecast arima313_f arima313_se

' ARIMA(3,1,4)
smpl 1997:9 2008:5
equation arima314.ls d(log(tot_m2_hp)) c ar(1) ar(2) ar(3) ma(1) ma(2) ma(3) ma(4)
smpl 2008:6 2013:5
arima314.forecast arima314_f arima314_se

' ARIMA(4,1,1)
smpl 1997:9 2008:5
equation arima411.ls d(log(tot_m2_hp)) c ar(1) ar(2) ar(3) ar(4) ma(1)
smpl 2008:6 2013:5
arima411.forecast arima411_f arima411_se

' ARIMA(4,1,2)
smpl 1997:9 2008:5
equation arima412.ls d(log(tot_m2_hp)) c ar(1) ar(2) ar(3) ar(4) ma(1) ma(2)
smpl 2008:6 2013:5
arima412.forecast arima412_f arima412_se

' ARIMA(4,1,3)
smpl 1997:9 2008:5
equation arima413.ls d(log(tot_m2_hp)) c ar(1) ar(2) ar(3) ar(4) ma(1) ma(2) ma(3)
smpl 2008:6 2013:5
arima413.forecast arima413_f arima413_se

' ARIMA(4,1,4)
smpl 1997:9 2008:5
equation arima414.ls d(log(tot_m2_hp)) c ar(1) ar(2) ar(3) ar(4) ma(1) ma(2) ma(3) ma(4)
smpl 2008:6 2013:5
arima414.forecast arima414_f arima414_se


' VAR(p) models

' VAR(1)
smpl 1997:9 2008:5
var var1.ls 1 1 d(log(tot_m2_hp)) d(log(cur_m2)) d(log(dep_m2)) d(log(ip_ki)) d(log(ndq_ki)) d(log(r)) d(log(tec_i))
var1.makemodel(var1_mod)
var1_mod.scenario(n,a=_v1) "var1"
smpl 2008m6 2013m5
var1_mod.solve (i=p,s=s)

' VAR(2)
smpl 1997:9 2008:5
var var2.ls 1 2 d(log(tot_m2_hp)) d(log(cur_m2)) d(log(dep_m2)) d(log(ip_ki)) d(log(ndq_ki)) d(log(r)) d(log(tec_i))
var2.makemodel(var2_mod)
var2_mod.scenario(n,a=_v2) "var2"
smpl 2008m6 2013m5
var2_mod.solve (i=p,s=s)

' VAR(3)
smpl 1997:9 2008:5
var var3.ls 1 3 d(log(tot_m2_hp)) d(log(cur_m2)) d(log(dep_m2)) d(log(ip_ki)) d(log(ndq_ki)) d(log(r)) d(log(tec_i))
var3.makemodel(var3_mod)
var3_mod.scenario(n,a=_v3) "var3"
smpl 2008m6 2013m5
var3_mod.solve (i=p,s=s)

' VAR(4)
smpl 1997:9 2008:5
var var4.ls 1 4 d(log(tot_m2_hp)) d(log(cur_m2)) d(log(dep_m2)) d(log(ip_ki)) d(log(ndq_ki)) d(log(r)) d(log(tec_i))
var4.makemodel(var4_mod)
var4_mod.scenario(n,a=_v4) "var4"
smpl 2008m6 2013m5
var4_mod.solve (i=p,s=s)


' VEC(p) models - trace stat indicates r=5

' VECM(1)
smpl 1997:9 2008:5
var vec1.ec(d,5) 1 1 log(tot_m2_hp) log(cur_m2) log(dep_m2) log(ip_ki) log(ndq_ki) log(r) log(tec_i)
vec1.makemodel(vec1_mod)
vec1_mod.scenario(n,a=_vc1) "vec1"
smpl 2008m6 2013m5
vec1_mod.solve (i=p,s=s)

' VECM(2)
smpl 1997:9 2008:5
var vec2.ec(d,5) 1 2 log(tot_m2_hp) log(cur_m2) log(dep_m2) log(ip_ki) log(ndq_ki) log(r) log(tec_i)
vec2.makemodel(vec2_mod)
vec2_mod.scenario(n,a=_vc2) "vec2"
smpl 2008m6 2013m5
vec2_mod.solve (i=p,s=s)

' VECM(3)
smpl 1997:9 2008:5
var vec3.ec(d,5) 1 3 log(tot_m2_hp) log(cur_m2) log(dep_m2) log(ip_ki) log(ndq_ki) log(r) log(tec_i)
vec3.makemodel(vec3_mod)
vec3_mod.scenario(n,a=_vc3) "vec3"
smpl 2008m6 2013m5
vec3_mod.solve (i=p,s=s)

' VECM(4)
smpl 1997:9 2008:5
var vec4.ec(d,5) 1 4 log(tot_m2_hp) log(cur_m2) log(dep_m2) log(ip_ki) log(ndq_ki) log(r) log(tec_i)
vec4.makemodel(vec4_mod)
vec4_mod.scenario(n,a=_vc4) "vec4"
smpl 2008m6 2013m5
vec4_mod.solve (i=p,s=s)


' COMPUTATION OF AVERAGE FORECAST
' ****************************************************

vector(!tt) for_1
	stom(grwd_f,for_1)
vector(!tt) for_2
	stom(ar2_f,for_2)
vector(!tt) for_3
	stom(ar3_f,for_3)
vector(!tt) for_4
	stom(ar4_f,for_4)

vector(!tt) for_5
	stom(arima111_f,for_5)
vector(!tt) for_6
	stom(arima112_f,for_6)
vector(!tt) for_7
	stom(arima113_f,for_7)
vector(!tt) for_8
	stom(arima114_f,for_8)

vector(!tt) for_9
	stom(arima211_f,for_9)
vector(!tt) for_10
	stom(arima212_f,for_10)
vector(!tt) for_11
	stom(arima213_f,for_11)
vector(!tt) for_12
	stom(arima214_f,for_12)

vector(!tt) for_13
	stom(arima311_f,for_13)
vector(!tt) for_14
	stom(arima312_f,for_14)
vector(!tt) for_15
	stom(arima313_f,for_15)
vector(!tt) for_16
	stom(arima314_f,for_16)

vector(!tt) for_17
	stom(arima411_f,for_17)
vector(!tt) for_18
	stom(arima412_f,for_18)
vector(!tt) for_19
	stom(arima413_f,for_19)
vector(!tt) for_20
	stom(arima414_f,for_20)

vector(!tt) for_21
	stom(tot_m2_hp_v1,for_21)
vector(!tt) for_22
	stom(tot_m2_hp_v2,for_22)
vector(!tt) for_23
	stom(tot_m2_hp_v3,for_23)
vector(!tt) for_24
	stom(tot_m2_hp_v4,for_24)

vector(!tt) for_25
	stom(tot_m2_hp_vc1,for_25)
vector(!tt) for_26
	stom(tot_m2_hp_vc2,for_26)
vector(!tt) for_27
	stom(tot_m2_hp_vc3,for_27)
vector(!tt) for_28
	stom(tot_m2_hp_vc4,for_28)

For !i=1 to !n_mod
matrix(!h,!n_mod) for_m
matplace(for_m,for_!i,1,!i)
Next

For !j = 1 to !h
vector(!h) for_mean_v
for_mean_v(!j)=@mean(@rowextract(for_m,!j))
Next

series for_mean=na
mtos(for_mean_v,for_mean)


' PLOTTING FORECAST GRAPHS
' ***************************************

smpl @all
graph ave_f_gr.line tot_m2_hp for_mean
	ave_f_gr.draw(line,bottom) 2008m5 2008m5

vector(!tt) ave_tt
stomna(tot_m2_hp,ave_tt)
matplace(ave_tt,for_mean_v,130,1)

series ave=na
mtos(ave_tt,ave)

series ave_growth = 1200*log(ave/ave(-1))

graph ave_growth_gr.line ave_growth
	ave_growth_gr.draw(line,bottom) 2008:5 2008:5


' END OF PROGRAMME

