' ###########################################################
' #								      #
' # 	         E-money Growth Forecasting Programme		      #
' #								      #
' ###########################################################



' CREATION OF THE WORKFILE
' **************************************

wfcreate(wf=gomp, page=monthly) m 1997m09 2013m05
read(a2, s=sheet1) "C:\Progs\ch3\dataset\emoney.xls" 17


' DECLARATION OF WILDCARDS
' ***************************************

!t=129
!tt=189
!h=!tt-!t
!n=200

series t=@trend

smpl 1997:9 2008:5


' HP FILTERING
' *****************
tot_m2.hpf(lambda=14400) tot_m2_hp


' RUN INITIAL REGRESSION
' *********************************

coef(2) gomp
param gomp(1) 0.1 gomp(2) 1.5

' Estimation of gamma (gomp(1)) and beta (gomp(2))
equation gomp_eq.ls log(d(log(tot_m2_hp)))=-gomp(1)*t+log(gomp(2)*exp(gomp(1))-gomp(2))
gomp_eq.makeresids gomp_res

' Computation of the upper asymptote (saturation level)
series ahat=exp(log(tot_m2_hp)+gomp(2)*exp(-gomp(1)*t))

' Out-of-sample forecasting
smpl 2008:6 2013:5
gomp_eq.forecast(f=na) gomp_f


' NON-PARAMETRIC BOOTSTRAPPING
' **********************************************

smpl 1997:9 2008:5

' Construction of y* by resampling the errors
For !i = 1 to !t
For !j = 1 to !n
vector(!t) ehat
stom(gomp_res, ehat)
vector(!t) e_star_vec_!j
e_star_vec_!j = @resample(ehat,1)
matrix(!t,!n) e_star_mat
matplace(e_star_mat,e_star_vec_!j,1,!j)
vector(!t) gompertz
gompertz = -gomp(1)*t+log(gomp(2)*exp(gomp(1))-gomp(2))
matrix(!t,!n) y_star
y_star(!i,!j) = e_star_mat(!i,!j)+gompertz(!i)
vector(!t) v_!j
vector v_!j = @columnextract(y_star, !j)
series vv_!j=na
mtos(v_!j,vv_!j)
Next
Next

' Estimation of the bootstrap equations
For !i = 1 to !t
For !j = 1 to !n
For !k = 1 to !h
coef(!n) gomp_b1
param gomp_b1(!j) 0.1
coef(!n) gomp_b2
param gomp_b2(!j) 1.5
smpl 1997:9 2008:5
equation bootstrap_!j.ls vv_!j=-gomp_b1(!j)*t+log(gomp_b2(!j)*exp(gomp_b1(!j))-gomp_b2(!j))
smpl 2008:6 2013:5
bootstrap_!j.forecast(f=na) bootstrap_!j_f
vector(!h) bootstrap_!j_ff
stom(bootstrap_!j_f,bootstrap_!j_ff)
Next
Next
Next

' Reintegration of the forecasts
' !init = 8.24399220317345E-05
!init = 9.34124883881428E-05
smpl 2008:6 2013:5
For !m = 1 to !n
vector(!h) x_star_!m_f
For !o = 1 to !h
if !o=1 then
x_star_!m_f(!o) = exp(exp(bootstrap_!m_ff(!o))+log(!init))
else
x_star_!m_f(!o) = exp(exp(bootstrap_!m_ff(!o))+log(x_star_!m_f(!o-1)))
endif
Next
Next

' Computation of the bootstrap mean and se
For !i = 1 to !t
For !j = 1 to !n
For !k = 1 to !h
matrix(!h,!n) xbf_mat
matplace(xbf_mat,x_star_!j_f,1,!j)
vector(!h) bootstrap_mean
vector(!h) bootstrap_se
bootstrap_mean(!k)=@mean(@rowextract(xbf_mat,!k))
bootstrap_se(!k)=@sqr(@var(@rowextract(xbf_mat,!k)))
Next
Next
Next
series boot_mean=na
mtos(bootstrap_mean,boot_mean)
series boot_se=na
mtos(bootstrap_se,boot_se)


' GRAPHING
' **************

' h-step ahead interval forecast graph
smpl @all
graph gomp_f_gr.line tot_m2_hp ahat gomp_f gomp_f+2*boot_se gomp_f-2*boot_se
	gomp_f_gr.draw(shade, bottom) 2008:5 2008:5

' h-step ahead growth rate graph
vector(!h) gomp_f_vec
stom(gomp_f,gomp_f_vec)
vector(!tt) gomp_tt
stomna(tot_m2_hp,gomp_tt)
matplace(gomp_tt,gomp_f_vec,130,1)
series gomp_for=na
mtos(gomp_tt,gomp_for)
series gomp_growth = 1200*log(gomp_for/gomp_for(-1))
graph gomp_growth_gr.line gomp_growth
	gomp_growth_gr.draw(line,bottom) 2008:5 2008:5


' PSEUDO OUT-OF-SAMPLE FORECASTING
' ****************************************************

smpl 1997m9 2007m5
coef(2) gomp_pos
param gomp_pos(1) 0.1 gomp_pos(2) 1.5

' Estimation of gamma (gomp_pos(1)) and beta (gomp_pos(2))
equation gomp_pos_eq.ls log(d(log(tot_m2_hp)))=-gomp_pos(1)*t+log(gomp_pos(2)*exp(gomp_pos(1))-gomp_pos(2))

' Computation of the upper asymptote (saturation level)
series ahat=exp(log(tot_m2_hp)+gomp_pos(2)*exp(-gomp_pos(1)*t))

' Pseudo out-of-sample forecasting
smpl 2007:6 2008:5
gomp_pos_eq.forecast(f=na) gomp_pos_f

' Graphing
smpl 1997m9 2008m5
graph gomp_pos_f_gr.line tot_m2_hp gomp_pos_f
	gomp_pos_f_gr.draw(line,bottom) 2007m5

smpl 2006m5 2008m5
graph gomp_pos_f_gr2.line tot_m2_hp gomp_pos_f
	gomp_pos_f_gr2.draw(line,bottom) 2007m5


' END OF PROGRAMME

