/* ***********************************************
*  Bootstrapping for Impulse Response Function
*  Confidence Intervales in the Presence of 
*  Parameter Uncertainty
*  --------------------------------------------
*  Written by Matthew Greenwood Sept 2008
*  Email: mjgn@lubs.leeds.ac.uk
*  --------------------------------------------
*  NB - the code is based on Yongcheol Shin's
*  2007 Long--Run Structural Model Programme
*********************************************** */

output file = irfboot.out reset;
Screen on; cls; OUTPUT OFF;


@ ******************* @
@ Initial information @
@ ******************* @

stt0=2; @ Sample adjustment for VAR lag length @
porder=2; @ VAR lag length @
stt=porder;
pyorder=porder;
case=4;
nhorz=40;
rcoin=4;


@ **************** @
@ Read the dataset @
@ **************** @

data = xlsreadm("coin_bs","b2:k165",8,""); @ 1967q1-2007q4 @

@* Original Exogenous I(1) variables, Xo @
ys = data[.,2]; @ Real translog FIML pot output assuming 0 unemployment @
poil = data[.,1]; @ West Texas Intermediate oil price, FRED @

Xo = poil~ys;
dXo = Xo-lag(Xo);
mx = cols(Xo);

@* Original Endogenous I(1) variables, Yo @
gdp = data[.,10]; @ Real GDP @
rus = data[.,6]; @ Federal funds rate, IFS @
inf = data[.,5]; @ Log--approx to GDP deflator inflation, IFS @
winf = data[.,4]; @ Productivity adjusted wage inflation,  BLS and author's calculations @
dsc = data[.,7]; @ Real debt service cost, FoF, IFS @
tobq = data[.,3]; @ Tobin's q @
invest = data[.,9]; @ Gross investment of nonfinancial corporate sector, FoF @
cf = data[.,8]; @ Real cash-flow measured by book value of internal funds, FoF @

Yo = tobq~winf~inf~rus~dsc~cf~invest~gdp;
dYo = Yo-lag(Yo);
my = cols(Yo);

Zo = Xo~Yo;
dZo = Zo-lag(Zo);
mz = cols(Zo);
m = mz;

@* dimensions @
yorder = cols(Yo);  @ number of endo I(1) variables @
xorder = cols(Xo);  @ number of exo I(1) variables @


@ ******************* @
@ Read the parameters @
@ ******************* @

ALPo = xlsreadm("coin_bs","b2:e11",1,"");
    AlpYo = ALPo[3:10,.];

betao = xlsreadm("coin_bs","b2:e12",2,"");
    Bbetao = betao[1:10,.];
    PAIo = -ALPo*Bbetao';
    PaiYo = -AlpYo*Bbetao';

gtmpo = xlsreadm("coin_bs","b2:k11",3,"");
    gamXo = gtmpo[1:2,.];
    gamYo = gtmpo[3:10,.];

lamo = xlsreadm("coin_bs","b2:c9",4,"");

b0o = xlsreadm("coin_bs","b2:b11",5,"");
    b0yo = b0o[3:10,.];

b1o = xlsreadm("coin_bs","b2:b11",6,"");
    b1yo = b1o[3:10,.];

a0o = xlsreadm("coin_bs","b2:k11",9,"");
    ia0o = inv(a0o);
    a0yo = eye(8);
    ia0yo = eye(8);

residso = xlsreadm("coin_bs","b2:k163",7,"");

omegao = xlsreadm("coin_bs","b2:k11",10,""); @ cov matrix @
    omegayo = xlsreadm("coin_bs","d4:k11",10,"");


sample = rows(data);
sample = sample-stt0;
inpt = ones(sample,1); tr1 = seqa(0,1,sample); tr = seqa(1,1,sample); 
inpt = inpt'; tr1 = tr1'; tr = tr';


@ *************************** @
@  Big loop for bootstrapping @
@ *************************** @

"Specify the number of bootstrap samples"; n_iter = con(1,1);

@ for saving the cointest and LR stats later @
EIGB = zeros(my,n_iter);
TRB = zeros(my,n_iter);
MAXB = zeros(my,n_iter);
LRB = zeros(1,n_iter);
BCFLAG = zeros(1,n_iter);

@ for saving IRFs and PPs later @
vGIR1 = zeros((m*(1+nhorz)),n_iter); vGIR2 = zeros((m*(1+nhorz)),n_iter);
vGIR3 = zeros((m*(1+nhorz)),n_iter); vGIR4 = zeros((m*(1+nhorz)),n_iter);
vGIR5 = zeros((m*(1+nhorz)),n_iter); vGIR6 = zeros((m*(1+nhorz)),n_iter);
vGIR7 = zeros((m*(1+nhorz)),n_iter); vGIR8 = zeros((m*(1+nhorz)),n_iter);
vGIR9 = zeros((m*(1+nhorz)),n_iter); vGIR10 = zeros((m*(1+nhorz)),n_iter);

vGIRZ1 = zeros((rcoin*(1+nhorz)),n_iter); vGIRZ2 = zeros((rcoin*(1+nhorz)),n_iter);
vGIRZ3 = zeros((rcoin*(1+nhorz)),n_iter); vGIRZ4 = zeros((rcoin*(1+nhorz)),n_iter);
vGIRZ5 = zeros((rcoin*(1+nhorz)),n_iter); vGIRZ6 = zeros((rcoin*(1+nhorz)),n_iter);
vGIRZ7 = zeros((rcoin*(1+nhorz)),n_iter); vGIRZ8 = zeros((rcoin*(1+nhorz)),n_iter);
vGIRZ9 = zeros((rcoin*(1+nhorz)),n_iter); vGIRZ10 = zeros((rcoin*(1+nhorz)),n_iter);

vOIR1 = zeros((m*(1+nhorz)),n_iter); vOIR2 = zeros((m*(1+nhorz)),n_iter);
vOIR3 = zeros((m*(1+nhorz)),n_iter); vOIR4 = zeros((m*(1+nhorz)),n_iter);
vOIR5 = zeros((m*(1+nhorz)),n_iter); vOIR6 = zeros((m*(1+nhorz)),n_iter);
vOIR7 = zeros((m*(1+nhorz)),n_iter); vOIR8 = zeros((m*(1+nhorz)),n_iter);
vOIR9 = zeros((m*(1+nhorz)),n_iter); vOIR10 = zeros((m*(1+nhorz)),n_iter);

vOIRZ1 = zeros((rcoin*(1+nhorz)),n_iter); vOIRZ2 = zeros((rcoin*(1+nhorz)),n_iter);
vOIRZ3 = zeros((rcoin*(1+nhorz)),n_iter); vOIRZ4 = zeros((rcoin*(1+nhorz)),n_iter);
vOIRZ5 = zeros((rcoin*(1+nhorz)),n_iter); vOIRZ6 = zeros((rcoin*(1+nhorz)),n_iter);
vOIRZ7 = zeros((rcoin*(1+nhorz)),n_iter); vOIRZ8 = zeros((rcoin*(1+nhorz)),n_iter);
vOIRZ9 = zeros((rcoin*(1+nhorz)),n_iter); vOIRZ10 = zeros((rcoin*(1+nhorz)),n_iter);

vHZPP = zeros((rcoin*(1+nhorz)),n_iter);

"Select the bootstrapping method:
1. Parametric   2. Non-parametric"; boot_opt = con(1,1);

nboot = 1;
do until nboot > n_iter;


@ ****************************************************** @
@ Whiten and resample the estimated residuals, eX and eY @
@ ****************************************************** @

if boot_opt eq 1; @ PARAMETRIC METHOD @

    p = chol(omegao);
    p = p';
    invp = inv(p);
    res = rndn(mz,sample);
    ranseed = 1357;
    resp = p*res;
    ehat = resp;

else; @ NON-PARAMETRIC METHOD @

    
    @ Whitening @
    p = chol(omegao);
    p = p';
    invp = inv(p);
    residw = invp*residso';
    residw = residw';
    
    @ Resampling whitened residuals @
    rnum = rndu(sample,1)*(sample);
    ranseed = 1234;
    ehat = zeros(sample,mz);
    ik = 1;
        do until ik > sample;
        ehat[ik,.] = residw[ceil(rnum[ik,.]),.];
        ik = ik+1;
        endo;
    
    @ De-whitening resampled residuals @
    ehat = p*ehat';

endif;


@ ************************************************ @
@ Generate bootstrap samples assuming model is DGP @
@ ************************************************ @

h1 = eye(mx)~zeros(mx,my);
h2 = lamo~eye(my);
h = h1|h2;
ehat = h*ehat;
gamo=ia0o*gtmpo;
zo1 = lag(zo);
dzb = zeros(sample,mz);
zb = zeros(sample,mz);

@ NOTE THAT THIS PROCESS IS BASED ON THE REDUCED FORM MODEL @

ib = 1;
do until ib > sample;
    if ib eq 1;
	dzb[ib,.] = (ia0o*b0o)' + (gamo*dzo[ib+stt0-1,.]')' + (alpo*betao'(zo1[ib+stt0,.]~tr1[.,ib])')' + ehat[.,ib]';
	zb[ib,.] = zo[2,.]+dzb[ib,.];
    elseif ib eq 2;
	dzb[ib,.] = (ia0o*b0o)' + (gamo*dzb[1,.]')' + (alpo*betao'(zb[1,.]~tr1[.,ib])')' + ehat[.,ib]';
	zb[ib,.] = zo[2,.]+sumc(dzb[1:ib,.])';
    else; @ ib > 2 @
	dzb[ib,.] = (ia0o*b0o)' + (gamo*dzb[ib-1,.]')' + (alpo*betao'(zb[ib-1,.]~tr1[.,ib])')' + ehat[.,ib]';
	zb[ib,.] = zo[2,.]+sumc(dzb[1:ib,.])';
    endif;
ib = ib+1;
endo;

/* *********************************
* In Yongcheol's CVAR prog:
* allY = alpY~gamY~lam~inpt
* Yreg = beta'(lag(z)|tr)|dz|dx|inpt
********************************* */

@ Rename bootstrap samples for consistency with CVAR programme @
z=zeros(sample+stt0,mz);
z[1:2,.]=zo[1:2,.];
z[3:sample+stt0,.]=zb;
x=z[.,1:mx];
y=z[.,mx+1:mx+my];


/* **************************************************
*                                                   *
*        START OF YONGCHEOL'S CVAR PROGRAMME        *
*                                                   *
************************************************** */

@ *******************
* DATA TRANSFORMATION
* *******************
* dimensions of data matrices:
* DY: my by T, DX: mx by T, Z1: m by T, ADZLAGS: m(maxp-1) by T
* ************************************************************* @
{ DY, Z1, DX, ADZLAGS } = TRDATA(Y,X,yorder,xorder);

m = xorder + yorder; p = pyorder;
stt1 = 0; @ chk: remove it later @
@* sample adjustment @
DY = DY[.,stt+1:cols(DY)-stt1];
Z1 = Z1[.,stt+1:cols(Z1)-stt1];

if xorder eq 0;
else; DX = DX[.,stt+1:cols(DX)-stt1];
endif;
ActDX = DX; @ actual sample of DX for bootstap, where to put?? @

ADZLAGS = ADZLAGS[.,stt+1:cols(ADZLAGS)-stt1];
if p eq 1;
else; DZLAGS = ADZLAGS[1:m*(p-1),.];
endif;

sample = cols(dy); @ No. of observations @

@ *******************************************
* STEP 2: THE LONG-run COINTEGRATION ANALYSIS
* *******************************************
* We assume that you have already estimated ML estimates of
* the cointegrating vectors which are obtained via e.g. Microfit
* Then, you must provide this information below?? @

/* There are 7 cases depending on deterministic components:
  1. No intercepts and no trends.
  2. Restricted intercepts and no trends.
  3. Unrestricted intercepts and no trends.
  4. Unrestricted intercepts and restricted trends.
  5. Unrestricted intercepts and unrestricted trends.
  6. Case 4 plus restricted intercept shift dummies.
  7. Case 4 plus restricted trend shift dummies. */
case = 4; @ consider both cases 4 and 6. NB - case 6 nests case 4 as a special case @

@ **********************************
* CONSTRUCTION of ECM TERMS, ZS1 and 
* all other regressors, YY
  ********************************** @
inpt = ones(sample,1); tr1 = seqa(0,1,sample); tr = seqa(1,1,sample); 
inpt = inpt'; tr1 = tr1'; tr = tr';

@* Break dummies: provide information on break point @ 
@ if not, the default is the mid point @
brpoint = ceil(sample/2); 
brpoint = 138; @ in the original data: 2001q1 - greenspan gambit @

br = brpoint-stt; @ break point adjusted in sample deleting initial obs @
iduma = zeros(br-1,1);
idumb = ones(sample-br+1,1);
idum = iduma|idumb;
tduma = zeros(br-1,1);
tdumb = seqa(1,1,sample-br+1);
tdum = tduma|tdumb;

{ idum1, adidum, tdum1, adtdum} = TRDATA2(idum,tdum,stt);

idum1 = idum1[.,stt+1:cols(idum1)];
tdum1 = tdum1[.,stt+1:cols(tdum1)];
adidum = adidum[.,stt+1:cols(adidum)];
adtdum = adtdum[.,stt+1:cols(adtdum)];
didum = adidum[1:p,.]; @ including the contemporaneous one @
dtdum = adtdum[1:p,.]; @ including the contemporaneous one @

{ ZS1, YY } = CREG(case,Z1,inpt,TR1,idum1,tdum1,porder,
              DZlags,xorder,DX,TR,didum,dtdum);

@ ********************************** @
@ COINTEGRATION TEST & Johansen beta @
@ ********************************** @
{ COTEST, JBETA } = COINTEST(DY,ZS1,YY,yorder,xorder);

@* exactly identified (eigen) betas for r = 1,...,10: ms by r @
ms = rows(Jbeta);
E10beta = Jbeta[.,ms-9:ms];
E9beta = Jbeta[.,ms-8:ms];
E8beta = Jbeta[.,ms-7:ms];
E7beta = Jbeta[.,ms-6:ms];
E6beta = Jbeta[.,ms-5:ms];
E5beta = Jbeta[.,ms-4:ms];
E4beta = Jbeta[.,ms-3:ms];
E3beta = Jbeta[.,ms-2:ms];
E2beta = Jbeta[.,ms-1:ms];
E1beta = Jbeta[.,ms];

@* error correction terms: r by T @
E1xi = E1beta'ZS1; @ 1 by T @
E2xi = E2beta'ZS1; @ 2 by T @
E3xi = E3beta'ZS1; @ 3 by T @
E4xi = E4beta'ZS1; @ 4 by T @
E5xi = E5beta'ZS1; @ 5 by T @
E6xi = E6beta'ZS1; @ 6 by T @
E7xi = E7beta'ZS1; @ 7 by T @
E8xi = E8beta'ZS1; @ 8 by T @
E9xi = E9beta'ZS1; @ 9 by T @
E10xi = E10beta'ZS1; @ 10 by T @

rcoin = 4; @ # of cointegrating relations @

@* estimation of alpha and omega given Jbeta and max likelihood @
{ Jalpha, Jomega, JMLL } = AOest(E4beta,DY,ZS1,YY,yorder,sample,rcoin);

@ *************************************************************
* Estimation of cointegrating vectors subject to non-homogenous 
* over-identifying restrictions using BS and NR algorithms
* *************************************************************
* Rmatrix*vec(Beta) = bvec
* where dimR = number of restrictions
*       DOF = dimR - rcoin^2: degrees of freedom
* You should specify these restrictions prior to estimation
* which are saved in procedure, RESTR1, RESTR2, ... .
* ********************************************************** @

@ m = 10 (+1, trend) and rcoin = 4
  We have the following full theory restrictions:
  beta = { 0    0    0    0      poil 
           1    0    *    0      yscobb3
           0    0    *    0      tobq
           0    0    0    1      winfe
         -B11   *    0   -1      inf2
          B11  -1    0    *      rus
           0    0    *    0      dscr2
           0    0    *    0      cf2
           *    0   -1    0      invest
          -1    0    *    0      y
           *    0    *    0      t
* A total of 33 restrictions, so 17 over-identifying restrictions
* Later consider co-trending and co-breaking  @
        
kk = 33;
Rmatrix = zeros(kk,ms*rcoin); @ k by msr known matrix: 33 by 44 @
bvec = zeros(kk,1); @ k by 1 known vector @

@ 1st vector: Aggregate Demand: 8 res @
Rmatrix[1,1] = 1;  bvec[1] = 0;
Rmatrix[2,2] = 1;  bvec[2] = 1;
Rmatrix[3,3] = 1;  bvec[3] = 0;
Rmatrix[4,4] = 1;  bvec[4] = 0;
Rmatrix[5,5] = 1;  Rmatrix[5,6] = 1; bvec[5] = 0;    
Rmatrix[6,7] = 1;  bvec[6] = 0; 
Rmatrix[7,8] = 1;  bvec[7] = 0;
Rmatrix[8,10] = 1;  bvec[8] = -1;
    
@ 2nd vector: Taylor rule: 8 res @
Rmatrix[9,ms+1] = 1;  bvec[9] = 0;
Rmatrix[10,ms+2] = 1;  bvec[10] = 0;
Rmatrix[11,ms+3] = 1;  bvec[11] = 0;
Rmatrix[12,ms+4] = 1;  bvec[12] = 0;   
Rmatrix[13,ms+6] = 1;  bvec[13] = -1; 
Rmatrix[14,ms+7] = 1;  bvec[14] = 0;
Rmatrix[15,ms+8] = 1;  bvec[15] = 0;
Rmatrix[16,ms+9] = 1;  bvec[16] = 0;
Rmatrix[17,ms+10] = 1;  bvec[17] = 0;
Rmatrix[18,ms+11] = 1;  bvec[18] = 0;

@ 3rd vector: Investment function: 4 res @
Rmatrix[19,2*ms+1] = 1;  bvec[19] = 0;  
Rmatrix[20,2*ms+4] = 1;  bvec[20] = 0; 
Rmatrix[21,2*ms+5] = 1;  bvec[21] = 0;
Rmatrix[22,2*ms+6] = 1;  bvec[22] = 0;
Rmatrix[23,2*ms+9] = 1;  bvec[23] = -1;

@ 4th vector: Inflation: 6 res @
Rmatrix[24,3*ms+1] = 1;  bvec[24] = 0;
Rmatrix[25,3*ms+2] = 1;  bvec[25] = 0;
Rmatrix[26,3*ms+3] = 1;  bvec[26] = 0;
Rmatrix[27,3*ms+4] = 1;  bvec[27] = 1;
Rmatrix[28,3*ms+5] = 1;  bvec[28] = -1;  
Rmatrix[29,3*ms+7] = 1;  bvec[29] = 0; 
Rmatrix[30,3*ms+8] = 1;  bvec[30] = 0;
Rmatrix[31,3*ms+9] = 1;  bvec[31] = 0;
Rmatrix[32,3*ms+10] = 1;  bvec[32] = 0;
Rmatrix[33,3*ms+11] = 1;  bvec[33] = 0;

dimR = rows(Rmatrix); OVDOF = dimR - rcoin^2;

@* data transformation @
MYY = eye(sample) - YY'invpd(YY*YY')*YY; @ idempotent @
vDY = vec(DY');

@ **************************** 
* Choice of initial estimates:
* **************************** @
{ beta0 } = INICOIN(ms,rcoin,E4beta,Rmatrix,bvec); @ ms by r @
/* beta0 = E4beta; */
@* estimation of alpha and omega given initial beta and MLL @
{ alpha0, omega0, MLL0 } = AOest(beta0,DY,ZS1,YY,yorder,sample,rcoin);

JO4beta = beta0;
JO4xi = JO4beta'ZS1; @ 4 by T @

@ ***********
* BS Algorithm
* *********** @
@ Tolerance level used in convergence criteria @
tol1 = 10^(-4);   @ check if Like(i)-Like(i-1)<tol1 @
tol2 = 10^(-4)*(ms*rcoin-dimR); @ check if |Theta1(i)-Theta1(i-1)|< tol2 @

maxiter = 1000; @ max number of iterations @
ij = 1;
do until ij > maxiter;

@* estimation of beta subject to restrictions @
   { beta1, U11 } = Best(alpha0,Omega0,DY,ZS1,YY,ms,rcoin,Rmatrix,bvec);

@ *********************************
* check the collinearity of beta's
* ********************************* @
   fflag = 0;
   if rank(beta1'ZS1*MYY*ZS1'beta1) < rcoin; fflag = 1;
      "beta's are collinear";
   endif;

@* estimation of alpha and omega given updated beta and MLL @
   { alpha1, omega1, MLL1 } = AOest(beta1,DY,ZS1,YY,yorder,sample,rcoin);

@* check if the convergence occurred @
   convt = abs(vec(beta1)-vec(beta0)); convtr = sumc(convt);
   convlike = MLL1-MLL0;

   cls; screen on; output off; format /rd 12,8;
   "Number of bootstrap iterations completed:";;nboot-1;
   "Number of bootstrap iterations remaining:";; n_iter-nboot+1;
   "Number of Iterations:";; ij;
   "Restricted MLL: ";;MLL1;
   "MLL-MLL(-1):";;convlike;

cflag = 0;
   if    convlike < tol1;
      if    convtr < tol2;  cflag = 1;
            output on; "converged after " ij " iterations"; 
            break;  @ convergence occurred @
      endif;
   endif;

@ update @
   alpha0 = alpha1; Omega0 = Omega1; beta0 = beta1; MLL0 = MLL1;

   ij = ij + 1;"";
endo;

O4beta = beta1;
O4xi = O4beta'ZS1; @ 4 by T @

@ ************************************
* STEP 3: Short-run Dynamic Estimation
* ************************************ @
@ ***********************
* Conditional VECM for dy
* *********************** @
@* r = 0, no cointegration @
E0Yreg = YY;   @ ?? by T @
k0 = yorder*rows(YY); @ number of total parameters @
{ E0ally, E0ey, E0omey, E0LL, E0AIC, E0SBC, E0HQ }
= VECdy(Dy,E0Yreg,sample,yorder,k0);

@* r = 1 @
E1Yreg = E1xi|YY;   @ ?? by T @
k1 = k0 + (ms*1-1^1);
{ E1ally, E1ey, E1Omey, E1LL, E1AIC, E1SBC, E1HQ }
= VECdy(Dy,E1Yreg,sample,yorder,k1);

@* r = 2 @
E2Yreg = E2xi|YY;   @ ?? by T @
k2 = k0 + (ms*2-2^2);
{ E2ally, E2ey, E2Omey, E2LL, E2AIC, E2SBC, E2HQ }
= VECdy(Dy,E2Yreg,sample,yorder,k2);

@* r = 3 @
E3Yreg = E3xi|YY;   @ ?? by T @
k3 = k0 + (ms*3-3^2);
{ E3ally, E3ey, E3Omey, E3LL, E3AIC, E3SBC, E3HQ }
= VECdy(Dy,E3Yreg,sample,yorder,k3);

@* r = 4 @
E4Yreg = E4xi|YY;   @ ?? by T @
k4 = k0 + (ms*4-4^2);
{ E4ally, E4ey, E4Omey, E4LL, E4AIC, E4SBC, E4HQ }
= VECdy(Dy,E4Yreg,sample,yorder,k4);

O4Yreg = O4xi|YY;   @ ?? by T @
k4 = k0 + (ms*4-dimR);
{ O4ally, O4ey, O4Omey, O4LL, O4AIC, O4SBC, O4HQ }
= VECdy(Dy,O4Yreg,sample,yorder,k4);

JO4Yreg = JO4xi|YY;   @ ?? by T @
k4 = k0 + (ms*4-dimR);
{ JO4ally, JO4ey, JO4Omey, JO4LL, JO4AIC, JO4SBC, JO4HQ }
= VECdy(Dy,JO4Yreg,sample,yorder,k4);

@* r = 5 @
E5Yreg = E5xi|YY;   @ ?? by T @
k5 = k0 + (ms*5-5^2);
{ E5ally, E5ey, E5Omey, E5LL, E5AIC, E5SBC, E5HQ }
= VECdy(Dy,E5Yreg,sample,yorder,k5);

@* r = 6 @
E6Yreg = E6xi|YY;   @ ?? by T @
k6 = k0 + (ms*6-6^2);
{ E6ally, E6ey, E6Omey, E6LL, E6AIC, E6SBC, E6HQ }
= VECdy(Dy,E6Yreg,sample,yorder,k6);

@* r = 7 @
E7Yreg = E7xi|YY;   @ ?? by T @
k7 = k0 + (ms*7-7^2);
{ E7ally, E7ey, E7Omey, E7LL, E7AIC, E7SBC, E7HQ }
= VECdy(Dy,E7Yreg,sample,yorder,k7);

@ *******************
  Marginal VAR for dx
  ******************* @
@* Two options:
   A. dx on inpt, dzlags
   B. dx on inpt, significant dzlags @

@* Case A: Unrestricted  @
if     case eq 1; XregA = dzlags; 
elseif case eq 2; XregA = dzlags;
elseif case eq 3; XregA = dzlags|inpt; 
elseif case eq 4; XregA = dzlags|inpt; 
elseif case eq 5; XregA = dzlags|inpt|tr; 
elseif case eq 6; XregA = dzlags|inpt|didum; 
elseif case eq 7; XregA = dzlags|inpt|dtdum; 
endif;
{ allxA, exA, OmexA, LLxA, AICxA, SBCxA, HQxA }
= MARdx(DX,XregA);

@* Case B: Restricted ?? @
if     case eq 1; XregB = dzlags; 
elseif case eq 2; XregB = dzlags;
elseif case eq 3; XregB = dzlags|inpt; 
elseif case eq 4; XregB = dzlags|inpt; 
elseif case eq 5; XregB = dzlags|inpt|tr; 
elseif case eq 6; XregB = dzlags|inpt; 
elseif case eq 7; XregB = dzlags|inpt; 
endif;
{ allxB, exB, OmexB, LLxB, AICxB, SBCxB, HQxB }
= MARdx(DX,XregB);

@ *************************************************
* FULL SYSTEM VAR COMBINING CONDITIONAL VECM for DY 
* and MARGINAL VAR for DX
* *********************************************** @
@ maybe use EX4 for dy and A for dx as benchmark @
@ exact-identified beta with different cointegration rank or over-identified @
@ Abeta: all beta's including restricted deterministics @

e_op = 8;

if e_op eq 0; 
   allY=E0allY; OmegaY=E0OmeY; eY=E0eY; Yreg=E0Yreg;
elseif e_op eq 1; 
   Abeta=E1beta; allY=E1allY; OmegaY=E1OmeY; eY=E1eY; Yreg=E1Yreg;
elseif e_op eq 2; 
   Abeta=E2beta; allY=E2allY; OmegaY=E2OmeY; eY=E2eY; Yreg=E2Yreg;
elseif e_op eq 3; 
   Abeta=E3beta; allY=E3allY; OmegaY=E3OmeY; eY=E3eY; Yreg=E3Yreg;
elseif e_op eq 4; 
   Abeta=E4beta; allY=E4allY; OmegaY=E4OmeY; eY=E4eY; Yreg=E4Yreg; 
elseif e_op eq 5; 
   Abeta=E5beta; allY=E5allY; OmegaY=E5OmeY; eY=E5eY; Yreg=E5Yreg;
elseif e_op eq 6; 
   Abeta=E6beta; allY=E6allY; OmegaY=E6OmeY; eY=E6eY; Yreg=E6Yreg;
elseif e_op eq 7; 
   Abeta=E7beta; allY=E7allY; OmegaY=E7OmeY; eY=E7eY; Yreg=E7Yreg;
elseif e_op eq 8; 
   Abeta=O4beta; allY=O4allY; OmegaY=O4OmeY; eY=O4eY; Yreg=O4Yreg;
elseif e_op eq 9; 
   Abeta=JO4beta; allY=JO4allY; OmegaY=JO4OmeY; eY=JO4eY; Yreg=JO4Yreg;
endif; 

rcoin = cols(Abeta); @ rank of coin vector @

@ ******************************************
* Conditional VECM estimation resutls for DY
* ******************************************
* ESTRES: coefficients~standard err~t-ratio
* DIAGRES1: RSQ|RBARSQ|SIGE|LL|AIC|SBC|HQ: 7x1 
* DIAGRES2: chiSC~chiFF~chiNO~chiHE 
*            pSC  ~pFF  ~pNO  ~pHE          
* ****************************************** @ 
VRES = eY;          @ Residuals @
VYHAT = ALLY*Yreg;  @ dyhat @
VREG = Yreg';       @ regressors: T by nreg @
nreg = cols(VREG);

ecmi = 1;
do until ecmi > yorder;

    VDEP = DY[ecmi,.]';       @ dependent variable: T by 1 @
    RESID = VRES[ecmi,.]';    @ residual: T by 1 @
    ECMCOEFF = ALLY[ecmi,.]'; @ coefficients @
    YHAT = VYHAT[ecmi,.]';    @ fitted value: T by 1 @

    { ESTRES, DIAGRES1, DIAGRES2 } 
    = VECM(ECMCOEFF,VDEP,VREG,RESID,YHAT,nreg);

cls; output on;
"ECM Estimation Results for ";; ecmi;" Equation";
"**********************************************";"";
    PRECM(ESTRES,DIAGRES1,DIAGRES2);

    ecmi = ecmi + 1;
endo;

x_op = 1;
if x_op eq 1; 
   allX = allXA; Omegax = OmeXA; eX = eXA; Xreg=XregA;
elseif x_op eq 2; @ ?? @ 
   allX = allXB; Omegax = OmeXB; eX = eXB; Xreg=XregB;
endif;

@ **************************************
* Marginal VAR estimation results for DX 
* **************************************
* ESTRES: coefficients~standard err~t-ratio 
* DIAGRES1: RSQ|RBARSQ|SIGE|LL|AIC|SBC|HQ: 7x1 
* DIAGRES2: chiSC~chiFF~chiNO~chiHE 
             pSC  ~pFF  ~pNO  ~pHE  
* ****************************************** @ 
VRES = eX;          @ Residuals: n @
VXHAT = ALLX*Xreg;  @ dyhat: n @
VREG = Xreg';       @ common regressors:  @
nreg = cols(VREG);

ej = 1;
do until ej > xorder;

   VARCO = ALLX[ej,.]';  @ coefficients @
   VDEP = DX[ej,.]';     @ dependent variable @
   RESID = VRES[ej,.]';  @ residual @
   XHAT = VXHAT[ej,.]';  @ fitted value @

   { ESTRES, DIAGRES1, DIAGRES2 } 
   = VECM(VARCO,VDEP,VREG,RESID,XHAT,nreg);

cls; output on;
"Estimation Results for ";; ej;" Equation";
"***************************************************";"";
    PRECM(ESTRES,DIAGRES1,DIAGRES2);

   ej = ej + 1;
endo;

cls; output off;

Bbeta = Abeta[1:m,.]; 
@ m by r: deleting the last row(s) when restricted 
deterministics are present in cointegrating vectors @

@* Decomposition of parameters: assuming case 6 for now?? @

{AlpY,GamY,Lam,b0Y,b1Y,d0Y,d1Y,ddY,dtY} 
= decompY(ally,rcoin,pyorder,yorder,xorder,case,Abeta);

pxorder = pyorder;
{GamX,b0X,ddX,dtX} = decompX(allx,pxorder,yorder,xorder,case);

@* By construction we set the following parameters to zero @
AlpX = zeros(xorder,rcoin);
b1X = zeros(xorder,1);
d0X = zeros(xorder,1);
d1X = zeros(xorder,1);

@ NB: we are now using the same lag order @
if pxorder > pyorder;
   if pyorder eq 1; GamY = zeros(yorder,m*(pxorder-pyorder));
   else;       GAMY = GamY~zeros(yorder,m*(pxorder-pyorder));
   endif;
endif;

@ ************************* @
@ Contemporaneous MATRIX A0 @
@ ************************* @
A1 = eye(xorder)~zeros(xorder,yorder);
A2 = -Lam~eye(yorder);
A0 = A1|A2; IA0 = inv(A0); @ inverse @

@ ********************************************** @
@ CONSTRUCTION OF PARAMETERS for THE FULL SYSTEM @
@ ********************************************** @
ALP = AlpX|AlpY;
PAI = -ALP*Bbeta';  @ m by m @
gtmp = GamX|GamY; GAM = IA0*gtmp;
b0tmp = b0x|b0y;  b0 = IA0*b0tmp;
b1tmp = b1x|b1y;  b1 = IA0*b1tmp;
d0tmp = d0x|d0y;  d0 = IA0*d0tmp;
d1tmp = d1x|d1y;  d1 = IA0*d1tmp;
ddtmp = ddX|ddY;  dd = IA0*ddtmp;
dttmp = dtX|dtY;  dt = IA0*dttmp;

@ ****************** @
@ COVARIANCE MATRIX:
* OMEGA (ex and ey uncorrelated by construction) and
* COMEGA = IA0*Omega*IA0', which is correlated
* ************************************************** @
O1 = Omegax~zeros(xorder,yorder);
O2 = zeros(yorder,xorder)~Omegay;
Omega = O1|O2;
COmega = IA0*Omega*IA0'; @ reduced form COV @

@ ********************* @
@ FULL RESIDUAL VECTORS @
@ ********************* @
ez = ex|ey;

@ ******************************************************* @
@ Computation of A11 and W11 using Choleski decomposition:
* *******************************************************
* This is necessary for an analysis of structurally identified
* shocks such as monetary policy shock. Now assume the first 6 
* variables are recursive in the order of poil~ys~tobq~winf~
* inf~rus
* **************************************************** @
n_or = 6; @ No. of first block recursive variables @
Com11 = Comega[1:n_or,1:n_or]; @ z1 = (poild~ys~tobq~winf~inf~rus) @
P11 = chol(Com11); D11 = eye(n_or);
jj = 1;
do until jj > n_or;
   D11[jj,jj] = P11[jj,jj];
   jj = jj+1;
endo;
IA11 = P11'inv(D11);
W11 = D11.^2;  @ n_or by n_or variance matrix for structural errors @
A11 = inv(IA11);

@ *************************************************
* Computing DYNAMIC VMA PARAMETERS USING BIG VAR(1)
* ************************************************* @

nhorz = 40;
{ FAI, FF, BN } = DYPARA(pai,gam,porder,m,nhorz);

@ ********************************
* STEP 4: FURTHER DYNAMIC ANALYSIS
* ******************************** @


if cflag eq 1;
@ ***********************************
* 1. STABILITY OF THE FULL VEC SYSTEM
* *********************************** @
BEGSTAB:
@******@

ntrunc = 20;
{ va, modva } = CHKSTAB(m,rcoin,porder,alp,Bbeta,gam,ntrunc);
endif;


if cflag eq 1;
@ ************************
*  2. PERSISTENCE PROFILES
* ************************ @
BEGPER:
@******@

hz = zeros(nhorz,rcoin);
ir = 1;
do until ir > rcoin;
   BETTA = BBETA[.,ir];
   { hz[.,ir] } = persis(m,nhorz,BETTA,COMEGA,BN);
   ir = ir + 1;
endo;
ini = ones(1,rcoin); hz = ini|hz;

       vHZPP[.,nboot] = vec(hz);

else;

       miss2 = ones((rcoin*(nhorz+1)),1);
       miss2 = 9999.99*miss2;

       vHZPP[.,nboot] = miss2;
       vHZPP[.,nboot] = miss(miss2,9999.99);

endif;


if cflag eq 1;
@ ****************************************
* 3. Generalised Impulse Response Analysis
* **************************************** @
BEGIMP:
@******@

GIR = zeros((nhorz+1)*m,m); @ with respect to m shocks @
GIRZ = zeros((nhorz+1)*rcoin,m);
OIR = zeros((nhorz+1)*m,m);
OIRZ = zeros((nhorz+1)*rcoin,m);

ij = 1;
do until ij > m;
   sh_op = ij;

   if sh_op eq 1;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR1[.,nboot] = vec(tmpGIR); vGIRZ1[.,nboot] = vec(tmpGIRZ);
       vOIR1[.,nboot] = vec(tmpOIR); vOIRZ1[.,nboot] = vec(tmpOIRZ);
    
   elseif sh_op eq 2;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR2[.,nboot] = vec(tmpGIR); vGIRZ2[.,nboot] = vec(tmpGIRZ);
       vOIR2[.,nboot] = vec(tmpOIR); vOIRZ2[.,nboot] = vec(tmpOIRZ);

   elseif sh_op eq 3;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR3[.,nboot] = vec(tmpGIR); vGIRZ3[.,nboot] = vec(tmpGIRZ);
       vOIR3[.,nboot] = vec(tmpOIR); vOIRZ3[.,nboot] = vec(tmpOIRZ);

   elseif sh_op eq 4;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR4[.,nboot] = vec(tmpGIR); vGIRZ4[.,nboot] = vec(tmpGIRZ);
       vOIR4[.,nboot] = vec(tmpOIR); vOIRZ4[.,nboot] = vec(tmpOIRZ);

   elseif sh_op eq 5;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR5[.,nboot] = vec(tmpGIR); vGIRZ5[.,nboot] = vec(tmpGIRZ);
       vOIR5[.,nboot] = vec(tmpOIR); vOIRZ5[.,nboot] = vec(tmpOIRZ);

   elseif sh_op eq 6;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR6[.,nboot] = vec(tmpGIR); vGIRZ6[.,nboot] = vec(tmpGIRZ);
       vOIR6[.,nboot] = vec(tmpOIR); vOIRZ6[.,nboot] = vec(tmpOIRZ);

   elseif sh_op eq 7;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR7[.,nboot] = vec(tmpGIR); vGIRZ7[.,nboot] = vec(tmpGIRZ);
       vOIR7[.,nboot] = vec(tmpOIR); vOIRZ7[.,nboot] = vec(tmpOIRZ);

   elseif sh_op eq 8;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR8[.,nboot] = vec(tmpGIR); vGIRZ8[.,nboot] = vec(tmpGIRZ);
       vOIR8[.,nboot] = vec(tmpOIR); vOIRZ8[.,nboot] = vec(tmpOIRZ);

   elseif sh_op eq 9;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR9[.,nboot] = vec(tmpGIR); vGIRZ9[.,nboot] = vec(tmpGIRZ);
       vOIR9[.,nboot] = vec(tmpOIR); vOIRZ9[.,nboot] = vec(tmpOIRZ);

   elseif sh_op eq 10;
       { tmpGIR, tmpGIRZ, tmpOIR, tmpOIRZ }
       = Impulse(sh_op,m,nhorz,rcoin,BBETA,OMEGA,BN,IA0);
       vGIR10[.,nboot] = vec(tmpGIR); vGIRZ10[.,nboot] = vec(tmpGIRZ);
       vOIR10[.,nboot] = vec(tmpOIR); vOIRZ10[.,nboot] = vec(tmpOIRZ);
    endif;
   ij = ij + 1;
endo;

else;
       miss1 = ones((m*(nhorz+1)),1);
       miss1 = 9999.99*miss1;
       miss2 = ones((rcoin*(nhorz+1)),1);
       miss2 = 9999.99*miss2;

       vGIR1[.,nboot] = miss(miss1,9999.99); vGIRZ1[.,nboot] = miss(miss2,9999.99);
       vOIR1[.,nboot] = miss(miss1,9999.99); vOIRZ1[.,nboot] = miss(miss2,9999.99);

       vGIR2[.,nboot] = miss(miss1,9999.99); vGIRZ2[.,nboot] = miss(miss2,9999.99);
       vOIR2[.,nboot] = miss(miss1,9999.99); vOIRZ2[.,nboot] = miss(miss2,9999.99);

       vGIR3[.,nboot] = miss(miss1,9999.99); vGIRZ3[.,nboot] = miss(miss2,9999.99);
       vOIR3[.,nboot] = miss(miss1,9999.99); vOIRZ3[.,nboot] = miss(miss2,9999.99);

       vGIR4[.,nboot] = miss(miss1,9999.99); vGIRZ4[.,nboot] = miss(miss2,9999.99);
       vOIR4[.,nboot] = miss(miss1,9999.99); vOIRZ4[.,nboot] = miss(miss2,9999.99);

       vGIR5[.,nboot] = miss(miss1,9999.99); vGIRZ5[.,nboot] = miss(miss2,9999.99);
       vOIR5[.,nboot] = miss(miss1,9999.99); vOIRZ5[.,nboot] = miss(miss2,9999.99);

       vGIR6[.,nboot] = miss(miss1,9999.99); vGIRZ6[.,nboot] = miss(miss2,9999.99);
       vOIR6[.,nboot] = miss(miss1,9999.99); vOIRZ6[.,nboot] = miss(miss2,9999.99);

       vGIR7[.,nboot] = miss(miss1,9999.99); vGIRZ7[.,nboot] = miss(miss2,9999.99);
       vOIR7[.,nboot] = miss(miss1,9999.99); vOIRZ7[.,nboot] = miss(miss2,9999.99);

       vGIR8[.,nboot] = miss(miss1,9999.99); vGIRZ8[.,nboot] = miss(miss2,9999.99);
       vOIR8[.,nboot] = miss(miss1,9999.99); vOIRZ8[.,nboot] = miss(miss2,9999.99);

       vGIR9[.,nboot] = miss(miss1,9999.99); vGIRZ9[.,nboot] = miss(miss2,9999.99);
       vOIR9[.,nboot] = miss(miss1,9999.99); vOIRZ9[.,nboot] = miss(miss2,9999.99);

       vGIR10[.,nboot] = miss(miss1,9999.99); vGIRZ10[.,nboot] = miss(miss2,9999.99);
       vOIR10[.,nboot] = miss(miss1,9999.99); vOIRZ10[.,nboot] = miss(miss2,9999.99);

endif;


@ ***************************************
* 4. Structural Impulse Response Analysis
* *************************************** @
BEGSIR:
@******@

@ SIR: (h+1) by m, IRF of m variables
* SIRZ: (h+1) by rcoin, IRF of r cointegrating relationships
* where h is the forecast horizon.
* There are seven set of results with respect to shocks to
* O: oil price, YS: potential output, TQ: Tobin's Q, WI: wage
* inflation, IN: inflation, R: interest rate, I: intercept shift.
* ********************************************************** @

{ OSIR, OSIRZ, YSSIR, YSSIRZ, TQSIR, TQSIRZ, WISIR, WISIRZ, INSIR, INSIRZ, RSIR, RSIRZ, ISIR, ISIRZ }
= STIRF(m,Nhorz,rcoin,BBETA,COMEGA,BN,IA11,W11);


@ **************** @
@ Closing big loop @
@ **************** @

BCFLAG[.,nboot] = cflag;
nboot = nboot+1;
endo;


/* **************************************************************
*                                                               *
*   COMPUTATION OF BOOTSTRAP INTERVALS for GIRS, OIRS and SIRS  *
*                                                               *
************************************************************** */

"Total number of converged estimates:"; sumr(bcflag);
"Convergence rate (%):"; 100*sumr(bcflag)/(n_iter);
"Results exported to Excel sheets GIRBOOT.XLS, OIRBOOT.XLS AND PPBOOT.XLS";

@ Remove missing (non-convergent) values @

vGIR1 = packr(VGIR1')'; vGIR2 = packr(VGIR2')';
vGIR3 = packr(VGIR3')'; vGIR4 = packr(VGIR4')';
vGIR5 = packr(VGIR5')'; vGIR6 = packr(VGIR6')';
vGIR7 = packr(VGIR7')'; vGIR8 = packr(VGIR8')';
vGIR9 = packr(VGIR9')'; vGIR10 = packr(VGIR10')';

vGIRZ1 = packr(VGIRZ1')'; vGIRZ2 = packr(VGIRZ2')';
vGIRZ3 = packr(VGIRZ3')'; vGIRZ4 = packr(VGIRZ4')';
vGIRZ5 = packr(VGIRZ5')'; vGIRZ6 = packr(VGIRZ6')';
vGIRZ7 = packr(VGIRZ7')'; vGIRZ8 = packr(VGIRZ8')';
vGIRZ9 = packr(VGIRZ9')'; vGIRZ10 = packr(VGIRZ10')';

vOIR1 = packr(VOIR1')'; vOIR2 = packr(VOIR2')';
vOIR3 = packr(VOIR3')'; vOIR4 = packr(VOIR4')';
vOIR5 = packr(VOIR5')'; vOIR6 = packr(VOIR6')';
vOIR7 = packr(VOIR7')'; vOIR8 = packr(VOIR8')';
vOIR9 = packr(VOIR9')'; vOIR10 = packr(VOIR10')';

vOIRZ1 = packr(VOIRZ1')'; vOIRZ2 = packr(VOIRZ2')';
vOIRZ3 = packr(VOIRZ3')'; vOIRZ4 = packr(VOIRZ4')';
vOIRZ5 = packr(VOIRZ5')'; vOIRZ6 = packr(VOIRZ6')';
vOIRZ7 = packr(VOIRZ7')'; vOIRZ8 = packr(VOIRZ8')';
vOIRZ9 = packr(VOIRZ9')'; vOIRZ10 = packr(VOIRZ10')';

vHZPP = packr(VHZPP')';

@ Compute empirical distributions for shocks to mz variables @

mvhzpp = zeros(((nhorz+1)*rcoin),1); vvhzpp = zeros(((nhorz+1)*rcoin),1);
sdvhzpp = zeros(((nhorz+1)*rcoin),1);

jvv = 1;
do until jvv > ((nhorz+1)*rcoin);
    mvHZPP[jvv] = sumr(vHZPP[jvv,.])/cols(vHZPP);
    vvHZPP[jvv] = sumr((vHZPP[jvv,.]-mvHZPP[jvv])^2)/(cols(vHZPP)-1);
    sdvHZPP[jvv] = vvHZPP[jvv].^0.5;
jvv = jvv+1;
endo;

mHZPP = TRMAT(mvHZPP,(nhorz+1),rcoin); sdHZPP = TRMAT(sdvHZPP,(nhorz+1),rcoin);

{ mGIR1, mdGIR1, sdGIR1, l90GIR1, u90GIR1, l95GIR1, u95GIR1 }
= EMPDIST(vGIR1,(nhorz+1),mz,cols(vGIR1));
    { mGIR2, mdGIR2, sdGIR2, l90GIR2, u90GIR2, l95GIR2, u95GIR2 }
    = EMPDIST(vGIR2,(nhorz+1),mz,cols(vGIR2));
{ mGIR3, mdGIR3, sdGIR3, l90GIR3, u90GIR3, l95GIR3, u95GIR3 }
= EMPDIST(vGIR3,(nhorz+1),mz,cols(vGIR3));
    { mGIR4, mdGIR4, sdGIR4, l90GIR4, u90GIR4, l95GIR4, u95GIR4 }
    = EMPDIST(vGIR4,(nhorz+1),mz,cols(vGIR4));
{ mGIR5, mdGIR5, sdGIR5, l90GIR5, u90GIR5, l95GIR5, u95GIR5 }
= EMPDIST(vGIR5,(nhorz+1),mz,cols(vGIR5));
    { mGIR6, mdGIR6, sdGIR6, l90GIR6, u90GIR6, l95GIR6, u95GIR6 }
    = EMPDIST(vGIR6,(nhorz+1),mz,cols(vGIR6));
{ mGIR7, mdGIR7, sdGIR7, l90GIR7, u90GIR7, l95GIR7, u95GIR7 }
= EMPDIST(vGIR7,(nhorz+1),mz,cols(vGIR7));
    { mGIR8, mdGIR8, sdGIR8, l90GIR8, u90GIR8, l95GIR8, u95GIR8 }
    = EMPDIST(vGIR8,(nhorz+1),mz,cols(vGIR8));
{ mGIR9, mdGIR9, sdGIR9, l90GIR9, u90GIR9, l95GIR9, u95GIR9 }
= EMPDIST(vGIR9,(nhorz+1),mz,cols(vGIR9));
    { mGIR10, mdGIR10, sdGIR10, l90GIR10, u90GIR10, l95GIR10, u95GIR10 }
    = EMPDIST(vGIR10,(nhorz+1),mz,cols(vGIR10));

{ mOIR1, mdOIR1, sdOIR1, l90OIR1, u90OIR1, l95OIR1, u95OIR1 }
= EMPDIST(vOIR1,(nhorz+1),mz,cols(vOIR1));
    { mOIR2, mdOIR2, sdOIR2, l90OIR2, u90OIR2, l95OIR2, u95OIR2 }
    = EMPDIST(vOIR2,(nhorz+1),mz,cols(vOIR2));
{ mOIR3, mdOIR3, sdOIR3, l90OIR3, u90OIR3, l95OIR3, u95OIR3 }
= EMPDIST(vOIR3,(nhorz+1),mz,cols(vOIR3));
    { mOIR4, mdOIR4, sdOIR4, l90OIR4, u90OIR4, l95OIR4, u95OIR4 }
    = EMPDIST(vOIR4,(nhorz+1),mz,cols(vOIR4));
{ mOIR5, mdOIR5, sdOIR5, l90OIR5, u90OIR5, l95OIR5, u95OIR5 }
= EMPDIST(vOIR5,(nhorz+1),mz,cols(vOIR5));
    { mOIR6, mdOIR6, sdOIR6, l90OIR6, u90OIR6, l95OIR6, u95OIR6 }
    = EMPDIST(vOIR6,(nhorz+1),mz,cols(vOIR6));
{ mOIR7, mdOIR7, sdOIR7, l90OIR7, u90OIR7, l95OIR7, u95OIR7 }
= EMPDIST(vOIR7,(nhorz+1),mz,cols(vOIR7));
    { mOIR8, mdOIR8, sdOIR8, l90OIR8, u90OIR8, l95OIR8, u95OIR8 }
    = EMPDIST(vOIR8,(nhorz+1),mz,cols(vOIR8));
{ mOIR9, mdOIR9, sdOIR9, l90OIR9, u90OIR9, l95OIR9, u95OIR9 }
= EMPDIST(vOIR9,(nhorz+1),mz,cols(vOIR9));
    { mOIR10, mdOIR10, sdOIR10, l90OIR10, u90OIR10, l95OIR10, u95OIR10 }
    = EMPDIST(vOIR10,(nhorz+1),mz,cols(vOIR10));

{ mGIRZ1, mdGIRZ1, sdGIRZ1, l90GIRZ1, u90GIRZ1, l95GIRZ1, u95GIRZ1 }
= EMPDIST(vGIRZ1,(nhorz+1),rcoin,cols(vGIRZ1));
    { mGIRZ2, mdGIRZ2, sdGIRZ2, l90GIRZ2, u90GIRZ2, l95GIRZ2, u95GIRZ2 }
    = EMPDIST(vGIRZ2,(nhorz+1),rcoin,cols(vGIRZ2));
{ mGIRZ3, mdGIRZ3, sdGIRZ3, l90GIRZ3, u90GIRZ3, l95GIRZ3, u95GIRZ3 }
= EMPDIST(vGIRZ3,(nhorz+1),rcoin,cols(vGIRZ3));
    { mGIRZ4, mdGIRZ4, sdGIRZ4, l90GIRZ4, u90GIRZ4, l95GIRZ4, u95GIRZ4 }
    = EMPDIST(vGIRZ4,(nhorz+1),rcoin,cols(vGIRZ4));
{ mGIRZ5, mdGIRZ5, sdGIRZ5, l90GIRZ5, u90GIRZ5, l95GIRZ5, u95GIRZ5 }
= EMPDIST(vGIRZ5,(nhorz+1),rcoin,cols(vGIRZ5));
    { mGIRZ6, mdGIRZ6, sdGIRZ6, l90GIRZ6, u90GIRZ6, l95GIRZ6, u95GIRZ6 }
    = EMPDIST(vGIRZ6,(nhorz+1),rcoin,cols(vGIRZ6));
{ mGIRZ7, mdGIRZ7, sdGIRZ7, l90GIRZ7, u90GIRZ7, l95GIRZ7, u95GIRZ7 }
= EMPDIST(vGIRZ7,(nhorz+1),rcoin,cols(vGIRZ7));
    { mGIRZ8, mdGIRZ8, sdGIRZ8, l90GIRZ8, u90GIRZ8, l95GIRZ8, u95GIRZ8 }
    = EMPDIST(vGIRZ8,(nhorz+1),rcoin,cols(vGIRZ8));
{ mGIRZ9, mdGIRZ9, sdGIRZ9, l90GIRZ9, u90GIRZ9, l95GIRZ9, u95GIRZ9 }
= EMPDIST(vGIRZ9,(nhorz+1),rcoin,cols(vGIRZ9));
    { mGIRZ10, mdGIRZ10, sdGIRZ10, l90GIRZ10, u90GIRZ10, l95GIRZ10, u95GIRZ10 }
    = EMPDIST(vGIRZ10,(nhorz+1),rcoin,cols(vGIRZ10));

{ mOIRZ1, mdOIRZ1, sdOIRZ1, l90OIRZ1, u90OIRZ1, l95OIRZ1, u95OIRZ1 }
= EMPDIST(vOIRZ1,(nhorz+1),rcoin,cols(vOIRZ1));
    { mOIRZ2, mdOIRZ2, sdOIRZ2, l90OIRZ2, u90OIRZ2, l95OIRZ2, u95OIRZ2 }
    = EMPDIST(vOIRZ2,(nhorz+1),rcoin,cols(vOIRZ2));
{ mOIRZ3, mdOIRZ3, sdOIRZ3, l90OIRZ3, u90OIRZ3, l95OIRZ3, u95OIRZ3 }
= EMPDIST(vOIRZ3,(nhorz+1),rcoin,cols(vOIRZ3));
    { mOIRZ4, mdOIRZ4, sdOIRZ4, l90OIRZ4, u90OIRZ4, l95OIRZ4, u95OIRZ4 }
    = EMPDIST(vOIRZ4,(nhorz+1),rcoin,cols(vOIRZ4));
{ mOIRZ5, mdOIRZ5, sdOIRZ5, l90OIRZ5, u90OIRZ5, l95OIRZ5, u95OIRZ5 }
= EMPDIST(vOIRZ5,(nhorz+1),rcoin,cols(vOIRZ5));
    { mOIRZ6, mdOIRZ6, sdOIRZ6, l90OIRZ6, u90OIRZ6, l95OIRZ6, u95OIRZ6 }
    = EMPDIST(vOIRZ6,(nhorz+1),rcoin,cols(vOIRZ6));
{ mOIRZ7, mdOIRZ7, sdOIRZ7, l90OIRZ7, u90OIRZ7, l95OIRZ7, u95OIRZ7 }
= EMPDIST(vOIRZ7,(nhorz+1),rcoin,cols(vOIRZ7));
    { mOIRZ8, mdOIRZ8, sdOIRZ8, l90OIRZ8, u90OIRZ8, l95OIRZ8, u95OIRZ8 }
    = EMPDIST(vOIRZ8,(nhorz+1),rcoin,cols(vOIRZ8));
{ mOIRZ9, mdOIRZ9, sdOIRZ9, l90OIRZ9, u90OIRZ9, l95OIRZ9, u95OIRZ9 }
= EMPDIST(vOIRZ9,(nhorz+1),rcoin,cols(vOIRZ9));
    { mOIRZ10, mdOIRZ10, sdOIRZ10, l90OIRZ10, u90OIRZ10, l95OIRZ10, u95OIRZ10 }
    = EMPDIST(vOIRZ10,(nhorz+1),rcoin,cols(vOIRZ10));

{ mHZPP, mHZPP, sdHZPP, l90HZPP, u90HZPP, l95HZPP, u95HZPP }
= EMPDIST(vHZPP,(nhorz+1),rcoin,cols(vHZPP));


@ *************************** @
@ Export the results to Excel @
@ *************************** @

@ Export GIR and OIR @
GIRout1 = xlswritem(mGIR1~mGIRZ1~sdGIR1~sdGIRZ1~l90GIR1~l90GIRZ1~u90GIR1~u90GIRZ1~l95GIR1~l95GIRZ1~u95GIR1~u95GIRZ1,"GIRBOOT","b3",1,"");
GIRout2 = xlswritem(mGIR2~mGIRZ2~sdGIR2~sdGIRZ2~l90GIR2~l90GIRZ2~u90GIR2~u90GIRZ2~l95GIR2~l95GIRZ2~u95GIR2~u95GIRZ2,"GIRBOOT","b3",2,"");
GIRout3 = xlswritem(mGIR3~mGIRZ3~sdGIR3~sdGIRZ3~l90GIR3~l90GIRZ3~u90GIR3~u90GIRZ3~l95GIR3~l95GIRZ3~u95GIR3~u95GIRZ3,"GIRBOOT","b3",3,"");
GIRout4 = xlswritem(mGIR4~mGIRZ4~sdGIR4~sdGIRZ4~l90GIR4~l90GIRZ4~u90GIR4~u90GIRZ4~l95GIR4~l95GIRZ4~u95GIR4~u95GIRZ4,"GIRBOOT","b3",4,"");
GIRout5 = xlswritem(mGIR5~mGIRZ5~sdGIR5~sdGIRZ5~l90GIR5~l90GIRZ5~u90GIR5~u90GIRZ5~l95GIR5~l95GIRZ5~u95GIR5~u95GIRZ5,"GIRBOOT","b3",5,"");
GIRout6 = xlswritem(mGIR6~mGIRZ6~sdGIR6~sdGIRZ6~l90GIR6~l90GIRZ6~u90GIR6~u90GIRZ6~l95GIR6~l95GIRZ6~u95GIR6~u95GIRZ6,"GIRBOOT","b3",6,"");
GIRout7 = xlswritem(mGIR7~mGIRZ7~sdGIR7~sdGIRZ7~l90GIR7~l90GIRZ7~u90GIR7~u90GIRZ7~l95GIR7~l95GIRZ7~u95GIR7~u95GIRZ7,"GIRBOOT","b3",7,"");
GIRout8 = xlswritem(mGIR8~mGIRZ8~sdGIR8~sdGIRZ8~l90GIR8~l90GIRZ8~u90GIR8~u90GIRZ8~l95GIR8~l95GIRZ8~u95GIR8~u95GIRZ8,"GIRBOOT","b3",8,"");
GIRout9 = xlswritem(mGIR9~mGIRZ9~sdGIR9~sdGIRZ9~l90GIR9~l90GIRZ9~u90GIR9~u90GIRZ9~l95GIR9~l95GIRZ9~u95GIR9~u95GIRZ9,"GIRBOOT","b3",9,"");
GIRout10 = xlswritem(mGIR10~mGIRZ10~sdGIR10~sdGIRZ10~l90GIR10~l90GIRZ10~u90GIR10~u90GIRZ10~l95GIR10~l95GIRZ10~u95GIR10~u95GIRZ10,"GIRBOOT","b3",10,"");

OIRout1 = xlswritem(mOIR1~mOIRZ1~sdOIR1~sdOIRZ1~l90OIR1~l90OIRZ1~u90OIR1~u90OIRZ1~l95OIR1~l95OIRZ1~u95OIR1~u95OIRZ1,"OIRBOOT","b3",1,"");
OIRout2 = xlswritem(mOIR2~mOIRZ2~sdOIR2~sdOIRZ2~l90OIR2~l90OIRZ2~u90OIR2~u90OIRZ2~l95OIR2~l95OIRZ2~u95OIR2~u95OIRZ2,"OIRBOOT","b3",2,"");
OIRout3 = xlswritem(mOIR3~mOIRZ3~sdOIR3~sdOIRZ3~l90OIR3~l90OIRZ3~u90OIR3~u90OIRZ3~l95OIR3~l95OIRZ3~u95OIR3~u95OIRZ3,"OIRBOOT","b3",3,"");
OIRout4 = xlswritem(mOIR4~mOIRZ4~sdOIR4~sdOIRZ4~l90OIR4~l90OIRZ4~u90OIR4~u90OIRZ4~l95OIR4~l95OIRZ4~u95OIR4~u95OIRZ4,"OIRBOOT","b3",4,"");
OIRout5 = xlswritem(mOIR5~mOIRZ5~sdOIR5~sdOIRZ5~l90OIR5~l90OIRZ5~u90OIR5~u90OIRZ5~l95OIR5~l95OIRZ5~u95OIR5~u95OIRZ5,"OIRBOOT","b3",5,"");
OIRout6 = xlswritem(mOIR6~mOIRZ6~sdOIR6~sdOIRZ6~l90OIR6~l90OIRZ6~u90OIR6~u90OIRZ6~l95OIR6~l95OIRZ6~u95OIR6~u95OIRZ6,"OIRBOOT","b3",6,"");
OIRout7 = xlswritem(mOIR7~mOIRZ7~sdOIR7~sdOIRZ7~l90OIR7~l90OIRZ7~u90OIR7~u90OIRZ7~l95OIR7~l95OIRZ7~u95OIR7~u95OIRZ7,"OIRBOOT","b3",7,"");
OIRout8 = xlswritem(mOIR8~mOIRZ8~sdOIR8~sdOIRZ8~l90OIR8~l90OIRZ8~u90OIR8~u90OIRZ8~l95OIR8~l95OIRZ8~u95OIR8~u95OIRZ8,"OIRBOOT","b3",8,"");
OIRout9 = xlswritem(mOIR9~mOIRZ9~sdOIR9~sdOIRZ9~l90OIR9~l90OIRZ9~u90OIR9~u90OIRZ9~l95OIR9~l95OIRZ9~u95OIR9~u95OIRZ9,"OIRBOOT","b3",9,"");
OIRout10 = xlswritem(mOIR10~mOIRZ10~sdOIR10~sdOIRZ10~l90OIR10~l90OIRZ10~u90OIR10~u90OIRZ10~l95OIR10~l95OIRZ10~u95OIR10~u95OIRZ10,"OIRBOOT","b3",10,"");

@ Headers for GIR and OIR @
    ff1 = {"MEAN" "" "" "" "" "" "" "" "" "" "" "" "" "" "SD" "" "" "" "" "" "" "" "" "" "" ""
     "" "" "l90" "" "" "" "" "" "" "" "" "" "" "" "" "" "u90" "" "" "" "" "" "" "" "" "" "" ""
     "" "" "l95" "" "" "" "" "" "" "" "" "" "" "" "" "" "u95"};
    ff2 = {"poil" "ys" "tobq" "winf" "inf" "rus" "dsc" "cf" "invest" "y" "CV1" "CV2" "CV3" "CV4"
           "poil" "ys" "tobq" "winf" "inf" "rus" "dsc" "cf" "invest" "y" "CV1" "CV2" "CV3" "CV4"
           "poil" "ys" "tobq" "winf" "inf" "rus" "dsc" "cf" "invest" "y" "CV1" "CV2" "CV3" "CV4"
           "poil" "ys" "tobq" "winf" "inf" "rus" "dsc" "cf" "invest" "y" "CV1" "CV2" "CV3" "CV4"
           "poil" "ys" "tobq" "winf" "inf" "rus" "dsc" "cf" "invest" "y" "CV1" "CV2" "CV3" "CV4"
           "poil" "ys" "tobq" "winf" "inf" "rus" "dsc" "cf" "invest" "y" "CV1" "CV2" "CV3" "CV4"};

    fffa1 = xlswritesa(ff1,"GIRBOOT","b1",1,"");
    fffa2 = xlswritesa(ff1,"GIRBOOT","b1",2,"");
    fffa3 = xlswritesa(ff1,"GIRBOOT","b1",3,"");
    fffa4 = xlswritesa(ff1,"GIRBOOT","b1",4,"");
    fffa5 = xlswritesa(ff1,"GIRBOOT","b1",5,"");
    fffa6 = xlswritesa(ff1,"GIRBOOT","b1",6,"");
    fffa7 = xlswritesa(ff1,"GIRBOOT","b1",7,"");
    fffa8 = xlswritesa(ff1,"GIRBOOT","b1",8,"");
    fffa9 = xlswritesa(ff1,"GIRBOOT","b1",9,"");
    fffa10 = xlswritesa(ff1,"GIRBOOT","b1",10,"");

    fffb1 = xlswritesa(ff2,"GIRBOOT","b2",1,"");
    fffb2 = xlswritesa(ff2,"GIRBOOT","b2",2,"");
    fffb3 = xlswritesa(ff2,"GIRBOOT","b2",3,"");
    fffb4 = xlswritesa(ff2,"GIRBOOT","b2",4,"");
    fffb5 = xlswritesa(ff2,"GIRBOOT","b2",5,"");
    fffb6 = xlswritesa(ff2,"GIRBOOT","b2",6,"");
    fffb7 = xlswritesa(ff2,"GIRBOOT","b2",7,"");
    fffb8 = xlswritesa(ff2,"GIRBOOT","b2",8,"");
    fffb9 = xlswritesa(ff2,"GIRBOOT","b2",9,"");
    fffb10 = xlswritesa(ff2,"GIRBOOT","b2",10,"");

    ffffa1 = xlswritesa(ff1,"OIRBOOT","b1",1,"");
    ffffa2 = xlswritesa(ff1,"OIRBOOT","b1",2,"");
    ffffa3 = xlswritesa(ff1,"OIRBOOT","b1",3,"");
    ffffa4 = xlswritesa(ff1,"OIRBOOT","b1",4,"");
    ffffa5 = xlswritesa(ff1,"OIRBOOT","b1",5,"");
    ffffa6 = xlswritesa(ff1,"OIRBOOT","b1",6,"");
    ffffa7 = xlswritesa(ff1,"OIRBOOT","b1",7,"");
    ffffa8 = xlswritesa(ff1,"OIRBOOT","b1",8,"");
    ffffa9 = xlswritesa(ff1,"OIRBOOT","b1",9,"");
    ffffa10 = xlswritesa(ff1,"OIRBOOT","b1",10,"");

    ffffb1 = xlswritesa(ff2,"OIRBOOT","b2",1,"");
    ffffb2 = xlswritesa(ff2,"OIRBOOT","b2",2,"");
    ffffb3 = xlswritesa(ff2,"OIRBOOT","b2",3,"");
    ffffb4 = xlswritesa(ff2,"OIRBOOT","b2",4,"");
    ffffb5 = xlswritesa(ff2,"OIRBOOT","b2",5,"");
    ffffb6 = xlswritesa(ff2,"OIRBOOT","b2",6,"");
    ffffb7 = xlswritesa(ff2,"OIRBOOT","b2",7,"");
    ffffb8 = xlswritesa(ff2,"OIRBOOT","b2",8,"");
    ffffb9 = xlswritesa(ff2,"OIRBOOT","b2",9,"");
    ffffb10 = xlswritesa(ff2,"OIRBOOT","b2",10,"");

f1 = {"Shock", "poil"}; 
f2 = {"Shock", "ys"}; 
f3 = {"Shock", "tobq"}; 
f4 = {"Shock", "winf"};
f5 = {"Shock", "inf"};   
f6 = {"Shock", "rus"};
f7 = {"Shock", "dsc"};
f8 = {"Shock", "cf"};   
f9 = {"Shock", "invest"}; 
f10 = {"Shock", "y"}; 

af1 = xlswritesa(f1, "GIRBOOT", "a1", 1,"");
af2 = xlswritesa(f2, "GIRBOOT", "a1", 2,"");
af3 = xlswritesa(f3, "GIRBOOT", "a1", 3,"");
af4 = xlswritesa(f4, "GIRBOOT", "a1", 4,"");
af5 = xlswritesa(f5, "GIRBOOT", "a1", 5,"");
af6 = xlswritesa(f6, "GIRBOOT", "a1", 6,"");
af7 = xlswritesa(f7, "GIRBOOT", "a1", 7,"");
af8 = xlswritesa(f8, "GIRBOOT", "a1", 8,"");
af9 = xlswritesa(f9, "GIRBOOT", "a1", 9,"");
af10 = xlswritesa(f10, "GIRBOOT", "a1", 10,"");

aff1 = xlswritesa(f1, "OIRBOOT", "a1", 1,"");
aff2 = xlswritesa(f2, "OIRBOOT", "a1", 2,"");
aff3 = xlswritesa(f3, "OIRBOOT", "a1", 3,"");
aff4 = xlswritesa(f4, "OIRBOOT", "a1", 4,"");
aff5 = xlswritesa(f5, "OIRBOOT", "a1", 5,"");
aff6 = xlswritesa(f6, "OIRBOOT", "a1", 6,"");
aff7 = xlswritesa(f7, "OIRBOOT", "a1", 7,"");
aff8 = xlswritesa(f8, "OIRBOOT", "a1", 8,"");
aff9 = xlswritesa(f9, "OIRBOOT", "a1", 9,"");
aff10 = xlswritesa(f10, "OIRBOOT", "a1", 10,"");

@ Export PP @
pph1 = {"MEAN" "" "" "" "SD" "" "" "" "l90" "" "" "" "u90" "" "" "" "l95" "" "" "" "u95" "" "" ""};
pph2 = {"CV1" "CV2" "CV3" "CV4" "CV1" "CV2" "CV3" "CV4" "CV1" "CV2" "CV3" "CV4" "CV1" "CV2" "CV3" "CV4" "CV1" "CV2" "CV3" "CV4" "CV1" "CV2" "CV3" "CV4"};
PPout1 = xlswritesa(pph1, "PPBOOT", "b1", 1,"");
PPout2 = xlswritesa(pph2, "PPBOOT", "b2", 1,"");
PPout3 = xlswritem(mHZPP~sdHZPP~l90HZPP~u90HZPP~l95HZPP~u95HZPP,"PPBOOT","b3",1,"");

"Finished"; end;

@* SUB PROCEDURES *@

@ ************************************* @
  proc (4) = TRDATA(Y,X,yorder,xorder);
@ ***************************************************************
*   Generate the appropriate Data matrices of DY, DX, Z1, DZLAGS,
*   to be used in the VEC model, depending on the order of VAR(p)
*   DY: First difference of endogenous I(1) variables
*   DX: First difference of exogenous I(1) variables
*   Z = X~Y
*   Z1: First lagged values of Z
*   ADZLAGS = DZ1~DZ2~...~DZ(p-1): T by m(p-1)
*   ************************************************************** @
local  MM,Y1,DY,Z,Z1,adzlags,dX,DZ,DZ1,DZ2,DZ3,DZ4,DZ5,DZ6,DZ7,DZ8,
       DZ9,DZ10,DZ11;

mm = yorder + xorder;
Y1 = LAG(Y); DY = Y - Y1;

if Xorder eq 0;   Z = Y; DX = 0; @ No exog variables: Txmy @
else;             Z = X~Y;       @ Txm @
                  DX = X - LAG(X);
endif;

Z1 = LAG(Z);     DZ = Z - Z1;    DZ1 = Lag(DZ);  DZ2 = Lag(DZ1);
DZ3 = Lag(DZ2);  DZ4 = Lag(DZ3); DZ5 = Lag(DZ4); DZ6 = Lag(DZ5);
DZ7 = Lag(DZ6);  DZ8 = Lag(DZ7); DZ9 = Lag(DZ8); DZ10 = Lag(DZ9);
DZ11 = Lag(DZ10);

@ all lagged values up to maximum lag order @
ADZLAGS = DZ1~DZ2~DZ3~DZ4~DZ5~DZ6~DZ7~DZ8~DZ9~DZ10~DZ11;

@* transpose the data for matching the specification *@
DY = DY';  DX = DX';  Z1 = Z1'; ADZLAGS = ADZLAGS';

retp(DY,Z1,DX,ADZLAGS);endp;

@ ********************************* @
  proc (4) = TRDATA2(idum,tdum,stt);
@ ********************************* @
@ idum1: lagged intercept shift dummy
* tdum1: lagged trend shift dummy
* ADidum = Didum~Didum1~...~Didum(p-1)
* ADtdum = Dtdum~Dtdum1~...~Dtdum(p-1)
* adding initial stt observations
* ************************************ @
local  stmp,idum1,Di0,Di1,Di2,Di3,Di4,Di5,Di6,Di7,Di8,Di9,Di10,Di11,
       tdum1,Dt0,Dt1,Dt2,Dt3,Dt4,Dt5,Dt6,Dt7,Dt8,Dt9,Dt10,Dt11;

stmp = zeros(stt,1);
idum = stmp|idum; tdum = stmp|tdum;

idum1 = LAG(idum); 
Di0 = idum - idum1; Di1 = Lag(Di0); Di2 = Lag(Di1); Di3 = Lag(Di2); 
Di4 = Lag(Di3); Di5 = Lag(Di4); Di6 = Lag(Di5); Di7 = Lag(Di6);  
Di8 = Lag(Di7); Di9 = Lag(Di8); Di10 = Lag(Di9); Di11 = Lag(Di10);
ADidum = Di0~Di1~Di2~Di3~Di4~Di5~Di6~Di7~Di8~Di9~Di10~Di11;

tdum1 = LAG(tdum); 
Dt0 = tdum - tdum1; Dt1 = Lag(Dt0); Dt2 = Lag(Dt1); Dt3 = Lag(Dt2); 
Dt4 = Lag(Dt3); Dt5 = Lag(Dt4); Dt6 = Lag(Dt5); Dt7 = Lag(Dt6);  
Dt8 = Lag(Dt7); Dt9 = Lag(Dt8); Dt10 = Lag(Dt9); Dt11 = Lag(Dt10);
ADtdum = Dt0~Dt1~Dt2~Dt3~Dt4~Dt5~Dt6~Dt7~Dt8~Dt9~Dt10~Dt11;

@* transpose the data for matching the specification *@
idum1 = idum1'; tdum1 = tdum1';
ADidum = ADidum'; ADtdum = ADtdum';

retp(idum1,ADidum,tdum1,ADtdum);endp;

@ *********************************************************** @
  proc (2) 
  = CREG(case,Z1,inpt,TR1,id1,td1,p,DZlags,mx,DX,TR,did,dtd);
@ *********************************************************** 
* CONSTRUCTION of ECM term, ZS1 and all other regressors, YY 
* ********************************************************** @
local zs1,yy;

if     case eq 1; ZS1 = Z1; 
elseif case eq 2; ZS1 = Z1|inpt;
elseif case eq 3; ZS1 = Z1;
elseif case eq 4; ZS1 = Z1|TR1;
elseif case eq 6; ZS1 = Z1|TR1|id1;
elseif case eq 7; ZS1 = Z1|TR1|td1;
endif;

if p eq 1; YY = 0;
else; YY = DZLAGS;
endif;

if YY eq 0;
   if mx eq 0;
   else; YY = DX;
   endif;
else;
   if mx eq 0;
   else; YY = YY|DX;
   endif;
endif;

if YY eq 0;
   if     case eq 3; YY = inpt;
   elseif case eq 4; YY = inpt;
   elseif case eq 5; YY = inpt|tr;
   elseif case eq 6; YY = inpt|did;
   elseif case eq 7; YY = inpt|dtd;
   endif;
else;
   if     case eq 3; YY = YY|inpt;
   elseif case eq 4; YY = YY|inpt;
   elseif case eq 5; YY = YY|inpt|tr;
   elseif case eq 6; YY = YY|inpt|did;
   elseif case eq 7; YY = YY|inpt|dtd; @ ?? @
   endif;
endif;

retp(ZS1,YY);endp;

@ ***************************************** @
  proc (2) = COINTEST(DY,Z1,YY,my,mx);
@ *****************************************
* (1) OLS of DY on YY to obtain R0 and then calculate
*     S00 = (1/T)R0*R0'
* (2) OLS of Z1 on YY to obtain R1 and then calculate
*     S11 = (1/T)R1*R1',      S01 = (1/T)R0*R1'
* (3) Then, calculate
*     Bmatirx = S11
*     Amatrix = S11 - S01'Invpd(S00)*S01
* for further details see Johansen (1988, 1991) and PSS (2000)
* ************************************************************* @
local  sample,URPAI,dreg,invYY,invS00,Amatrix,Bmatrix,
       S00,S01,S11,C0,C1,R0,R1,zorder,testres,
       pp,va,ve,dd,zi,sumva,maxva,Ltrace,Lmax,sumv,cvmax,cvtrace,
       vva,vaa,im1,im2,jbeta,cc;
sample = cols(dy);

@* Z1 = (Y1 X1) and any deterministic regressors @
zorder = rows(z1);  @ dimension of z1 @

@* Computation of S00, S11 and S10 @
if YY eq 0; R0 = DY; R1 = Z1;
else;
     InvYY = Invpd(YY*YY');
     C0 = DY*YY'InvYY;
     R0 = DY - C0*YY;
     C1 = Z1*YY'InvYY;
     R1 = Z1 - C1*YY;
endif;

S00 = R0*R0'./sample;
S11 = R1*R1'./sample;
S01 = R0*R1'./sample;

/* CC = invpd(S11)*s01'invpd(S00)*S01; save cc; */

@* Computation of Amatrix and Bmatrix @
InvS00 = invpd(S00);
Amatrix = S11 - S01'InvS00*S01;
Bmatrix = S11;

@* Unrestricted pi matrix @
URPAI = S01*invpd(S11);

@* Johansen Beta's using eigenvalue routine @
pp = chol(S11);
dd = inv(pp')*S01'invS00*S01*inv(pp);
{va,ve} = EIGHV(dd);

Jbeta = zeros(zorder,zorder);
zi = 1;
do until zi > zorder;
   Jbeta[.,zi] = inv(pp)*ve[.,zi];
   zi = zi + 1;
endo;

vva = zeros(zorder,1);
zi = 1;
do until zi > zorder;
   vva[zi,1] = va[zorder-zi+1,1];
   zi = zi + 1;
endo;

@* Trace and the Max Eigenvalue statistics @
vaa = zeros(zorder,1);
sumv = 0;
zi = 1;
do until zi > zorder;
   vaa[zi,1] = ln(1 - va[zi,1]);
   sumv = sumv + vaa[zi,1];
   zi = zi + 1;
endo;

sumva = zeros(my,1);
maxva = zeros(my,1);
sumva[1,1] = sumv;
maxva[1,1] = ln(1 - va[zorder,1]);
zi = 2;
do until zi > my;
   im1 = zorder + 2 - zi;
   sumva[zi,1] = sumva[zi-1,1] - vaa[im1,1];
   im2 = zorder + 1 - zi;
   maxva[zi,1] = ln(1 - va[im2,1]);
   zi = zi + 1;
endo;

Ltrace = zeros(my,1);  Lmax = zeros(my,1);
zi = 1;
do until zi > my;
   Ltrace[zi,1] = -sample*sumva[zi,1];
   Lmax[zi,1] = -sample*maxva[zi,1];
   zi = zi + 1;
endo;

VVA = VVA[1:my];
TESTRES = VVA~LTRACE~LMAX;

retp(TESTRES,JBETA);endp;

@ ************************************* @
  proc (7) = VECdy(dep,reg,sample,my,k);
@ ************************************* @
local all,e,omega,AIC,SBC,HQ,LL;

all = dep*reg'*invpd(reg*reg'); @ parameters: my by ?? @
e = dep - all*reg;              @ residuals: my by T @
omega = e*e'./(sample-rows(reg)); @ covariance: my by my @
/* omega = e*e'./(sample-rows(reg)-1); @ Mfit: restricted trend? @ */

LL = -(sample/2)*ln(det(e*e'./sample))-(sample*my/2)*(1+ln(2*pi));
@ log-likelihood function @
AIC = LL - k;
SBC = LL - (k/2)*ln(sample);
HQ  = LL - k*ln(ln(sample));

retp(all,e,omega,LL,AIC,SBC,HQ);endp;

@ ************************** @
  proc (7) = MARdx(dep,reg);
@ ************************** @
local all,e,omega,AIC,SBC,HQ,LL,k,mx,obs;

mx = rows(dep); obs = cols(dep);
all = dep*reg'*invpd(reg*reg'); @ parameters: mx by ?? @
e = dep - all*reg;              @ residuals: mx by T @
omega = e*e'./(obs-rows(reg)); @ covariance: my by my @

if mx eq 1;
   LL = -(obs/2)*ln(e'e/obs)-(obs/2)*(1+ln(2*pi));
else;
   LL = -(obs/2)*ln(det(e*e'./obs))-(obs*mx/2)*(1+ln(2*pi));
endif;

k = rows(all)*cols(all);
AIC = LL - k;
SBC = LL - (k/2)*ln(obs);
HQ  = LL - k*ln(ln(obs));

retp(all,e,omega,LL,AIC,SBC,HQ);endp;

@ ********************************************* @
  proc (3) = VECM(ECMco,DEP,REG,RES,YHAT,nreg);
@ ********************************************* 
  Computation of estimation and diagnostic test results 
* ***************************************************** @
local diagres,sige,invreg,var,se,tratio,ti,estres;

@* computation of R2 and diagnostic statistics using VECM @
{ DIAGRES1, DIAGRES2 } = DIAGNOS(dep,reg,res,yhat,nreg);

sige = diagres1[3]^2;       @ sig^2 @
invreg = invpd(reg'reg);
var = sige.*invreg;
se = sqrt(diag(var));

tratio = zeros(rows(ecmco),1);
ti = 1;
do until ti > rows(ecmco);
   tratio[ti] = ecmco[ti]/se[ti];
   ti = ti + 1;
endo;

ESTRES = ecmco~se~tratio;

retp(ESTRES,DIAGRES1,DIAGRES2); endp;

@ ****************************************** @
  proc (2) = DIAGNOS(dep,reg,res,yhat,nreg);
@ **************************************************
* This procedure computes the following for ECM eq:
* dy(i) = phi(i)*y(i,-1) + theta(i)*x(i) + e(i),??
* (1) R-squared and Rbar-squared
* (2) Maximized log-likelihood, AIC, SBC, HQ
* (3) Diagnostic statistics such as
* Godfrey's test of residual serial correlation: chi(4)
* Ramsey's RESET test of functional form: chi(1)
* Jarque-Bera's test of the normality: chi(2)
* Test of Heterosckedasticity: chi(1)
* see Microfit manual
* ********************************************** @
local obs,sig2,sige,LL,AIC,SBC,HQ,Rbar,RSQ,ysq2,rbarsq,
      e,x,p,mx,w,p2,p1,num,den,mu1,e2,mu2,e3,mu3,e4,mu4,
      yhat2,re,ure,R2,tmpa,tmpb,tmpc,hdep,hconst,hindep,
      chiSC,chiFF,chiNO,chiHE,diagres1,diagres2,
      tmp2a,tmp2b,wtmp,pSC,pFF,pNO,pHE;

@* computation of the likelihood, AIC, SC, HQ and Rbar square @
obs = rows(res);
sig2 = res'res/obs;
sige = res'res/(obs-nreg);   @ variance estimate with dof @
sige = sqrt(sige);           @ standard error of regression @

LL = -(obs/2)*(1+ln(2*pi*sig2));   @ log-likelihood @

AIC = LL - nreg;                   @ Akaike @
SBC = LL - (nreg/2)*ln(obs);       @ Schwarz @
HQ  = LL - nreg*ln(ln(obs));       @ Hannan-Quinn @

ysq2 = sumc((dep-meanc(dep))^2)/obs;  @ sum of squared dmeaned dep @
RSQ = 1-(sig2/ysq2);                  @ R-squared @
RBARSQ = 1-((obs-1)/(obs-nreg))*(1-RSQ); @ adjyusted R2 @

@* computation of diagnostic test statistics using the OLS residuals @
e = res;   @ Tx1 residual @
x = reg;   @ Txk indep vble @
p = 4;     @ lags used in SC test @

@* LM test for serial correlation @
@ computing W matrix for SC @
w = zeros(obs,p);   @ Txp @
p2 = 1;
do until p2 > p;
   w[obs,p+1-p2] = e[obs-p+p2-1,1];
   p1 = 1;
   do until p1 > obs-1;
      if obs-p+p2-1-p1 > 0;
         w[obs-p1,p+1-p2] = e[obs-p+p2-1-p1,1];
      endif;
      p1 = p1 + 1;
   endo;
   p2 = p2 + 1;
endo;
wtmp = w'w - w'x*invpd(x'x)*x'w;
num = e'w*invpd(wtmp)*w'e;
den = e'e/obs;
chiSC = num/den;        @ statistic @
pSC = cdfchic(chiSC,4); @ p-value @

@* LM test for functional form @
w = yhat.^2;
wtmp = w'w - w'x*invpd(x'x)*x'w;
num = e'w*invpd(wtmp)*w'e;
chiFF = num/den;
pFF = cdfchic(chiFF,1); @ p-value @

@* LM test for normality @
mu1 = sumc(e)/obs;
e2 = e.^2;
mu2 = sumc(e2)/obs;
e3 = e.^3;
mu3 = sumc(e3)/obs;
e4 = e.^4;
mu4 = sumc(e4)/obs;
tmpa = mu3^2/(6*mu2^3);
tmpb = (mu4/mu2^2 - 3)^2;
tmpc = 3*mu1^2/(2*mu2) - (mu3*mu1/mu2^2);
chiNO = obs*(tmpa+(1/24)*tmpb) + obs*tmpc;
pNO = cdfchic(chiNO,2); @ p-value @

@* LM test for heterockedasticity @
hdep   = e2;
hconst = ones(obs,1);
yhat2 = yhat.^2;
hindep = hconst~yhat2;
ure = hdep - hindep*invpd(hindep'hindep)*hindep'hdep; @ unrestricted res @
re = hdep - hconst*invpd(hconst'hconst)*hconst'hdep;  @ restricted res @
R2 = (re're - ure'ure)/re're;
chiHE = obs*R2;
pHE = cdfchic(chiHE,1); @ p-value @

DIAGRES1 = RSQ|RBARSQ|SIGE|LL|AIC|SBC|HQ; @ 7x1 @
tmp2a = chiSC~chiFF~chiNO~chiHE; 
tmp2b = pSC~pFF~pNO~pHE; 
DIAGRES2 = tmp2a|tmp2b; @ 2x4 @

retp(DIAGRES1,DIAGRES2);endp;

@ ********************************* @
  proc (0) = PRECM(EST,DIAG1,DIAG2);
@ ********************************* @
local pval1,pval2,pval3,pval4,rr2,rr3;

"    Coeff.   Std. Err  T-ratio";"";
call printfm(est,1~1~1,"lf"~10~4|"lf"~10~4|"lf"~10~4);"";
"Diagnostic Statistics";"";
" RSQ    RBARSQ   SIGE     LL    AIC    SBC     HQ";"";
rr2 = diag1';
call printfm(rr2,1~1~1~1~1~1~1,"lf"~7~3|"lf"~7~3|"lf"~7~3|"lf"~7~2|"lf"~7~2
               |"lf"~7~2|"lf"~7~2);"";
rr3 = diag2;
"Ch-SC  Ch-FF  Ch-NO  Ch-HE";"";
call printfm(rr3,1~1~1~1,"lf"~7~2|"lf"~7~2|"lf"~7~2|"lf"~7~2);"";"";

retp;endp;

@ ******************************************** @
  proc (9) = decompY(ally,r,p,my,mx,case,beta);
@ ******************************************** @
@ Decomposition into separate parameter blocks for VEC for dy 
* Remind: the order of regressors in VECM is:
* ecm(-1)|dzlags|DX|deterministics where
* the dimensions (row) are r, m(p-1), mx, and 1,2 or 3 
* In Cases 6 and 7, we need to conisder the coeffs on FD dummies 
* whose dimension is equal to p 
* ***************************** @
local Alp,Gam,Lam,b0,b1,d0,d1,dd,dt,i1,i2,m;

m = my + mx;
if r eq 0;
   Alp = zeros(my,r); @ no coin @
else;
   i1 = 1; i2 = i1+r-1;
   Alp = allY[.,i1:i2];
endif;

if p eq 1; 
   Gam = 0; @ lag order @
else; 
   i1 = r+1;  i2 = i1+(p-1)*m-1;
   Gam = allY[.,i1:i2];  @ my by m(p-1)  @
endif;

if mx eq 0;  @ xorder @
   Lam = 0;
else;  
   i1 = r+(p-1)*m+1; i2 = i1+mx-1;
   Lam = allY[.,i1:i2];  @ my by mx @
endif;

b0 = zeros(my,1);  @ intercepts @
b1 = zeros(my,1);  @ trend coefficients @
d0 = zeros(my,1);  @ intercept dummy coefficients @
d1 = zeros(my,1);  @ trend dummy coefficients @
dd = zeros(my,p);  @ coefficients of FD int dum @
dt = zeros(my,p);  @ coefficients of FD tr dum @

if     case eq 2;  
       if r eq 0;
       else;  b0 = alp*beta[m+1,.]';@ restricted trends @
       endif;
elseif case eq 3;
       i1 = r+(p-1)*m+mx+1; b0 = allY[.,i1]; 
elseif case eq 4;
       i1 = r+(p-1)*m+mx+1; b0 = allY[.,i1]; 
       if r eq 0;
       else; b1 = alp*beta[m+1,.]'; @ restricted trends @
       endif;
elseif case eq 5;
       i1 = r+(p-1)*m+mx+1; b0 = allY[.,i1]; b1 = allY[.,i1+1];   
elseif case eq 6;
       i1 = r+(p-1)*m+mx+1; 
       b0 = allY[.,i1]; dd = allY[.,i1+1:i1+p]; 
       if r eq 0;
       else; b1 = alp*beta[m+1,.]';  @ restricted trends @
             d0 = alp*beta[m+2,.]';  @ restricted int dum @
       endif;
elseif case eq 7;
       i1 = r+(p-1)*m+mx+1;
       b0 = allY[.,i1]; dt = allY[.,i1+1:i1+p]; 
       if r eq 0;
       else; b1 = alp*beta[m+1,.]';  @ restricted trends @
             d1 = alp*beta[m+2,.]';  @ restricted int dum @
       endif;
endif;

retp(Alp,Gam,Lam,b0,b1,d0,d1,dd,dt);endp;

@ ************************************** @
  proc (4) = decompX(allx,p,my,mx,case);
@ ************************************** @
@ Decomposition into separate parameter blocks for VAR for dx:
* Remind: the order of regressors in VAR is: dzlags|deterministics 
* where the dimensions (row) are m(p-1) and 1,2 or plus p. 
* In Cases 6 and 7, we need to conisder the coeffs on FD dummies 
* whose dimension is equal to p 
* ***************************** @
local Gam,b0,b1,dd,dt,i1,i2,m;

m = my+mx;
if p eq 1; @ lag order @
   Gam = 0; 
else; 
   i1 = 1;  i2 = i1+(p-1)*m-1;
   Gam = allX[.,i1:i2];  @ mx by m(p-1)  @
endif;

b0 = zeros(mx,1);  @ intercepts @
b1 = zeros(mx,1);  @ trends @
dd = zeros(mx,p);  @ coefficients of FD int dum @
dt = zeros(mx,p);  @ coefficients of FD tr dum @

if     case eq 3;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; 
elseif case eq 4;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; 
elseif case eq 5;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; b1 = allX[.,i1+1];   
elseif case eq 6;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; dd = allX[.,i1+1:i1+p]; 
elseif case eq 7;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; dt = allX[.,i1+1:i1+p]; 
endif;

retp(Gam,b0,dd,dt);endp;

@ ******************************** @
  proc(3) = DYPARA(pai,gam,p,m,n);
@ ********************************
* Computation of MA parameters
* FAI: FAI(1),...,FAI(p), m by m*p
* FF: F matrix, mp by mp augmented matrix
* BN: m by m MA matrix
* **************************************** @
local  pgam,kk,jj,fai,ff,bn,ki,fi,ktmp1,ktmp2,ktmp3,
       Ftmp1,Ftmp2,Ftmp3,bi,bi1,btemp;

@* Computation of Fai matrix @
if p eq 1; PGAM = PAI;
else;      PGAM = PAI~GAM;   @ m by pm @
endif;

if p eq 1; KK = -eye(m); JJ = eye(m);
else;
   Ktmp1 = -eye(m*p);
   Ktmp2 = zeros(p,p);
   ki = 1;
   do until ki > p-1;
      Ktmp2[ki+1,ki] = 1;
      ki = ki + 1;
   endo;
   Ktmp3 = Ktmp2.*.eye(m);

   KK = Ktmp1 + Ktmp3;
   JJ = eye(m)~zeros(m,m*(p-1));
endif;
FAI = PGAM*KK + JJ;  @ FAI1,...,FAIp: m by m*p @

if p eq 1;  FF = FAI;
else;
   ftmp1 = FAI|zeros((p-1)*m,m*p);
   ftmp2 = zeros(p,p);
   fi = 1;
   do until fi > p-1;
      ftmp2[fi+1,fi] = 1;
      fi = fi + 1;
   endo;
   ftmp3 = ftmp2.*.eye(m);
   FF = ftmp1 + ftmp3;
endif;

@* Computation of Bn matrix (MA) @
BN = zeros(m,n*m);  @ MA coefficients @
btemp = eye(m*p);
bi = 1;
do until bi > n;
   btemp = FF*btemp;
   bi1 = (bi-1)*m+1;
   BN[.,bi1:bi1+m-1] = JJ*btemp*JJ';
   bi = bi + 1;
endo;

retp(FAI,FF,BN);endp;

@ *********************************************** @
  proc (2) = CHKSTAB(m,rcoin,p,alpha,beta,gam,n);
@ *********************************************** @
local  Amat,bet,aj,nGam,Gam0,Gamj,Amatj,eigstab,ve,Ir,Zr,gtmp,
       ggtmp1,ggtmp2,ggtmp,atmp1,atmp2,atmp,AN,GN,BigA,va,jg,gg,gi,
       gi1,gi2,rj0,neig,vatmp,modva,modvatmp;

Ir = eye(rcoin); Zr = zeros(rcoin,rcoin);
if p eq 1;
   Amat = Ir + beta'alpha;
   va = eig(Amat);
   va = sortc(va,1);
   va = rev(vatmp);  @ descending order @
   { modva } = MODULUS(va);

elseif p > 1;
   if p eq 2;     JG = eye(m);
                       GG = Gam;
   elseif p > 2;  JG = eye(m)~zeros(m,m*(p-2));
                       GGtmp1 = eye(m*(p-2));
                       GGtmp2 = zeros(m*(p-2),m);
                       GGtmp = GGtmp1~GGtmp2;
                       GG = Gam|GGtmp;
   endif;

   GN = zeros(m,m*n);  @ MA matrix: G1,G2,...,GN @
   gtmp = eye(m*(p-1));
   gi = 1;
   do until gi > n;
      gtmp = GG*gtmp;
      gi1 = (gi-1)*m+1;
      GN[.,gi1:gi1+m-1] = JG*gtmp*JG';
      gi = gi + 1;
   endo;

   AN = zeros(rcoin,rcoin*n);  @ A1,A2,...,AN @
   AN[.,1:rcoin] = Ir + beta'alpha;  @ A1 @
   gi = 2;
   do until gi > n;
      gi1 = (gi-1)*rcoin+1;
      gi2 = (gi-2)*m+1;
      AN[.,gi1:gi1+rcoin-1] = beta'GN[.,gi2:gi2+m-1]*alpha;
      gi = gi + 1;
   endo;

   atmp1 = eye(rcoin*(n-1));
   atmp2 = zeros(rcoin*(n-1),rcoin);
   atmp = atmp1~atmp2;
   BigA = AN|atmp;   @ Big A matrix: rl by rl @

   va = zeros(rcoin*n,n);  @ eigenvalues of BigA @
   modva = zeros(rcoin*n,n);  @ modulus of BigA @
   aj = 1;
   do until aj > n;
      rj0 = aj*rcoin;
      Amat = BigA[1:rj0,1:rj0];  @ rl by rl @
      neig = rows(Amat);  @ number of eigenvalues @
      vatmp = eig(Amat);
      vatmp = sortc(vatmp,1);
      vatmp = rev(vatmp);  @ descending order @
      { modvatmp } = MODULUS(vatmp);
      va[1:neig,aj] = vatmp;
      modva[1:neig,aj] = modvatmp;
      aj = aj + 1;
   endo;
endif;

retp(va,modva);endp;

@ *************************** @
  proc (1) = MODULUS(vatemp);
@ *************************** @
local rva,iva,vj,modvatmp;

rva = real(vatemp);  @ real part of eigenvalues @
iva = imag(vatemp);  @ imaginary part of eigenvalues @

modvatmp = zeros(rows(vatemp),1);
vj = 1;
do until vj > rows(vatemp);
   modvatmp[vj] = sqrt(rva[vj]^2 + iva[vj]^2);
   vj = vj + 1;
endo;

modvatmp = sortc(modvatmp,1);
modvatmp = rev(modvatmp);  @ descending order @

retp(modvatmp);endp;

@ ************************************* @
   proc (1) = persis(m,n,BETA,OMEGA,BN);
@ *************************************
*  Calculate persistence profiles of LR relations,
*  hz(i) = (beta'Bi*Omega*Bi'beta)/(beta'Omega*beta)
*  Here Omega is correlated.
* ************************************************** @
local   hz,DENOM,jp,temp,temp1,ro,co,vect,j1;

BETA = BETA.*1000;    @ hz is invariant to scale @
DENOM = beta'OMEGA*beta;

hz = zeros(n,1);
temp = zeros(m,m);
jp = 1;
do until jp > n;
   j1 = (jp-1)*m+1;
   temp = BN[.,j1:j1+m-1];
   temp1 = beta'(temp*OMEGA*temp')*beta;
   hz[jp,1] = temp1/Denom;
   jp = jp + 1;
endo;

retp(hz);endp;

@ ******************************************************* @
  proc (4) = Impulse(sh_op,m,n,RCOIN,BETA,gOMEGA,BN,IA0);
@ *******************************************************
* GIR (OIR) is the generalized (orthogonalised) impulse 
* responses of m variables, whilst GIRZ (OIRZ) is GIR (OIR) 
* of r cointegrating relations with respect to shocks in jth 
* equation, with h = 1,2,... and j = 1 to m.
* ******************************************************* @
local   evec,sigjj,gir0,gir,girz0,girz,zi,j1,bj,chk1,chk2,
        upp,lpp,ipp,oir0,oir,oirz0,oirz;

evec = zeros(m,1);
evec[sh_op] = 1;    @ selection vector @
sigjj = evec'gomega*evec;

GIR0  = IA0*gOmega*evec./sqrt(sigjj);    @ impact impulse @
GIRZ0 = beta'IA0*gOmega*evec./sqrt(sigjj);

UPP = Chol(gomega); LPp = UPp';
Ipp = Inv(LPp);
OIR0  = IA0*LPP*evec;           @ impact impulse @
OIRZ0 = beta'IA0*LPP*evec;

GIR = zeros(m,n); GIRZ = zeros(rcoin,n);     
OIR = zeros(m,n); OIRZ = zeros(rcoin,n);     
zi = 1;
do until zi > n;
   j1 = (zi-1)*m + 1;
   Bj = BN[.,j1:j1+m-1];
   GIR[.,zi]  = Bj*IA0*gOmega*evec./sqrt(sigjj);
   GIRZ[.,zi] = beta'Bj*IA0*gOmega*evec./sqrt(sigjj);
   OIR[.,zi]  = Bj*IA0*LPP*evec;
   OIRZ[.,zi] = beta'Bj*IA0*LPP*evec;
   zi = zi + 1;
endo;

GIR = GIR0~GIR;   GIRZ = GIRZ0~GIRZ;
OIR = OIR0~OIR;   OIRZ = OIRZ0~OIRZ;

@ transposing the results such that nh+1 (or r) by m @
GIR = GIR'; GIRZ = GIRZ'; OIR = OIR'; OIRZ = OIRZ';

retp(GIR,GIRZ,OIR,OIRZ);endp;

@ ************************************************* @
  proc (14) = STIRF(m,n,r,BETA,VOMEGA,BN,IA11,W11);
@ *************************************************
  SIR is structural IRF of m variables,
  SIRZ is structural IRF of r coin relations wrt
  shocks in jth equation, with n = 1,2,... and j = 1 to m.
* ******************************************************** @
local A11,ovec,ysvec,tqvec,psvec,wivec,invec,rvec, ivec,zi,j1,bj,
      saie,sair,deltar,osir0a,osir0b,osir0,osirz0,yssir0a,yssir0b,
      yssir0,yssirz0,tqsir0a,tqsir0b,tqsir0,tqsirz0,wisir0a,wisir0b,
      wisir0,wisirz0,insir0a,insir0b,insir0,insirz0,rsir0a,rsir0b,
      rsir0,rsirz0,isir0a,isir0b,isir0,isirz0,OSIR,OSIRZ,YSSIR,
      YSSIRZ,TQSIR,TQSIRZ,PSSIR,PSSIRZ,WISIR,WISIRZ,INSIR,INSIRZ,
      RSIR,RSIRZ,ISIR,ISIRZ;

@ Now n_or = 6, when it changes, the proc should change accordingly. @ 
A11 = Inv(IA11);

@* Oil price shock @
ovec = zeros(6,1); ovec[1] = W11[1,1];
OSIR0A = IA11*ovec;
OSIR0B = Vomega[7:m,1];
OSIR0 = OSIR0A|OSIR0B; @ m by 1 @
OSIR0 = OSIR0./sqrt(ovec[1]);
OSIRZ0 = beta'OSIR0;

@* Potential output (productivity?) shock @
ysvec = zeros(6,1); ysvec[2] = W11[2,2];
YSSIR0A = IA11*ysvec;
YSSIR0B = Vomega[7:m,2] + A11[2,1].*Vomega[7:m,1];
YSSIR0 = YSSIR0A|YSSIR0B;
YSSIR0 = YSSIR0./sqrt(ysvec[2]);
YSSIRZ0 = beta'YSSIR0;

@* Tobin's Q (stock market?) shock @
tqvec = zeros(6,1); tqvec[3] = W11[3,3];  
TQSIR0A = IA11*tqvec;
TQSIR0B = Vomega[7:m,3] + A11[3,2].*Vomega[7:m,2] 
        + A11[3,1].*Vomega[7:m,1];
TQSIR0 = TQSIR0A|TQSIR0B;
TQSIR0 = TQSIR0./sqrt(tqvec[3]);
TQSIRZ0 = beta'TQSIR0;

@* Wage inflation shock @
wivec = zeros(6,1); wivec[4] = W11[4,4];
WISIR0A = IA11*wivec;
WISIR0B = Vomega[7:m,4] + A11[4,3].*Vomega[7:m,3] 
       + A11[4,2].*Vomega[7:m,2] + A11[4,1].*Vomega[7:m,1];
WISIR0 = WISIR0A|WISIR0B;
WISIR0 = WISIR0./sqrt(wivec[4]);
WISIRZ0 = beta'WISIR0;

@* Inflation shock @
invec = zeros(6,1); invec[5] = W11[5,5];
INSIR0A = IA11*invec;
INSIR0B = Vomega[7:m,5] + A11[5,4].*Vomega[7:m,4] 
       + A11[5,3].*Vomega[7:m,3] + A11[5,2].*Vomega[7:m,2] 
       + A11[5,1].*Vomega[7:m,1];
INSIR0 = INSIR0A|INSIR0B;
INSIR0 = INSIR0./sqrt(invec[5]);
INSIRZ0 = beta'INSIR0;

@* Interest rate (monetary policy) shock @
rvec = zeros(6,1); rvec[6] = W11[6,6];
RSIR0A = IA11*rvec;
RSIR0B = Vomega[7:m,6] + A11[6,5].*Vomega[7:m,5] 
       + A11[6,4].*Vomega[7:m,4] + A11[6,3].*Vomega[7:m,3] 
       + A11[6,2].*Vomega[7:m,2] + A11[6,1].*Vomega[7:m,1];
RSIR0 = RSIR0A|RSIR0B;
RSIR0 = RSIR0./sqrt(rvec[6]);
RSIRZ0 = beta'RSIR0;

@* interest intercept shift @
ISIR0 = zeros(m,1);
ISIR0[6] = sqrt(W11[6,6]); @ exogenous impact @
ISIRZ0 = beta'ISIR0;

OSIR = zeros(m,n);  OSIRZ = zeros(r,n);
YSSIR = zeros(m,n); YSSIRZ = zeros(r,n);
TQSIR = zeros(m,n); TQSIRZ = zeros(r,n);
PSSIR = zeros(m,n);  PSSIRZ = zeros(r,n);
WISIR = zeros(m,n);  WISIRZ = zeros(r,n);
INSIR = zeros(m,n);  INSIRZ = zeros(r,n);
RSIR = zeros(m,n);  RSIRZ = zeros(r,n);
ISIR = zeros(m,n);  ISIRZ = zeros(r,n);
zi = 1;
do until zi > n;
   j1 = (zi-1)*m + 1;
   Bj = BN[.,j1:j1+m-1];
   OSIR[.,zi] = Bj*OSIR0;   OSIRZ[.,zi] = beta'Bj*OSIR0;
   YSSIR[.,zi] = Bj*YSSIR0; YSSIRZ[.,zi] = beta'Bj*YSSIR0;
   TQSIR[.,zi] = Bj*TQSIR0; TQSIRZ[.,zi] = beta'Bj*TQSIR0;
   WISIR[.,zi] = Bj*WISIR0;   WISIRZ[.,zi] = beta'Bj*WISIR0;
   INSIR[.,zi] = Bj*INSIR0;   INSIRZ[.,zi] = beta'Bj*INSIR0;
   RSIR[.,zi] = Bj*RSIR0;   RSIRZ[.,zi] = beta'Bj*RSIR0;
   ISIR[.,zi] = Bj*ISIR0;   ISIRZ[.,zi] = beta'Bj*ISIR0;
zi = zi + 1;
endo;

OSIR = OSIR0~OSIR;    OSIRZ = OSIRZ0~OSIRZ;
YSSIR = YSSIR0~YSSIR; YSSIRZ = YSSIRZ0~YSSIRZ;
TQSIR = TQSIR0~TQSIR; TQSIRZ = TQSIRZ0~TQSIRZ;
WISIR = WISIR0~WISIR;    WISIRZ = WISIRZ0~WISIRZ;
INSIR = INSIR0~INSIR;    INSIRZ = INSIRZ0~INSIRZ;
RSIR = RSIR0~RSIR;    RSIRZ = RSIRZ0~RSIRZ;
ISIR = ISIR0~ISIR;    ISIRZ = ISIRZ0~ISIRZ;

@* transposing the results such that nh by m (or r) @

OSIR = OSIR';   OSIRZ = OSIRZ';
YSSIR = YSSIR'; YSSIRZ = YSSIRZ';
TQSIR = TQSIR'; TQSIRZ = TQSIRZ';
WISIR = WISIR';   WISIRZ = WISIRZ';
INSIR = INSIR';   INSIRZ = INSIRZ';
RSIR = RSIR';   RSIRZ = RSIRZ';
ISIR = ISIR';   ISIRZ = ISIRZ';

retp(OSIR,OSIRZ,YSSIR,YSSIRZ,TQSIR,TQSIRZ,WISIR,WISIRZ,INSIR,INSIRZ,RSIR,RSIRZ,ISIR,ISIRZ );
endp;

@ ****************************** @
  proc (2) = TRDATA4(Zd,my,mx,p);
@ ****************************** @
local  z,Z1,adzlags,DZ,DZ1,DZ2,DZ3,DZ4,DZ5,DZ6,DZ7,DZ8,
       DZ9,DZ10,DZ11,ZT,DZLAGST,m;
z=zd;
Z1 = LAG(Z);     DZ = Z - Z1;    DZ1 = Lag(DZ);  DZ2 = Lag(DZ1);
DZ3 = Lag(DZ2);  DZ4 = Lag(DZ3); DZ5 = Lag(DZ4); DZ6 = Lag(DZ5);
DZ7 = Lag(DZ6);  DZ8 = Lag(DZ7); DZ9 = Lag(DZ8); DZ10 = Lag(DZ9);
DZ11 = Lag(DZ10);

@ all lagged values up to maximum lag order @
m = my+mx;
ADZLAGS = DZ~DZ1~DZ2~DZ3~DZ4~DZ5~DZ6~DZ7~DZ8~DZ9~DZ10;
DZLAGST = ADZLAGS[rows(ADZLAGS),1:m*(p-1)]'; @ m(p-1) by 1 @
ZT = Z[rows(Z),.]';  @ m by 1 @

retp(ZT,DZLAGST);endp;

@ ************************************* @
  proc (1) = REGXT(DZT,case,sample,k,p); 
@ ************************************* @
local XregT,k1;

@* Case A: Unrestricted  @
k1 = sample+k; @ T+k-1 @
XregT = DZT; @ m(p-1) by 1 at T @
if     case eq 3; XregT = XregT|1; 
elseif case eq 4; XregT = XregT|1; 
elseif case eq 5; XregT = XregT|1|k1; 
elseif case eq 6; XregT = XregT|1|zeros(p,1); 
elseif case eq 7; XregT = XregT|1|ones(p,1); 
endif;

retp(XregT);endp; 

@ ****************************************************** @
  proc (1) = REGYT(Z1T,DZLT,DXT,case,p,mx,beta,sampl,k,h); 
@ ****************************************************** @
local k1,k2,k3,ZS1T,YregT;

k1 = sampl+k; @ T+k @
k2 = sampl+k-1; @ T+k-1 @
k3 = h+k-1; @ Tdum+k-1 @

ZS1T = Z1T;
if     case eq 2; ZS1T = ZS1T|1;
elseif case eq 3; ZS1T = ZS1T;
elseif case eq 4; ZS1T = ZS1T|k2;
elseif case eq 6; ZS1T = Z1T|k2|1;
elseif case eq 7; ZS1T = Z1T|k2|k3;
endif;

YregT = beta'ZS1T; @ r by 1 @

if p eq 1; YregT = YregT|DXT;
else; YregT = YregT|DZLT|DXT;
endif;

if     case eq 3; YregT = YregT|1;
elseif case eq 4; YregT = YregT|1;
elseif case eq 5; YregT = YregT|1|k1;
elseif case eq 6; YregT = YregT|1|zeros(p,1);
elseif case eq 7; YregT = YregT|1|ones(p,1); @ ?? @
endif;

retp(YregT);endp; 

@ ******************************** @
  proc (2) = A4CHANGE(lhat,z,n,m);
@ ******************************** @
local a4hat,ahat,zi,zt4,suml,a4tmp,zj,ztmp,obs;

@* Forecasts of annual 4-qt MA and average over n @
obs = rows(z);
zt4 = z[obs-3:obs,.];  @ initial 4 by m @
A4hat = zeros(n,m); Ahat = zeros(n,m);  suml = zeros(1,m);
zi = 1;
do until zi > n;
   zt4 = zt4[2:4,.]|Lhat[zi,.];
   A4tmp = zeros(1,m);
   zj = 1;
   do until zj > m;
      ztmp = zt4[.,zj];
      A4tmp[zj] = meanc(ztmp);
      zj = zj + 1;
   endo;
   A4hat[zi,.] = A4tmp;
   sumL = sumL + Lhat[zi,.];
   Ahat[zi,.] = sumL./zi;
   zi = zi + 1;
endo;

retp(a4hat,ahat); endp;

@ ************************************************************************* @
  proc (2) = stocfor(ZD,sample,n,mx,my,p,case,hh,beta,allY,allX,Lpz,s,v,rn);
@ ************************************************************************* 
* h-step ahead point forecasts plus future forecast errors
* rn is m by n vector of N(0,1) 
* ***************************** @
local m,sdz,sz,h,k,Z1T,DZlagsT,XregT,DXT,YregT,DYT,DZT,ZT,rhat,rxhat,ryhat,rnum;

m = mx + my;
SDZ = zeros(n,m); @ forecasts of dz: n by m @
SZ  = zeros(n,m); @ forecasts of z:  n by m @
h = 1;
do until h > n;

@* draws of future errors (uncertainty): jointly @
   if     s eq 1; rhat = rn[.,h];
   elseif s eq 2; rnum = rndu(1,1)*sample;
                  rhat = v[.,ceil(rnum)];
   endif;
   rhat = lpz*rhat;  @ m by 1 @ 
/*  rhat = zeros(m,1); */
   rxhat = rhat[1:mx]; ryhat = rhat[mx+1:m];

@* data transformation for forecasting @
   { Z1T, DZlagsT } = TRDATA4(Zd,my,mx,p);

@* generating the regressors for VAR dx @
   k=h;
   { XregT } = REGXT(DZlagsT,case,sample,k,p); 

@* forcast of DX at T+1: mx by 1 @
   DXT = allX*XregT + rxhat; 

@* generating the regressors for VEC dy @
   { YregT } 
   = REGYT(Z1T,DZlagsT,DXT,case,p,mx,beta,sample,k,hh);

@* forcast of DY at T+1: my by 1 @
   DYT = allY*YregT + ryhat; 

@* forcast of DZ and Z at T+j, m by 1 @
   DZT = DXT|DYT;  ZT = DZT + Z1T; 

@* saving forcasts of DZ and Z at h=1,...,n, m by 1 @
   SDZ[h,.] = DZT'; SZ[h,.] = ZT';

@* updating Z @
   ZD = ZD|ZT'; 

   h = h + 1;
endo;

retp(SZ,SDZ);endp;

@ ********************************************************** @
  proc (8) = PREV2(dz,d2z,a4dz,a4d2z,adz,ad2z,a4adz,a4ad2z);
@ ********************************************************** 
* Probility evaluation for 9 events for directional change and
* integral transforms: see the def for 9 events above 
* Currently, the variables in the order of
* po,rs,ps,e,r,dp,y,pps,hy
* ************************ @
local fdd,fdd2,a4fdd,a4fdd2,fid,fid2,a4fid,a4fid2,fj;

m = cols(dz); @ dimension @

@ increase the precision @
dz = 10000.*dz; d2z = 10000.*d2z;
a4dz = 10000.*a4dz; a4d2z = 10000.*a4d2z;
adz = 10000.*adz; ad2z = 10000.*ad2z;
a4adz = 10000.*a4adz; a4ad2z = 10000.*a4ad2z;

@ dummies for directional changes and integral transforms @
fdd = zeros(1,m);   @ direction of diff, quarterly @
fdd2 = zeros(1,m);  @ direction of 2nd diff @
a4fdd = zeros(1,m);   @ direction of diff, 4-qt MA @
a4fdd2 = zeros(1,m);  @ direction of 2nd diff @
fid = zeros(1,m);   @ integral transform of diff, quarterly @
fid2 = zeros(1,m);  @ integral transform of 2nd diff @
a4fid = zeros(1,m);   @ integral transform of diff, 4-qt MA @
a4fid2 = zeros(1,m);  @ integral transform of 2nd diff @

fj=1;
do until fj > m;

   if dz[fj] > 0; fdd[fj]=1;
   endif;
   if d2z[fj] > 0; fdd2[fj]=1;
   endif;
   if a4dz[fj] > 0; a4fdd[fj]=1;
   endif;
   if a4d2z[fj] > 0; a4fdd2[fj]=1;
   endif;

   if dz[fj] > adz[fj]; fid[fj]=1;
   endif;
   if d2z[fj] > ad2z[fj]; fid2[fj]=1;
   endif;
   if a4dz[fj] > a4adz[fj]; a4fid[fj]=1;
   endif;
   if a4d2z[fj] > a4ad2z[fj]; a4fid2[fj]=1;
   endif;

   fj=fj+1;
endo;

retp(fdd,fdd2,a4fdd,a4fdd2,fid,fid2,a4fid,a4fid2);endp;

@ ****************************** @
  proc (0) = HITCF(fo,act,n,m);
@ ******************************
*  Counting hits and misses:
*  1 = UD, 2 = DD, 3 = DU, 4 = UU
*  for: central forecasts
*  **************************** @
local  UD,DD,DU,UU,HPR,i,j,HITFOR,den,num,tmp1,tmp2,
       hrfor,ksfor,ptf,sumud,sumdd,sumdu,sumuu;

HPR = zeros(n,m);  @ hit and miss: 1=UD, 2=DD, 3=DU, 4=UU @
UD = zeros(m,1); DD = zeros(m,1); DU = zeros(m,1); UU = zeros(m,1);

i=1;
do until i > n;  @ number of horizons @
   j=1;
   do until j > m; @ number of events @

      if fo[i,j] > 0;
         if act[i,j] < 0; hpr[i,j]=1; ud[j] = ud[j]+1;
         else;            hpr[i,j]=4; uu[j] = uu[j]+1;
         endif;
      else;
         if act[i,j] < 0; hpr[i,j]=2; dd[j] = dd[j]+1;
         else;            hpr[i,j]=3; du[j] = du[j]+1;
         endif;
      endif;

      j=j+1;
   endo;
   i=i+1;
endo;

@* SUMMARY results for ups and downs @
HITFOR = UD~DD~DU~UU;  @ m by 4 @

sumUD = sumc(UD);
sumDD = sumc(DD);
sumDU = sumc(DU);
sumUU = sumc(UU);

@* hit ratio = (DD+UU)/(UD+DD+DU+UU) @
den = sumUD + sumDD + sumDU + sumUU;
num = sumDD + sumUU;
hrfor = num/den;

@ Kuipers Score = H-F,
* where H = UU/(UU+UD) and F = DU/(DU+DD) 
* *************************************** @
tmp1 = sumUU/(sumUU+sumUD);
tmp2 = sumDU/(sumDU+sumDD);
ksfor = tmp1-tmp2;

@* Pesaran-Timmerman test @
{ ptf } = PT(UD,DD,DU,UU);

format /rd 4,0;
"Forecast Evaluation";
"  UD   DD   DU   UU";
HITFOR;
"Sum";
sumud~sumdd~sumdu~sumuu;"";
format /rd 6,3;"hit rate: ";; hrfor;
"Kuipers score: ";; ksfor;
"Pesaran-Timmerman test: ";; ptf;"";

retp;endp;

@ ****************************** @
  proc (0) = HITPR(spr,act,n,m);
@ ******************************
* Coounting hits and misses:
* 1 = UD, 2 = DD, 3 = DU, 4 = UU
* pr: prob forecast with furture and/or parameter uncertainty
* ************************************************************ @
local  UD,DD,DU,UU,HPR,i,j,HITSPR,den,num,tmp1,tmp2,
       hrSPR,ksSPR,pts,sumud,sumdd,sumdu,sumuu;

HPR = zeros(n,m);  @ hit and miss: 1=UD, 2=DD, 3=DU, 4=UU @
UD = zeros(m,1); DD = zeros(m,1); DU = zeros(m,1); UU = zeros(m,1);

i=1;
do until i > n;  @ number of horizons @
   j = 1;
   do until j > m; @ number of events @

      if spr[i,j] > 0.5;
         if act[i,j] < 0; hpr[i,j]=1; ud[j] = ud[j] + 1;
         else;            hpr[i,j]=4; uu[j] = uu[j] + 1;
         endif;
      else;
         if act[i,j] < 0; hpr[i,j] = 2; dd[j] = dd[j] + 1;
         else;            hpr[i,j] = 3; du[j] = du[j] + 1;
         endif;

      endif;

      j=j+1;
   endo;
   i=i+1;
endo;

@* SUMMARY results for ups and downs @
HITSPR = UD~DD~DU~UU;  @ m by 4 @

sumUD = sumc(UD);
sumDD = sumc(DD);
sumDU = sumc(DU);
sumUU = sumc(UU);

@* hit ratio = (DD+UU)/(UD+DD+DU+UU) @
den = sumUD + sumDD + sumDU + sumUU;
num = sumDD + sumUU;
hrSPR = num/den;

@ Kuipers Score = H-F,
* where H = UU/(UU+UD) and F = DU/(DU+DD) 
* *************************************** @
tmp1 = sumUU/(sumUU+sumUD);
tmp2 = sumDU/(sumDU+sumDD);
ksSPR = tmp1-tmp2;

@* Pesaran-Timmerman test @
{ pts } = PT(UD,DD,DU,UU);

format /rd 4,0;
"Forecast Evaluation";
"  UD   DD   DU   UU"; HITSPR;
"Sum"; sumud~sumdd~sumdu~sumuu;"";
format /rd 6,3;"hit rate: ";; hrSPR;
"Kuipers score: ";; ksSPR;
"Pesaran-Timmerman test: ";; pts;"";

retp;endp;

@ *************************** @
   proc (1) = PT(UD,DD,DU,UU);
@ ***************************
* Computing the Pesaran-Timmerman test
* ************************************ @
local n,p,py,px,ps,varp,varps,pttest;

n = sumc(UD) + sumc(DD) + sumc(DU) + sumc(UU);

p = (sumc(DD)+sumc(UU))/n;
py = (sumc(UD)+sumc(UU))/n;
px = (sumc(DU)+sumc(UU))/n;
ps = py*px + (1-py)*(1-px);

varp = ps*(1-ps)/n;
varps = (((2*py-1)^2)*px*(1-px)/n) + (((2*px-1)^2)*py*(1-py)/n)
      + (4*py*px*(1-py)*(1-px)/n^2);

pttest = (p-ps)/sqrt(varp-varps);

retp(pttest);endp;

@ ************************ @
   proc (0) = KS(IPR,n,m);
@ ************************
* Computing the Kolmogorov-Smirnov test:
* KS = sup_x|F_n(x)-U(x)|, where F_n(x) is the empirical CDF of 
* the probability integral transforms and U(x) is the CDF of U[0,1].
* ****************************************************************** @
local  unif,schk,bchk,chk,semp,bemp,cemp,i,j,sks,bks,cks,nm;

nm = n*m;
unif = seqa(1,1,nm); unif = unif./nm; @ uniform cdf @
chk = unif; @ for checking @
schk = vec(ipr); schk = sortc(schk,1);

semp = zeros(nm,1);
cemp = zeros(nm,1);
i=1;
do until i > nm;
   j=1;
   do until j > nm;

      if schk[i] < unif[j];
      else;  semp[i] = semp[i] + 1;
      endif;
      if chk[i] < unif[j];
      else; cemp[i] = cemp[i] + 1;
      endif;

      j=j+1;
   endo;

   i=i+1;
endo;

semp = semp./nm; cemp = cemp./nm;
sks = semp - unif; cks = cemp - unif;
sks = maxc(abs(sks)); cks = maxc(abs(cks));

format /rd 6,3;
"Kolmogorov-Smirnov statistic";;sks;

retp;endp;

@ **************************************************** @
  proc (7) = PREVENT(sz,sdz,actz,actdz,n,m,DZT,popgr);
@ **************************************************** @
local a4sz,asz,a4sdz,asdz,flag1,flag2,flag3,flag4,flag5,flag6,flag7,
      hat1,hat2,hat3,inidy;

@* constructing 4-quarter moving averages and h-step avg @
{ a4sz, asz }   = A4CHANGE(sz,actz,n,m);   @ levels: n by m @
{ a4sdz, asdz } = A4change(sdz,actdz,n,m); @ 1st differences @

@* Probility evaluation for 7 events: E1-E7 @
@* E1: 4-qt inflation with 12 thresholds, 6th in level @
@* E3: PR( 1.5% < 4qt MA dp < 3%) @
hat1 = a4sz[.,6];   @ n by 1 vec of CPI inflation forecasts @
{ flag1, flag3 } = EVALE1(hat1);  @ n by ?? & n by 1 @

@* E2: 4-qt output growth with 10 thresholds, 7th in 1st difference @
@* E5: Bad growth Pr(4-qt dy <2%) @
hat2 = a4sdz[.,7];   @ n by 1 vec of 4qt MA growth forecasts @
/* hat2 = hat2 + ones(nhorz,1).*popgrowth;  @ ?? plus pop growth @ */
{ flag2, flag5 } = EVALE2(hat2);  @ n by ?? @

@* E4: Recession: Pr(quarterly dy <0 and dy(-1) 0) @
inidy = DZT[7];    @ output growth at T @
hat3 = sdz[.,7];   @ quarterly @
/* inidy = DZT[7] + popgrowth;      @ total output growth at T @
hat3 = sdz[.,7]+ ones(nhorz,1).*popgrowth;   @ quarterly @ */
{ flag4 } = EVALE4(hat3,inidy); @ n by 1 @

@* E6: Joint event of no recession and inflation target @
{ flag6 } = EVALE6(hat1,hat3,inidy); @ n by 1 @

@* E7: Joint event of good growth and inflation target:
   Pr(4 qt MA dy > 2% and 1.5% < 4-qt dp < 3.5%) @
{ flag7 } = EVALE7(hat1,hat2); @ n by 1 @

retp(flag1,flag2,flag3,flag4,flag5,flag6,flag7);endp;

@ *********************** @
  proc (2) = EVALE1(hat);
@ ************************ 
* Prob evaluation of inflation < a% 
* ********************************* @
local n,thr,thr1,thr2,flag,ti,tj,flag1;

@* thresholds as % per annum @
thr = seqa(0,0.5,14); @ 0-6.5% @
thr1 = 1.5; thr2 = 3.5; @ between 1.5 and 3.5% @
thr = thr./400;  @ quarterly rates @
thr1 = thr1./400; thr2 = thr2./400;  
 
n = rows(hat);  @ forecast horizon @
flag = zeros(n,rows(thr));  @ number of thresholds @
flag1 = zeros(n,1);
ti = 1;
do until ti > n;
   tj = 1;
   do until tj > rows(thr);
      if hat[ti] < thr[tj];  flag[ti,tj] = 1;  @ pr(dp < a%) @
      endif;
      tj = tj + 1;
   endo;

@ pr(1.5% < dp < 3.5%) @
   if hat[ti] > thr1;  if hat[ti] < thr2; flag1[ti] = 1;
                       endif;
   endif;
   ti = ti + 1;
endo;

retp(flag,flag1);endp;

@ *********************** @
  proc (2) = EVALE2(hat);
@ ************************ 
* Prob evaluation of output growth < a% 
* ************************************* @
local n,thr,thr1,flag,flag1,ti,tj;

@* thresholds as % per annum @
thr = seqa(0,1,10); @ 0-9% @
thr1 = 2; @ below 2%: bad growth @ 
thr = thr./400;  @ quarterly rates @
thr1 = thr1./400;

n = rows(hat);  @ forecast horizon @
flag = zeros(n,rows(thr));  @ No. of thresholds @
flag1 = zeros(n,1);
ti = 1;
do until ti > n;
   tj = 1;
   do until tj > rows(thr);
      if hat[ti] < thr[tj];  flag[ti,tj] = 1;
      endif;
      tj = tj + 1;
   endo;
@ pr(g < 2%): bad growth @
   if hat[ti] < thr1;  flag1[ti] = 1;
   endif;
   ti = ti + 1;
endo;

retp(flag,flag1);endp;

@ *************************** @
  proc (1) = EVALE4(hat,dy0);
@ *************************** 
* Prob evaluation of recession 
* **************************** @
local n,thr,flag,ti;

thr = 0; thr = thr./400;  @ quarterly rate threshold @
n = rows(hat);  @ forecast horizon @
flag = zeros(n,1);
ti = 1;
do until ti > n;
   if ti eq 1;
      if hat[ti] < thr;
         if dy0 < thr;       flag[ti] = 1;
         endif;
      endif;
   else;
      if hat[ti] < thr;
         if hat[ti-1] < thr; flag[ti] = 1;
         endif;
      endif;
   endif;
   ti = ti + 1;
endo;

retp(flag);endp;

@ ********************************* @
  proc (1) = EVALE6(hat1,hat3,dy0);
@ ********************************* 
* E6: Joint event of no recession and inflation target 
* **************************************************** @
local n,thr1,thr3,flag,ti;

thr1 = zeros(2,1);  @ inflation percent per annum @
thr1[1] = 1.5; thr1[2] = 3.5;
thr3 = 0;           @ output growth @
thr1 = thr1./400; thr3 = thr3./400;   @ quarterly rates @
n = rows(hat1);  @ forecast horizon @
flag = zeros(n,1);
ti = 1;
do until ti > n;
   if ti eq 1;
      if hat3[ti] < thr3;   @ dy < 0% @
         if dy0 < thr3;     @ dy(-1) < 0% @
         else;
            if hat1[ti] > thr1[1];     @ dp > 1.5% @
               if hat1[ti] < thr1[2];  @ dp < 3.5% @
                  flag[ti] = 1;
               endif;
            endif;
         endif;
      else;   @ dy > 0% @
         if hat1[ti] > thr1[1];     @ dp > 1.5% @
            if hat1[ti] < thr1[2];  @ dp < 3.5% @
               flag[ti] = 1;
            endif;
         endif;
      endif;
   else;
      if hat3[ti] < thr3;       @ dy < 0% @
         if hat3[ti-1] < thr3;  @ dy(-1) < 0% @
         else;
            if hat1[ti] > thr1[1];     @ dp > 1.5% @
               if hat1[ti] < thr1[2];  @ dp < 3.5% @
                  flag[ti] = 1;
               endif;
            endif;
         endif;
      else;   @ dy > 0% @
         if hat1[ti] > thr1[1];     @ dp > 1.5% @
            if hat1[ti] < thr1[2];  @ dp < 3.5% @
               flag[ti] = 1;
            endif;
         endif;
      endif;
   endif;
   ti = ti + 1;
endo;

retp(flag);endp;

@ ***************************** @
  proc (1) = EVALE7(hat1,hat2);
@ ***************************** 
* E7: Joint event of good growth and BOE inflation target:
* Pr(4 qt MA dy > 1% and 1.5% < 4-qt dp < 3.5%) 
* ********************************************* @
local n,thr1,thr2,flag,ti;

thr1 = zeros(2,1);  @ inflation percent per annum @
thr1[1] = 1.5; thr1[2] = 3.5;
thr2 = 1;           @ output growth @
thr1 = thr1./400; thr2 = thr2./400;   @ quarterly rates @
n = rows(hat1);  @ forecast horizon @
flag = zeros(n,1);
ti = 1;
do until ti > n;
   if hat2[ti] < thr2;
   else;  @ dy > 1% @
          if hat1[ti] > thr1[1];  @ dp > 1.5% @
             if hat1[ti] < thr1[2]; flag[ti] = 1;   @ dp < 3.5% @
             endif;
          endif;
   endif;
   ti = ti + 1;
endo;

retp(flag);endp;

@ ********************************* @
  proc (7) = EMPDIST(ForZ,nh,m,ns);
@ ********************************* @
@ Empirical mean, variance and CIs of central forecasts @
@ ***************************************************** @
local nhm,mean,med,std,L90,U90,L95,U95,j,tmp;

nhm = nh*m;
mean = zeros(nhm,1); @ empirical mean: n*m by 1 @
med = zeros(nhm,1);  @ empirical median @
std = zeros(nhm,1);  @ empirical standard deviation @
L90 = zeros(nhm,1);  @ 90% lower confidence bound @
U90 = zeros(nhm,1);  @ 90% upper confidence bound @
L95 = zeros(nhm,1);  @ 95% lower confidence bound @
U95 = zeros(nhm,1);  @ 95% upper confidence bound @

j=1;
do until j > rows(ForZ);
   tmp = ForZ[j,.]';
   { mean[j], med[j], std[j], l90[j], u90[j], l95[j], u95[j] } 
   = EMPD(tmp,ns);
   j=j+1;
endo;

mean = TRMAT(mean,nh,m);
med = TRMAT(med,nh,m);
std = TRMAT(std,nh,m);
L90 = TRMAT(l90,nh,m);
U90 = TRMAT(u90,nh,m);
L95 = TRMAT(l95,nh,m);
U95 = TRMAT(u95,nh,m);

retp(mean,med,std,L90,U90,L95,U95);endp;

@ ************************ @
  proc (7) = EMPD(tmp,n1);
@ ************************ @
local mean,med,std,fr50,l90,u90,l95,u95,stmp;

l90 = n1*0.05;   l90 = ceil(l90);
u90 = n1*0.95;   u90 = floor(u90);
l95 = n1*0.025;  l95 = ceil(l95);
u95 = n1*0.975;  u95 = floor(u95);
fr50 = n1*0.5;   fr50 = round(fr50); 

mean = meanc(tmp);
std = stdc(tmp);

stmp = sortc(tmp,1);
l90 = stmp[l90];
u90 = stmp[u90];
l95 = stmp[l95];
u95 = stmp[u95];
med = stmp[fr50];

retp(mean,med,std,l90,u90,l95,u95);endp;

@ ***************************** @
  proc (1) = TRMAT(vect,ro,co);
@ ***************************** 
* Converting nm by 1 vector to n by m matrix 
* ****************************************** @
local matri,smj,rj,cj;

matri = zeros(ro,co);
smj = 1;
do until smj > co;
   rj = (smj-1)*ro + 1;
   cj = smj*ro;
   matri[.,smj] = vect[rj:cj];
   smj = smj + 1;
endo;

retp(Matri);endp;

@ ************************************** @
  proc (3) = BCREG(Zin,DX,p,case,br,jj);
@ ************************************************
* CONSTRUCTION of ZS1 and all other regressors, YY
* in recursive bootstrap
* jj: t-index, br: break point index
* ********************************** @
local  tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,tmp9,tmp10,
       DZIN,Z0,DZ0,DZlags,jj0,id1,td1,did,dtd,ZS0,YY;

DZIN = ZIN-lag(ZIN);
Z0 = ZIN[rows(ZIN),.]; DZ0 = DZIN[rows(DZIN),.]; @ 1 by m @

if p eq 1;
elseif p eq 2; 
   DZlags = DZin;
elseif p eq 3;
   tmp1 = lag(DZIN); DZlags = DZIN~tmp1;
elseif p eq 4;
   tmp1 = lag(DZIN); tmp2 = lag(tmp1);
   DZlags = DZIN~tmp1~tmp2;
elseif p eq 5;
   tmp1=lag(DZIN); tmp2=lag(tmp1); tmp3=lag(tmp2);
   DZlags = DZIN~tmp1~tmp2~tmp3;
elseif p eq 6;
   tmp1=lag(DZIN); tmp2=lag(tmp1); tmp3=lag(tmp2); tmp4=lag(tmp3);
   DZlags = DZIN~tmp1~tmp2~tmp3~tmp4;
elseif p eq 7;
   tmp1=lag(DZIN); tmp2=lag(tmp1); tmp3=lag(tmp2); tmp4=lag(tmp3);
   tmp5=lag(tmp4); DZlags = DZIN~tmp1~tmp2~tmp3~tmp4~tmp5;
elseif p eq 8;
   tmp1=lag(DZIN); tmp2=lag(tmp1); tmp3=lag(tmp2); tmp4=lag(tmp3);
   tmp5=lag(tmp4); tmp6=lag(tmp5); 
   DZlags = DZIN~tmp1~tmp2~tmp3~tmp4~tmp5~tmp6;
elseif p eq 9;
   tmp1=lag(DZIN); tmp2=lag(tmp1); tmp3=lag(tmp2); tmp4=lag(tmp3);
   tmp5=lag(tmp4); tmp6=lag(tmp5); tmp7=lag(tmp6); 
   DZlags = DZIN~tmp1~tmp2~tmp3~tmp4~tmp5~tmp6~tmp7;
elseif p eq 10;
   tmp1=lag(DZIN); tmp2=lag(tmp1); tmp3=lag(tmp2); tmp4=lag(tmp3);
   tmp5=lag(tmp4); tmp6=lag(tmp5); tmp7=lag(tmp6); tmp8=lag(tmp7);
   DZlags = DZIN~tmp1~tmp2~tmp3~tmp4~tmp5~tmp6~tmp7~tmp8;
elseif p eq 11;
   tmp1=lag(DZIN); tmp2=lag(tmp1); tmp3=lag(tmp2); tmp4=lag(tmp3);
   tmp5=lag(tmp4); tmp6=lag(tmp5); tmp7=lag(tmp6); tmp8=lag(tmp7);
   tmp9=lag(tmp8);
   DZlags = DZIN~tmp1~tmp2~tmp3~tmp4~tmp5~tmp6~tmp7~tmp8~tmp9;
elseif p eq 12;
   tmp1=lag(DZIN); tmp2=lag(tmp1); tmp3=lag(tmp2); tmp4=lag(tmp3);
   tmp5=lag(tmp4); tmp6=lag(tmp5); tmp7=lag(tmp6); tmp8=lag(tmp7);
   tmp9=lag(tmp8);tmp10=lag(tmp9);
   DZlags = DZIN~tmp1~tmp2~tmp3~tmp4~tmp5~tmp6~tmp7~tmp8~tmp9~tmp10;
endif;
@* sample adjustment @
DZlags = DZlags[rows(DZlags),.]; @ 1 by m(p-1) @

jj0 = jj-1; @ t-1 @
id1 = 0; td1 = 0; @ intercept and tr dummies @
did = zeros(p,1); dtd = zeros(p,1); @ differences @
if jj < br; 
else; id1 = 1; td1 = 1+(jj-br); dtd[1] = 1;
endif;

if jj eq br; did[1] = 1;
elseif jj eq br+1; did[2] = 1; 
endif;

if jj < br+1; 
else; dtd[2] = 1;
endif;

if     case eq 1; ZS0 = Z0'; 
elseif case eq 2; ZS0 = Z0'|1;
elseif case eq 3; ZS0 = Z0';
elseif case eq 4; ZS0 = Z0'|jj0;
elseif case eq 6; ZS0 = Z0'|jj0|id1;
elseif case eq 7; ZS0 = Z0'|jj0|td1;
endif;

@ DX: contemporaneous: mx by sample @

if p eq 1; YY = DX[.,jj];
else; YY = DZLAGS'|DX[.,jj];
endif;

if     case eq 3; YY = YY|1;
elseif case eq 4; YY = YY|1;
elseif case eq 5; YY = YY|jj;
elseif case eq 6; YY = YY|1|did;
elseif case eq 7; YY = YY|1|dtd; @ ?? @
endif;

Z0 = Z0'; @ m by 1 @
retp(Z0,ZS0,YY);endp;


@* Long-run structural FIML estimation @ 

@ **************************************** @
  proc (1) = INICOIN(ms,r,bet,Rmat,bvec);
@ ****************************************
* Initial estimates of restricted cointegrating vectors:
* Transform Johansen beta using the whole restrictions
* ****************************************************** @
local  Q1,Q2,Ir,IQ2,theta0,beta0;

Ir = eye(rcoin);
Q1 = Ir.*.bet; Q2 = Rmat*Q1;
IQ2 = INV(Q2'Q2);
theta0 = Q1*IQ2*Q2'bvec;
beta0 = TRMAT(theta0,ms,rcoin); @ ms by r @

retp(beta0); endp;

@ ********************************************* @
  proc (3) = AOest(rbeta,DY,ZS1,YY,m,sample,r);
@ ********************************************* @
local  Ralpha,Romega,MYY,vDY,bzb,ibzb,bz,rkapa,rPAI,E0,MLL;

MYY = eye(sample) - YY'invpd(YY*YY')*YY; @ idempotent @
vDY = vec(DY');
bzb = rbeta'ZS1*MYY*ZS1'rbeta;
ibzb = inv(bzb); @ invpd ?? @
bz = rbeta'ZS1*MYY;
rkapa = eye(m).*.(ibzb*bz)*vDY;

ralpha = TRMAT(rkapa,r,m); @ r by m: @
ralpha = ralpha'; @ since kapa = vec(alpha') @

rPAI = ralpha*rbeta'; 
E0 = DY' - ZS1'rPAI';
rOmega = E0'MYY*E0./sample;

MLL = -(m*sample/2)*(1+ln(2*pi))-(sample/2)*ln(det(romega));

retp(Ralpha,Romega,MLL);endp;

@ ******************************************************** @
  proc (2) = Best(Ralpha,ROmega,DY,ZS1,YY,ms,r,Rmat,bvec);
@ ******************************************************** @
@* estimation of beta subject to restrictions @
local  Rbeta,U11,MYY,vDY,aoa,z1Mz1,Atheta,aom,z1M,Btheta,Jtheta,IJth,
       rth,rtheta;

MYY = eye(sample) - YY'invpd(YY*YY')*YY; @ idempotent @
vDY = vec(DY');
aoa = Ralpha'ROmega*Ralpha;
z1Mz1 = ZS1*MYY*ZS1';
Atheta = aoa.*.z1Mz1;
aom = Ralpha'ROmega;
z1M = ZS1*MYY;
Btheta = aom.*.z1M;
Jtheta = Atheta + Rmat'Rmat;
IJth = inv(Jtheta);
rth = IJth*Btheta*vDY;
rtheta = rth - IJth*Rmat'inv(Rmat*IJth*Rmat')*(Rmat*rth-bvec); 
rbeta = TRMAT(rtheta,ms,r); @ ms by r @
U11 = IJth - IJth*Rmat'Inv(Rmat*IJth*Rmat')*Rmat*IJth;

retp(Rbeta,U11);endp;
