' ************************************************************************************************
' *	CHAPTER 6 MODEL IT (PROCYCLICAL INFLATION TARGETING)	*
' *			         NOVEMBER 14, 2008				*
' ************************************************************************************************

' This code draws heavily on the Eviews macro for model OPENFLEX from ch. 12
' of Godley and Lavoie (2007) Monetary Economics: An Integrated Approach to
' Money, Income, Production and Wealth written by Gennaro Zezza. The original
' version of the code is available at:

'	http://gennaro.zezza.it/software/eviews/v6/gl12openflex.prg

' The copyright notice from Gennaro's code:

' Permission is hereby granted, free of charge, to any person obtaining a 
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense, 
' and/or sell copies of the Software, and to permit persons to whom the 
' Software is furnished to do so, subject to the following conditions:
' 
' The above copyright notice and this permission notice shall be included in 
' all copies or substantial portions of the Software.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
' EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
' OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-
' INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
' HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
' WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
' OTHER DEALINGS IN THE SOFTWARE.
' ************************************************************************************************


' Create a workfile, naming it open_IT2, to hold annual data from 1950 to 2050

wfcreate(wf = open_IT2, page = annual) a 1950 2050
smpl @all



' Define endogenous and exogenous variables
' ----------------------------------------------------------------


series b_ukuk_d
b_ukuk_d.label(d) Bills issued by the UK acquired by the UK: demand

series b_ukuk_s
b_ukuk_s.label(d) Bills issued by the UK acquired by the UK: supply

series b_cb_ukus_d
b_cb_ukus_d.label(d) Bills issued by the US, demanded by the UK Central Bank

series b_cb_ukus_s
b_cb_ukus_s.label(d) Bills issued by the US, supplied to the UK Central Bank

series b_cb_ukuk_d
b_cb_ukuk_d.label(d) Bills issued by the UK, demanded by the UK Central Bank

series b_cb_ukuk_s
b_cb_ukuk_s.label(d) Bills issued by the UK, supplied to the UK Central Bank

series b_uk_s
b_uk_s.label(d) Bills issued by the UK - total supply

series b_ukus_d
b_ukus_d.label(d) Bills issued by the US acquired by the UK: demand

series b_ukus_s
b_ukus_s.label(d) Bills issued by the US acquired by the UK: supply

series b_usuk_d
b_usuk_d.label(d) Bills issued by the UK acquired by the US: demand

series b_usuk_s
b_usuk_s.label(d) Bills issued by the UK acquired by the US: supply

series b_us_s
b_us_s.label(d) Bills issued by the US - total supply

series b_usus_d
b_usus_d.label(d) Bills issued by the US acquired by the US: demand

series b_usus_s
b_usus_s.label(d) Bills issued by the US acquired by the US: supply

series b_cb_usus_d
b_cb_usus_d.label(d) Bills issued by the US demanded by the US Central Bank

series b_cb_usus_s
b_cb_usus_s.label(d) Bills issued by the US supplied to the US Central Bank

series c_k_uk
c_k_uk.label(d) Real consumption in the UK

series c_k_us
c_k_us.label(d) Real consumption in the US

series cab_uk
cab_uk.label(d) Current account balance in the UK

series cab_us
cab_us.label(d) Current account balance in the US

series cons_uk
cons_uk.label(d) Consumption in the UK

series cons_us
cons_us.label(d) Consumption in the US

series ds_uk
ds_uk.label(d) Domestic sales in the UK

series ds_us
ds_us.label(d) Domestic sales in the US

series ds_k_uk
ds_k_uk.label(d) Real domestic sales in the UK

series ds_k_us
ds_k_us.label(d) Real domestic sales in the US

series dxre_uk
dxre_uk.label(d) Expected change in the exchange rate of the UK (measured as units of the UK currency against 1 unit of the US currency)

series dxre_us
dxre_us.label(d) Expected change in the exchange rate the US (measured as units of the US currency against 1 unit of the UK currency)

series f_cb_uk
f_cb_uk.label(d) Profits of Central Bank in the UK

series f_cb_us
f_cb_us.label(d) Profits of Central Bank in the US

series g_uk
g_uk.label(d) Government expenditure in the UK

series g_us
g_us.label(d) Government expenditure in the US

series g_k_uk
g_k_uk.label(d) Real government expenditure in the UK

series g_k_us
g_k_us.label(d) Real government expenditure in the US

series h_uk_d
h_uk_d.label(d) Demand for cash of the UK

series h_uk_s
h_uk_s.label(d) Supply of the UK cash

series h_us_d
h_us_d.label(d) Demand for cash of the US

series h_us_s
h_us_s.label(d) Supply of the US cash

series im_uk
im_uk.label(d) Imports of the UK from the US

series im_us
im_us.label(d) Imports of the US from the UK

series im_k_uk
im_k_uk.label(d) Real imports of the UK from the US

series im_k_us
im_k_us.label(d) Real imports of the US from the UK

series kab_uk
kab_uk.label(d) Capital account balance in the UK

series kab_us
kab_us.label(d) Current account balance in the US

series kabp_uk
kabp_uk.label(d) Capital account balance in the UK, excluding official transactions

series kabp_us
kabp_us.label(d) Current account balance in the US, excluding official transactions

series n_uk
n_uk.label(d) Employment in the UK

series n_us
n_us.label(d) Employment in the US

series n_fe_uk
n_fe_uk.label(d) UK full employment level

series n_fe_us
n_fe_us.label(d) US full employment level

series omegat_uk
omegat_uk.label(d) Target real wage for workers

series omegat_us
omegat_us.label(d) Target real wage for workers

series or_uk
or_uk.label(d) Gold reserves in the UK

series or_us
or_us.label(d) Gold reserves in the US

series pds_uk
pds_uk.label(d) Price of domestic sales in the UK

series pds_us
pds_us.label(d) Price of domestic sales in the US

series pg_uk
pg_uk.label(d) Price of gold in the UK

series pg_us
pg_us.label(d) Price of gold in the US

series pi_ds_uk
pi_ds_uk.label(d) UK domestic sales price inflation

series pi_ds_us
pi_ds_us.label(d) US domestic sales price inflation

series pi_s_uk
pi_s_uk.label(d) UK sales price inflation

series pi_s_us
pi_s_us.label(d) US sales price inflation

series pm_us
pm_us.label(d) Price of imports in the US

series pm_uk
pm_uk.label(d) Price of imports in the UK

series pr_uk
pr_uk.label(d) Productivity in the UK

series pr_us
pr_us.label(d) Productivity in the US

series ps_uk
ps_uk.label(d) Price of sales in the UK

series ps_us
ps_us.label(d) Price of sales in the US

series psbr_uk
psbr_uk.label(d) Government deficit in the UK

series psbr_us
psbr_us.label(d) Government deficit in the US

series py_us
py_us.label(d) Price of output in the US

series py_uk
py_uk.label(d) Price of imports in the UK

series px_us
px_us.label(d) Price of exports in the US

series px_uk
px_uk.label(d) Price of exports in the UK

series r_uk
r_uk.label(d) Interest rate on UK bills

series r_us
r_us.label(d) Interest rate on US bills

series rr_uk
rr_uk.label(d) UK real interest rate

series rr_us
rr_us.label(d) US real interest rate

series s_uk
s_uk.label(d) Value of sales in the UK

series s_us
s_us.label(d) Value of sales in the US

series s_k_uk
s_k_uk.label(d) Real sales in the UK

series s_k_us
s_k_us.label(d) Real sales in the US

series t_uk
t_uk.label(d) Tax revenue in the UK

series t_us
t_us.label(d) Tax revenue in the US

series uc_uk
uc_uk.label(d) Unit costs in the UK

series uc_us
uc_us.label(d) Unit costs in the US

series v_uk
v_uk.label(d) Net financial assets of the UK

series v_us
v_us.label(d) Net financial assets of the US

series v_k_uk
v_k_uk.label(d) Real net financial assets of the UK

series v_k_us
v_k_us.label(d) Real net financial assets of the US

series w_uk
w_uk.label(d) Nominal wage rate in the UK

series w_us
w_us.label(d) Nominal wage rate in the US

series wb_uk
wb_uk.label(d) UK wage bill

series wb_us
wb_us.label(d) US wage bill

series x_uk
x_uk.label(d) Exports from the UK to the US

series x_us
x_us.label(d) Exports from the US to the UK

series x_k_uk
x_k_uk.label(d) Real exports from the UK to the US

series x_k_us
x_k_us.label(d) Real exports from the US to the UK

series xr_uk
xr_uk.label(d) Exchange rate: units of S currency against 1 unit of N currency

series xr_us
xr_us.label(d) Exchange rate: units of N currency against 1 unit of S currency

series xre_uk
xre_uk.label(d) Expected exchange rate: units of S currency against 1 unit of N currency

series xre_us
xre_us.label(d) Expected exchange rate: units of N currency against 1 unit of S currency

series yd_uk
yd_uk.label(d) Disposable income in the UK

series yd_us
yd_us.label(d) Disposable income in the US

series ydhs_uk
ydhs_uk.label(d) Haig-Simons disposable income in the UK

series ydhs_us
ydhs_us.label(d) Haig-Simons disposable income in the US

series ydhs_k_uk
ydhs_k_uk.label(d) Real Haig-Simons disposable income in the UK

series ydhs_k_us
ydhs_k_us.label(d) Real Haig-Simons disposable income in the US

series ydhse_k_uk
ydhse_k_uk.label(d) Expected real Haig-Simons disposable income in the UK

series ydhse_k_us
ydhse_k_us.label(d) Expected real Haig-Simons disposable income in the US

series y_uk
y_uk.label(d) Income in the UK

series y_us
y_us.label(d) Income in the US

series y_k_uk
y_k_uk.label(d) Real income in the UK

series y_k_us
y_k_us.label(d) Real income in the US

series z1_uk
z1_uk.label(d) Indicator variable =1 if alpha1_uk<alpha1l_uk

series z1_us
z1_us.label(d) Indicator variable =1 if alpha1_us<alpha1l_us

series z2_uk
z2_uk.label(d) Indicator variable =1 if alpha1_us>alpha1u_uk

series z2_us
z2_us.label(d) Indicator variable =1 if alpha1_us>alpha1u_us

series z3_uk
z3_uk.label(d) Indicator variable =1 if alpha2_uk<alpha2l_uk

series z3_us
z3_us.label(d) Indicator variable =1 if alpha2_us<alpha2l_us

series z4_uk
z4_uk.label(d) Indicator variable =1 if alpha2_us>alpha2u_uk

series z4_us
z4_us.label(d) Indicator variable =1 if alpha2_us>alpha2u_us

series z5_uk
z5_uk.label(d) Indicator variable =1 if r_uk>0

series z5_us
z5_us.label(d) Indicator variable =1 if r_us>0

series z6_uk
z6_uk.label(d) Indicator variable =1 if pi_ds_uk(-1) > infu_uk

series z6_us
z6_us.label(d) Indicator variable =1 if pi_ds_us(-1) > infu_us

series z7_uk
z7_uk.label(d) Indicator variable =1 if pi_ds_uk(-1) < infl_uk

series z7_us
z7_us.label(d) Indicator variable =1 if pi_ds_us(-1) < infl_us



' Define model parameters
' ------------------------------------

series alpha1_us
alpha1_us.label(d) Propensity to consume out of income the US

series alpha1_us0
alpha1_us0.label(d) Initial steady state propensity to consume out of income the US

series alpha2_us
alpha2_us.label(d) Propensity to consume out of wealth the US

series alpha2_us0
alpha2_us0.label(d) Initial steady state propensity to consume out of wealth the US

series alpha1_uk
alpha1_uk.label(d) Propensity to consume out of income the UK

series alpha1_uk0
alpha1_uk0.label(d) Initial steady state propensity to consume out of income the UK

series alpha2_uk
alpha2_uk.label(d) Propensity to consume out of wealth the UK

series alpha2_uk0
alpha2_uk0.label(d) Initial steady state propensity to consume out of wealth the UK

series alpha1l_uk
alpha1l_uk.label(d) Lower bound for alpha1_uk

series alpha1l_us
alpha1l_us.label(d) Lower bound for alpha1_us

series alpha1u_uk
alpha1u_uk.label(d) Upper bound for alpha1_uk

series alpha1u_us
alpha1u_us.label(d) Upper bound for alpha1_us

series alpha2l_uk
alpha2l_uk.label(d) Lower bound for alpha2_uk

series alpha2l_us
alpha2l_us.label(d) Lower bound for alpha2_us

series alpha2u_uk
alpha2u_uk.label(d) Upper bound for alpha2_uk

series alpha2u_us
alpha2u_us.label(d) Upper bound for alpha2_us

series eps0
eps0.label(d) Parameter determining real exports in the UK

series eps1
eps1.label(d) Parameter determining real exports in the UK

series eps2
eps2.label(d) Parameter determining real exports in the UK

series infl_uk
infl_uk.label(d) Lower bound of inflation target range in the UK

series infl_us
infl_us.label(d) Lower bound of inflation target range in the US

series infu_uk
infu_uk.label(d) Upper bound of inflation target range in the UK

series infu_us
infu_us.label(d) Upper bound of inflation target range in the US

series lambda10
lambda10.label(d) Parameter in asset demand function

series lambda11
lambda11.label(d) Parameter in asset demand function

series lambda12
lambda12.label(d) Parameter in asset demand function

series lambda20
lambda20.label(d) Parameter in asset demand function

series lambda21
lambda21.label(d) Parameter in asset demand function

series lambda22
lambda22.label(d) Parameter in asset demand function

series lambda40
lambda40.label(d) Parameter in asset demand function

series lambda41
lambda41.label(d) Parameter in asset demand function

series lambda42
lambda42.label(d) Parameter in asset demand function

series lambda50
lambda50.label(d) Parameter in asset demand function

series lambda51
lambda51.label(d) Parameter in asset demand function

series lambda52
lambda52.label(d) Parameter in asset demand function

series mu0
mu0.label(d) Parameter determining real imports in the UK

series mu1
mu1.label(d) Parameter determining real imports in the UK

series mu2
mu2.label(d) Parameter determining real imports in the UK

series mu_a1_uk
mu_a1_uk.label(d) Parameter in UK endogenous mpc equation

series mu_a1_us
mu_a1_us.label(d) Parameter in US endogenous mpc equation

series mu_a2_uk
mu_a2_uk.label(d) Parameter in UK endogenous mpc equation

series mu_a2_us
mu_a2_us.label(d) Parameter in US endogenous mpc equation

series nu1m
nu1m.label(d) Parameter determining import prices in the UK

series nu0m
nu0m.label(d) Parameter determining import prices in the UK

series nu1m
nu1m.label(d) Parameter determining import prices in the UK

series nu0x
nu0x.label(d) Parameter determining export prices in the UK

series nu1x
nu1x.label(d) Parameter determining export prices in the UK

series omega0_uk
omega0_uk.label(d) Parameter influencing the target real wage for workers

series omega1_uk
omega1_uk.label(d) Parameter influencing the target real wage for workers

series omega2_uk
omega2_uk.label(d) Parameter influencing the target real wage for workers

series omega3_uk
omega3_uk.label(d) Speed of adjustment of wages to target value

series omega0_us
omega0_us.label(d) Parameter influencing the target real wage for workers

series omega1_us
omega1_us.label(d) Parameter influencing the target real wage for workers

series omega2_us
omega2_us.label(d) Parameter influencing the target real wage for workers

series omega3_us
omega3_us.label(d) Speed of adjustment of wages to target value

series phi_us
phi_us.label(d) mark-up in the US

series phi_uk
phi_uk.label(d) mark-up in the UK

series pi0_uk
pi0_uk.label(d) UK inflation in the initial steady state

series pi0_us
pi0_us.label(d) US inflation in the initial steady state

series rr0_uk
rr0_uk.label(d) UK real interest rate in the initial steady state

series rr0_us
rr0_us.label(d) US real interest rate in the initial steady state

series sigma_fisc_uk
sigma_fisc_uk.label(d) Paramater in UK fiscal policy rule

series sigma_fisc_us
sigma_fisc_us.label(d) Parameter in US fiscal policy rule

series theta_uk
theta_uk.label(d) Tax rate in the UK

series theta_us
theta_us.label(d) Tax rate in the US

series theta1_uk
theta1_uk.label(d) Parameter in UK monetary policy rule

series theta1_us
theta1_us.label(d) Paramater in US monetary policy rule

series varrho_uk
varrho_uk.label(d) Parameter in UK monetary policy rule

series varrho_us
varrho_us.label(d) Parameter in US monetary policy rule

series zeta
zeta.label(d) Parameter in leader-follower rule



' Set starting values for parameters
' -------------------------------------------------

alpha1_uk = 0.75
alpha1_us = 0.75
alpha1_uk0 = 0.75
alpha1_us0 = 0.75
alpha2_uk0 = 0.13333
alpha2_us0 = 0.13333
alpha1l_uk = 0.65
alpha1l_us = 0.65
alpha1u_uk = 0.85
alpha1u_us = 0.85
alpha2l_uk = 0.08333
alpha2l_us = 0.08333
alpha2u_uk = 0.18333
alpha2u_us = 0.18333
alpha2_uk = 0.13333
alpha2_us = 0.13333
eps0 = - 2.1
eps1 = 0.7
eps2 = 1
infu_uk=0.003
infu_us=0.003
infl_uk=0.000
infl_us=0.000
lambda10 = 0.7
lambda11 = 5
lambda12 = 5
lambda20 = 0.25
lambda21 = 5
lambda22 = 5
lambda40 = 0.7
lambda41 = 5
lambda42 = 5
lambda50 = 0.25
lambda51 = 5
lambda52 = 5
mu0 = - 2.1
mu1 = 0.7
mu2 = 1
mu_a1_uk = 2.1
mu_a1_us = 2.1
mu_a2_uk = 0.35
mu_a2_us = 0.35
nu0m = - 0.00001
nu0x = - 0.00001
nu1m = 0.7
nu1x = 0.5
omega0_uk = -0.1 '-0.20594
omega0_us = -0.1 '-0.20594
omega1_uk = 1
omega1_us = 1
omega2_uk = 1.5
omega2_us = 1.5
omega3_uk = 0.15
omega3_us = 0.15
phi_uk = 0.2381
phi_us = 0.2381
pi0_uk = 0.00083933
pi0_us = 0.000845125
rr0_uk = 0.030135376
rr0_us = 0.030129412
sigma_fisc_uk = 0.15
sigma_fisc_us = 0.15
theta_uk = 0.2
theta_us = 0.2
theta1_uk = 0.0001
theta1_us = 0.0001
varrho_uk = 0.5
varrho_us = 0.5
zeta = 0.5



' Set starting values for exogenous and endogenous variables
' --------------------------------------------------------------------------------------

b_cb_ukus_s = 0.02031
dxre_us = 0
n_fe_uk = 80
n_fe_us = 80
or_uk = 7
pg_us = 1
pr_uk = 1.3333
pr_us = 1.3333

g_k_uk = 16
g_k_us = 16
r_uk = 0.031
r_us = 0.031

B_CB_UKUK_D=2.188644
B_CB_UKUK_S=2.188644
B_CB_UKUS_D=0.0202905
B_CB_USUS_D=2.214648
B_CB_USUS_S=2.214648
B_UK_S=177.128
B_UKUK_D=128.9051
B_UKUK_S=128.9051
B_UKUS_D=46.03752
B_UKUS_S=46.08167
B_US_S=177.3367
B_USUK_D=46.07859
B_USUK_S=46.03444
B_USUS_D=129.0201
B_USUS_S=129.0201
C_K_UK=81.30053
C_K_US=81.30145
CAB_UK=0.000263402
CAB_US=-0.000263655
CONS_UK=98.21106
CONS_US=98.29868
DS_K_UK=97.30053
DS_K_US=97.30145
DS_UK=117.5391
DS_US=117.6437
DXRE_UK=0
F_CB_UK=0.0682375
F_CB_US=0.0684128
G_K_US=16
G_UK=19.328
G_US=19.34503
H_UK_D=9.207504
H_UK_S=9.207504
H_US_D=9.215718
H_US_S=9.215718
IM_K_UK=11.91557
IM_K_US=11.915
IM_UK=14.39307
IM_US=14.40642
KABP_UK=-0.000263401
KABP_US=0.000263653
N_UK=72.97679
N_US=72.97834
OMEGAT_UK=1.051092
OMEGAT_US=1.051125
OR_US=7
PDS_UK=1.208
PDS_US=1.209064
PG_UK=0.9990419
PI_DS_UK=0.00083933
PI_DS_US=0.000845125
PI_S_UK=0.000839333
PI_S_US=0.000845123
PM_UK=1.207921
PM_US=1.209099
PS_UK=1.207994
PS_US=1.209066
PSBR_UK=0.1544907
PSBR_US=0.1556978
PX_UK=1.207941
PX_US=1.20908
PY_UK=1.208003
PY_US=1.209062
S_K_UK=109.2155
S_K_US=109.217
S_UK=131.9317
S_US=132.0506
T_UK=24.59146
T_US=24.61353
V_K_UK=152.4421
V_K_US=152.4438
V_UK=184.1501
V_US=184.3144
W_UK=1.262959
W_US=1.264066
X_K_UK=11.915
X_K_US=11.91557
X_UK=14.39261
X_US=14.40687
XR_UK=1.000959
XR_US=0.9990419
XRE_UK=1.000959
XRE_US=0.9990419
Y_K_UK=97.29996
Y_K_US=97.30202
Y_UK=117.5386
Y_US=117.6442
YD_UK=98.36556
YD_US=98.45437
YDHS_K_UK=81.30058
YDHS_K_US=81.3015
YDHSE_K_UK=81.30058
YDHSE_K_US=81.3015



' Initiate the model
' -------------------------

model open_IT2



' Firms' Equations
' ------------------------

' Value of output in the UK - eq. 6.1
	open_IT2.append y_uk = s_uk - im_uk
'	open_IT2.append y_uk=cons_uk+g_uk+x_uk-im_uk

' Value of real output in the UK - eq. 6.2
	open_IT2.append y_k_uk = s_k_uk - im_k_uk

' Employment in the UK - eq. 6.3
	open_IT2.append n_uk = y_k_uk/pr_uk

' Wage bill in the the UK - eq. 6.4
	open_IT2.append wb_uk=w_uk*n_uk

' Unit costs in the UK - eq. 6.5
	open_IT2.append uc_uk=(wb_uk+im_uk)/s_k_uk

' Price of sales in the UK - eq. 6.6
	open_IT2.append ps_uk = (1 + phi_uk)*(w_uk*n_uk + im_uk)/s_k_uk

' Real sales in the UK - eq. 6.7
	open_IT2.append s_k_uk = c_k_uk + g_k_uk + x_k_uk

' Nominal value of sales in the UK - eq. 6.8
	open_IT2.append s_uk = s_k_uk*ps_uk

' Value of output in the US - eq. 6.9
	open_IT2.append y_us = s_us - im_us
'	open_IT2.append y_us=cons_us+g_us+x_us-im_us

' Value of real output in the US - eq. 6.10
	open_IT2.append y_k_us = s_k_us - im_k_us

' Employment in the US - eq. 6.11
	open_IT2.append n_us = y_k_us/pr_us

' Wage bill in the US - eq. 6.12
	open_IT2.append wb_us=w_us*n_us

' Unit costs in the US - eq. 6.13
	open_IT2.append uc_us=(wb_us+im_us)/s_k_us

' Price of sales in the US - eq. 6.14
	open_IT2.append ps_us = (1 + phi_us)*(w_us*n_us + im_us)/s_k_us

' Real sales in the US - eq. 6.15
	open_IT2.append s_k_us = c_k_us + g_k_us + x_k_us

' Nominal value of sales in the US - eq. 6.16
	open_IT2.append s_us = s_k_us*ps_us



' Household Equations
' -------------------------------

' UK household disposable income - eq. 6.17
	open_IT2.append yd_uk = (y_uk + r_uk(-1)*b_ukuk_d(-1) + xr_us*r_us(-1)*b_ukus_s(-1))*(1 - theta_uk)

' Haig-Simons disposable income in the UK - eq. 6.18
	open_IT2.append yd_hs_uk = yd_uk + d(xr_us)*b_ukus_s(-1)

' Wealth accumulation in the UK - eq. 6.19
	open_IT2.append v_uk = v_uk(-1) + yd_hs_uk - cons_uk

' Real Haig-Simons disposable income in the UK - eq. 6.20
	open_IT2.append ydhs_k_uk = yd_uk/pds_uk - v_k_uk(-1)*d(pds_uk)/pds_uk

' Real wealth in the UK - eq. 6.21
	open_IT2.append v_k_uk = v_uk/pds_uk

' Real consumption in the UK - eq. 6.22
	open_IT2.append c_k_uk = alpha1_uk*ydhse_k_uk + alpha2_uk*v_k_uk(-1)

' Expected real Haig-Simons disposable income in the UK - eq. 6.23
	open_IT2.append ydhse_k_uk = (ydhs_k_uk + ydhs_k_uk(-1))/2

' Nominal consumption in the UK - eq. 6.24
	open_IT2.append cons_uk = c_k_uk*pds_uk

' US household disposable income - eq. 6.25
	open_IT2.append yd_us = (y_us + r_us(-1)*b_usus_d(-1) + xr_uk*r_uk(-1)*b_usuk_s(-1))*(1 - theta_us)

' Haig-Simons disposable income in the US - eq. 6.26
	open_IT2.append yd_hs_us = yd_us + d(xr_uk)*b_usuk_s(-1)

' Wealth accumulation in the US - eq. 6.27
	open_IT2.append v_us = v_us(-1) + yd_hs_us - cons_us

' Real Haig-Simons disposable income in the US - eq. 6.28
	open_IT2.append ydhs_k_us = yd_us/pds_us - v_k_us(-1)*d(pds_us)/pds_us 

' Real wealth in the US - eq. 6.29
	open_IT2.append v_k_us = v_us/pds_us

' Real consumption in the US - eq. 6.30
	open_IT2.append c_k_us = alpha1_us*ydhse_k_us + alpha2_us*v_k_us(-1)

' Expected real Haig-Simons disposable income in the US - eq. 6.31
	open_IT2.append ydhse_k_us = (ydhs_k_us + ydhs_k_us(-1))/2

' Nominal consumption in the US - eq. 6.32
	open_IT2.append cons_us = c_k_us*pds_us

' UK mpc out of income as a function of the real interest rate - eq. 6.33
	open_IT2.append alpha1_uk=(alpha1_uk0-mu_a1_uk*((((1+r_uk)/(1+pi_ds_uk))-1)-rr0_uk))*(1-z1_uk)*(1-z2_uk)+z1_uk*alpha1l_uk+z2_uk*alpha1u_uk

' US mpc out of income as a function of the real interest rate - eq. 6.34
	open_IT2.append alpha1_us=(alpha1_us0-mu_a1_us*((((1+r_us)/(1+pi_ds_us))-1)-rr0_us))*(1-z1_us)*(1-z2_us)+z1_us*alpha1l_us+z2_us*alpha1u_us

' Indicator variable (does UK mpc out of income exceed upper bound?) - eq. 6.35
	open_IT2.append z1_uk=0+(alpha1_uk<alpha1l_uk)

' Indicator variable (does US mpc out of income exceed upper bound?) - eq. 6.36
	open_IT2.append z1_us=0+(alpha1_us<alpha1l_us)

' Indicator variable (does UK mpc out of income breach lower bound?) - eq. 6.37
	open_IT2.append z2_uk=0+(alpha1_uk>alpha1u_uk)

' Indicator variable (does US mpc out of income breach lower bound?) - eq. 6.38
	open_IT2.append z2_us=0+(alpha1_us>alpha1u_us)

' Real interest rate in the UK - eq. 6.39
	open_IT2.append rr_uk=(1+r_uk)/(1+pi_ds_uk)-1

' Real interest rate in the US - eq. 6.40
	open_IT2.append rr_us=(1+r_us)/(1+pi_ds_us)-1

' UK mpc out of wealth as a function of the real interest rate - eq. 6.41
	open_IT2.append alpha2_uk=(alpha2_uk0-mu_a2_uk*((((1+r_uk)/(1+pi_ds_uk))-1)-rr0_uk))*(1-z3_uk)*(1-z4_uk)+z3_uk*alpha2l_uk+z4_uk*alpha2u_uk

' US mpc out of income as a function of the real interest rate - eq. 6.42
	open_IT2.append alpha2_us=(alpha2_us0-mu_a2_us*((((1+r_us)/(1+pi_ds_us))-1)-rr0_us))*(1-z3_us)*(1-z4_us)+z3_us*alpha2l_us+z4_us*alpha2u_us

' Indicator variable (does UK mpc out of wealth exceed upper bound?) - eq. 6.43
	open_IT2.append z3_uk=0+(alpha2_uk<alpha2l_uk)

' Indicator variable (does US mpc out of wealth exceed upper bound?) - eq. 6.44
	open_IT2.append z3_us=0+(alpha2_us<alpha2l_us)

' Indicator variable (does UK mpc out of wealth breach lower bound?) - eq. 6.45
	open_IT2.append z4_uk=0+(alpha2_uk>alpha2u_uk)

' Indicator variable (does US mpc out of wealth breach lower bound?) - eq. 6.46
	open_IT2.append z4_us=0+(alpha2_us>alpha2u_us)



' Household portfolio equations
' -------------------------------------------

' UK demand for domestic bills - eq. 6.47
	open_IT2.append b_ukuk_d = v_uk*(lambda10 + lambda11*r_uk - lambda12*(r_us + dxre_us))

' UK demand for foreign bills - eq. 6.48
	open_IT2.append b_ukus_d = v_uk*(lambda20 - lambda21*r_uk + lambda22*(r_us + dxre_us))

' UK demand for money - eq. 6.49
	open_IT2.append h_uk_d = v_uk - b_ukuk_d - b_ukus_d

' US demand for domestic bills - eq. 6.50
	open_IT2.append b_usus_d = v_us*(lambda40 + lambda41*r_us - lambda42*(r_uk + dxre_uk))

' US demand for foreign bills - eq. 6.51
	open_IT2.append b_usuk_d = v_us*(lambda50 - lambda51*r_us + lambda52*(r_uk + dxre_uk))

' US demand for money - eq. 6.52
	open_IT2.append h_us_d = v_us - b_usus_d - b_usuk_d

' NOTE: Expected change in the exchange rate xr
'	open_IT2.append dxre_uk = (xre_uk - xr_uk(-1))/xr_uk

' NOTE: Expected change in the exchange rate xr$
'	open_IT2.append dxre_us = (xre_us - xr_us(-1))/xr_us



' Government Equations
' ---------------------------------

' UK income tax - eq. 6.53
	open_IT2.append t_uk = theta_uk*(y_uk + r_uk(-1)*b_ukuk_d(-1) + xr_us*r_us(-1)*b_ukus_s(-1))

' Equation 6.54: real UK govt expenditure is exogenous (not in model CFP)

' Nominal government expenditure in the UK - eq. 6.55
	open_IT2.append g_uk = g_k_uk*pds_uk

' Government deficit in the UK - eq. 6.56
	open_IT2.append psbr_uk = g_uk + r_uk(-1)*b_uk_s(-1) - t_uk - f_cb_uk

' Government budget constraint in the UK - eq. 6.57
	open_IT2.append b_uk_s = b_uk_s(-1) + g_uk + r_uk(-1)*b_uk_s(-1) - t_uk - f_cb_uk

' US income tax - eq. 6.58
	open_IT2.append t_us = theta_us*(y_us + r_us(-1)*b_usus_d(-1) + xr_uk*r_uk(-1)*b_usuk_s(-1))

' Equation 6.59: real US govt expenditure is exogenous (not in model CFP)

' Nominal government expenditure in the US - eq. 6.60
	open_IT2.append g_us = g_k_us*pds_us

' Government deficit in the US - eq. 6.61
	open_IT2.append psbr_us = g_us + r_us(-1)*b_us_s(-1) - t_us - f_cb_us

' Government budget constraint in the US - eq. 6.62
	open_IT2.append b_us_s = b_us_s(-1) + g_us + r_us(-1)*b_us_s(-1) - t_us - f_cb_us



' Inflationary Forces
' ---------------------------

' Target real wage for UK workers - eq. 6.63
	open_IT2.append omegat_uk = exp(omega0_uk + omega1_uk*log(pr_uk) + omega2_uk*log((n_uk/n_fe_uk)))

' Nominal wages in the UK - eq. 6.64
	open_IT2.append w_uk = w_uk(-1)*(1 + omega3_uk*(omegat_uk(-1) - w_uk(-1)/pds_uk(-1)))

' UK sales price inflation - eq. 6.65
	open_IT2.append pi_s_uk = ps_uk/ps_uk(-1) - 1

' Domestic sales in the UK - eq. 6.66
	open_IT2.append ds_uk = s_uk - x_uk

' Real domestic sales in the UK - eq. 6.67
	open_IT2.append ds_k_uk = c_k_uk + g_k_uk

' UK domestic sales price - eq. 6.68
	open_IT2.append pds_uk = (s_uk - x_uk)/(s_k_uk - x_k_uk)

' UK domestic sales price inflation - eq. 6.69
open_IT2.append pi_ds_uk = pds_uk/pds_uk(-1) - 1

' Target real wage for US workers - eq. 6.70
	open_IT2.append omegat_us = exp(omega0_us + omega1_us*log(pr_us) + omega2_us*log((n_us/n_fe_us)))

' Nominal wages in the US - eq. 6.71
	open_IT2.append w_us = w_us(-1)*(1 + omega3_us*(omegat_us(-1) - w_us(-1)/pds_us(-1)))

' US sales price inflation - eq. 6.72
	open_IT2.append pi_s_us = ps_us/ps_us(-1) - 1

' Domestic sales in the US - eq. 6.73
	open_IT2.append ds_us = s_us - x_us

' Real domestic sales in the US - eq. 6.74
	open_IT2.append ds_k_us = c_k_us + g_k_us

' US domestic sales price - eq. 6.75
	open_IT2.append pds_us = (s_us - x_us)/(s_k_us - x_k_us)

' US domestic sales price inflation - eq. 6.76
	open_IT2.append pi_ds_us = pds_us/pds_us(-1) - 1



' Trade and the Balance of Payments
' ----------------------------------------------------

' UK import price - eq. 6.77
	open_IT2.append pm_uk = exp(nu0m + nu1m*log(py_us) + (1 - nu1m)*log(py_uk) - nu1m*log(xr_uk))

' UK export price - eq. 6.78
	open_IT2.append px_uk = exp(nu0x + nu1x*log(py_us) + (1 - nu1x)*log(py_uk) - nu1x*log(xr_uk))

' UK real exports - eq. 6.79 (current price ratio follows Lavoie's version of ch12)
	open_IT2.append x_k_uk = exp(eps0 - eps1*log(pm_us/py_us) + eps2*log(y_k_us))

' UK real imports - eq. 6.80
	open_IT2.append im_k_uk = exp(mu0 - mu1*log(pm_uk(-1)/py_uk(-1)) + mu2*log(y_k_uk))

' UK GDP deflator - eq. 6.81
	open_IT2.append py_uk = y_uk/y_k_uk

' US GDP deflator - eq. 6.82
	open_IT2.append py_us = y_us/y_k_us

' US export price - eq. 6.83
	open_IT2.append px_us = pm_uk*xr_uk

' US import price - eq. 6.84
	open_IT2.append pm_us = px_uk*xr_uk

' US real exports - eq. 6.85
	open_IT2.append x_k_us = im_k_uk

' US real imports - eq. 6.86
	open_IT2.append im_k_us = x_k_uk

' UK nominal exports - eq. 6.87
	open_IT2.append x_uk = x_k_uk*px_uk

' US nominal exports - eq. 6.88
	open_IT2.append x_us = x_k_us*px_us

' UK nominal imports - eq. 6.89
	open_IT2.append im_uk = im_k_uk*pm_uk

' US nominal imports - eq. 6.90
	open_IT2.append im_us = im_k_us*pm_us

' UK current account balance - eq. 6.91
	open_IT2.append cab_uk = x_uk - im_uk + xr_us*r_us(-1)*b_ukus_s(-1) - r_uk(-1)*b_usuk_s(-1) + r_us(-1)*b_cb_ukus_s(-1)*xr_us

' UK capital account balance including OSA - eq. 6.92
	open_IT2.append kab_uk = kabp_uk - (xr_us*d(b_cb_ukus_s) + pg_uk*d(or_uk))

' UK capital account balance  excluding OSA - eq. 6.93
	open_IT2.append kabp_uk = - d(b_ukus_s)*xr_us + d(b_usuk_s)

' US current account balance - eq. 6.94
	open_IT2.append cab_us = x_us - im_us + xr_uk*r_uk(-1)*b_usuk_s(-1) - r_us(-1)*b_ukus_s(-1) - r_us(-1)*b_cb_ukus_s(-1)

' US capital account balance - eq. 6.95
	open_IT2.append kab_us = kabp_us + d(b_cb_ukus_s) - pg_us*d(or_us)

' US capital account balance excluding OSA - eq. 6.96
	open_IT2.append kabp_us = - d(b_usuk_s)*xr_uk + d(b_ukus_s)

' NOTE: Net accumulation of financial assets in the UK
	open_IT2.append nafa_uk = psbr_uk + cab_uk

' NOTE: Net accumulation of financial assets in the US
	open_IT2.append nafa_us = psbr_us + cab_us



' Central Bank Equations
' ----------------------------------

' UK money supply - eq. 6.97
	open_IT2.append h_uk_s = h_uk_d

' Supply of domestic bills to the UK - eq. 6.98
	open_IT2.append b_ukuk_s = b_ukuk_d

' Supply of domestic bills to the UK Central Bank - eq. 6.99
	open_IT2.append b_cb_ukuk_s = b_cb_ukuk_d

' Balance sheet of the UK Central Bank - eq. 6.100
	open_IT2.append b_cb_ukuk_d = b_cb_ukuk_d(-1) + d(h_uk_s) - d(b_cb_ukus_s)*xr_us - d(or_uk)*pg_uk

' Price of gold in the UK - eq. 6.101
	open_IT2.append pg_uk = pg_us*xr_us

' Profits of the UK Central Bank - eq. 6.102
	open_IT2.append f_cb_uk = r_uk(-1)*b_cb_ukuk_d(-1) + r_us(-1)*b_cb_ukus_s(-1)*xr_us

' Equation 6.103: UK interest rate is exogenous (not in models IT, LF, CFPIT and CFPLF)

' US money supply - eq. 6.104
	open_IT2.append h_us_s = h_us_d

' Supply of domestic bills to the US - eq. 6.105
	open_IT2.append b_usus_s = b_usus_d

' Supply of domestic bills to the US Central Bank - eq. 6.106
	open_IT2.append b_cb_usus_s = b_cb_usus_d

' Balance sheet of US Central Bank - eq. 6.107
	open_IT2.append b_cb_usus_d = b_cb_usus_d(-1) + d(h_us_s) - d(or_us)*pg_us

' Equation 6.108: Price of gold in Dollars is fixed

' Profits of the US Central Bank - eq. 6.109
	open_IT2.append f_cb_us = r_us(-1)*b_cb_usus_d(-1)

' Equation 6.110: US interest rate is exogenous (not in models IT, LF, CFPIT and CFPLF)

' Equilibrium condition for UK bills acquired by the US - eq. 6.111
	open_IT2.append b_usuk_s = b_usuk_d*xr_us

' Equilibrium condition for US bills acquired by the UK Central Bank - eq. 6.112
	open_IT2.append b_cb_ukus_d = b_cb_ukus_s*xr_us

' Equation 6.113: Supply of US bills to the UK Central Bank is fixed

' Supply of US bills to UK households - eq. 6.114
	open_IT2.append b_ukus_s = b_us_s - b_cb_ukus_s - b_usus_s - b_cb_usus_s

' UK Exchange rate - eq. 6.115
	open_IT2.append xr_uk = b_ukus_s/b_ukus_d

' US exchange rate - eq. 6.116
	open_IT2.append xr_us = 1/xr_uk

' UK real exchange rate - eq. 6.117
	open_IT2.append rxr_uk = xr_uk*pds_uk/pds_us

' US real exchange rate - eq. 6.118
	open_IT2.append rxr_us = xr_us*pds_us/pds_uk



' Stabilisation Policies
' ------------------------------

' Indicator variable (is UK interest rate positive?) - eq. 6.119
	open_IT2.append z5_uk=1-(r_uk<0)

' Indicator variable (is US interest rate positive?) - eq. 6.120
	open_IT2.append z5_us=1-(r_us<0)

' Indicator variable (did UK inflation exceed the upper bound?) - eq. 6.121
	open_IT2.append z6_uk = 0 + (pi_ds_uk(-1) > infu_uk)

' Indicator variable (was UK inflation below the lower bound?) - eq. 6.122
	open_IT2.append z7_uk = 0 + (pi_ds_uk(-1) < infl_uk)

' Indicator variable (did US inflation exceed the upper bound?) - eq. 6.123
	open_IT2.append z6_us = 0 + (pi_ds_us(-1) > infu_us)

' Indicator variable (was UK inflation below the lower bound?) - eq. 6.124
	open_IT2.append z7_us = 0 + (pi_ds_us(-1) < infl_us)





' 1) Procyclical Inflation Targeting Monetary Policy (Taylor Principle)

' open_IT2.append r_uk=(rr0_uk+pi_ds_uk+varrho_uk*(pi_ds_uk-pi0_uk))*z5_uk
'	open_IT2.append r_us=(rr0_us+pi_ds_uk+varrho_us*(pi_ds_us-pi0_us))*z5_us



' 2) Interest Rate Leadership

' open_IT2.append r_uk=(zeta*(rr0_uk+pi_ds_uk+varrho_uk*(pi_ds_uk-pi0_uk))+(1-zeta)*r_us)*z5_uk
'	open_IT2.append r_us=(rr0_us+pi_ds_uk+varrho_us*(pi_ds_us-pi0_us))*z5_us



' 3) Counter-Cyclical Anti-Inflationary Fiscal Policy (band target)

' open_IT2.append g_k_uk = g_k_uk(-1) - sigma_fisc_uk*z6_uk + sigma_fisc_uk*z7_uk
'	open_IT2.append g_k_us = g_k_us(-1) - sigma_fisc_us*z6_us + sigma_fisc_us*z7_us

' Alternatively, using a point target:

' open_IT2.append g_k_uk = g_k_uk(-1) - 100*sigma_fisc_uk*(pi_ds_uk-pi0_uk)
'	open_IT2.append g_k_us = g_k_us(-1) - 100*sigma_fisc_us*(pi_ds_us-pi0_us)



' Solve the model for the baseline case
' ------------------------------------------------------

smpl 1952 @last
	open_IT2.scenario Baseline
		open_IT2.solve(m=50000,o=m,i=p)



' The redundant equation
' ----------------------------------

series red
red.label(d) Right hand side of redundant equation
red =  b_ukuk_d_0 + b_usuk_s_0 + b_cb_ukuk_d_0

' Plots the two sides of the redundant equation
smpl 1952 2050
graph redundant.line b_uk_s_0 red
redundant.options linepat
redundant.setelem(1) lcolor(black) lwidth(2) lpat(1)
redundant.setelem(2) lcolor(black) lwidth(2) lpat(2)
redundant.name(1) UK bill supply
redundant.name(2) Sum of bills supplied to UK and US households and the BoE 



' Experiment 1: Decreasing export propensity in the UK
' -----------------------------------------------------------------------------

smpl @all

' Store original value of shocked value to baseline
genr eps0_0 = eps0

' Increase UK propensity to export
smpl 1960 @last
eps0 = - 2.2
smpl @all

' Select the first alternative Scenario 1
open_IT2.scenario "Scenario 1"

' Set simulation sample
smpl @first+2 @last

' Solve the model for the current sample
open_IT2.solve(m=50000,o=m,i=p)

' Store shocked value to scenario
genr eps0_1 = eps0
' and get back shocked value to baseline value
eps0 = eps0_0

' Creates graphs

' Current account, trade balance and PSBR in the UK
smpl 1955 2050
graph ukcab_lf_exp1.line cab_uk_1 (x_uk_1-im_uk_1) psbr_uk_1
ukcab_lf_exp1.options linepat
ukcab_lf_exp1.setelem(1) lcolor(black) lwidth(2) lpat(1)
ukcab_lf_exp1.setelem(2) lcolor(black) lwidth(2) lpat(2)
ukcab_lf_exp1.setelem(3) lcolor(black) lwidth(2) lpat(3)
ukcab_lf_exp1.name(1) UK current account balance
ukcab_lf_exp1.name(2) UK trade balance
ukcab_lf_exp1.name(3) UK budget deficit
ukcab_lf_exp1.legend -display
ukcab_lf_exp1.options size(4,2) -inbox
	ukcab_lf_exp1.save(t=postscript) c:\Progs\ch6\sfc\graphs\ukcab_lf_exp1

' Current account, trade balance and PSBR in the US
smpl 1955 2050
graph uscab_lf_exp1.line cab_us_1 (x_us_1-im_us_1) psbr_us_1
uscab_lf_exp1.options linepat
uscab_lf_exp1.setelem(1) lcolor(black) lwidth(2) lpat(1)
uscab_lf_exp1.setelem(2) lcolor(black) lwidth(2) lpat(2)
uscab_lf_exp1.setelem(3) lcolor(black) lwidth(2) lpat(3)
uscab_lf_exp1.name(1) US current account balance
uscab_lf_exp1.name(2) US trade balance
uscab_lf_exp1.name(3) US budget deficit
uscab_lf_exp1.legend -display
uscab_lf_exp1.options size(4,2) -inbox
	uscab_lf_exp1.save(t=postscript) c:\Progs\ch6\sfc\graphs\uscab_lf_exp1

' Inflation relative to target
smpl 1955 2050
graph inf_lf_exp1.line pi_ds_uk_1 pi_ds_us_1 infl_uk infu_uk
inf_lf_exp1.options linepat
inf_lf_exp1.setelem(1) lcolor(black) lwidth(2) lpat(1)
inf_lf_exp1.setelem(2) lcolor(black) lwidth(2) lpat(3)
inf_lf_exp1.setelem(3) lcolor(black) lwidth(1) lpat(2)
inf_lf_exp1.setelem(4) lcolor(black) lwidth(1) lpat(2)
inf_lf_exp1.name(1) UK domestic price inflation
inf_lf_exp1.name(2) US domestic price inflation
inf_lf_exp1.name(3) Upper bound of target range
inf_lf_exp1.name(4) Lower bound of target range
inf_lf_exp1.legend -display
inf_lf_exp1.options size(4,2) -inbox
	inf_lf_exp1.save(t=postscript) c:\Progs\ch6\sfc\graphs\inf_lf_exp1

' Real GDP
smpl 1955 2050
graph yk_lf_exp1.line y_k_uk_1 y_k_us_1
yk_lf_exp1.options linepat
yk_lf_exp1.setelem(1) lcolor(black) lwidth(2) lpat(1)
yk_lf_exp1.setelem(2) lcolor(black) lwidth(2) lpat(2)
yk_lf_exp1.name(1) UK real GDP
yk_lf_exp1.name(2) US real GDP
yk_lf_exp1.legend -display
yk_lf_exp1.options size(4,2) -inbox
	yk_lf_exp1.save(t=postscript) c:\Progs\ch6\sfc\graphs\yk_lf_exp1

' Real consumption
smpl 1955 2050
graph ck_lf_exp1.line c_k_uk_1 c_k_us_1
ck_lf_exp1.options linepat
ck_lf_exp1.setelem(1) lcolor(black) lwidth(2) lpat(1)
ck_lf_exp1.setelem(2) lcolor(black) lwidth(2) lpat(2)
ck_lf_exp1.name(1) UK real consumption
ck_lf_exp1.name(2) US real consumption
ck_lf_exp1.legend -display
ck_lf_exp1.options size(4,2) -inbox
	ck_lf_exp1.save(t=postscript) c:\Progs\ch6\sfc\graphs\ck_lf_exp1

' Exchange rates
smpl 1955 2050
graph xr_lf_exp1.line xr_uk_1 rxr_uk_1
xr_lf_exp1.options linepat
xr_lf_exp1.setelem(1) lcolor(black) lwidth(2) lpat(1)
xr_lf_exp1.setelem(2) lcolor(black) lwidth(2) lpat(2)
xr_lf_exp1.name(1) US Dollars per Pound Sterling
xr_lf_exp1.name(2) UK real exchange rate
xr_lf_exp1.legend -display
xr_lf_exp1.options size(4,2) -inbox
	xr_lf_exp1.save(t=postscript) c:\Progs\ch6\sfc\graphs\xr_lf_exp1



' Experiment 2: Increased target real wage in the UK 
' ---------------------------------------------------------------------------

smpl @all

' Store original value of shocked value to baseline
genr omega0_uk_0 = omega0_uk

' Increase wage inflation in the UK
smpl 1960 @last
omega0_uk = 0
smpl @all

' Select the seocnd alternative Scenario 2

open_IT2.scenario(n, a="_2") "Scenario 2"

' Set simulation sample
smpl @first+2 @last

' Solve the model for the current sample
open_IT2.solve(m=50000,o=m,i=p)

' Store shocked value to scenario
genr omega0_uk_2 = omega0_uk
' and get back shocked value to baseline value
omega0_uk = omega0_uk_0

' Creates graphs

' Current account, trade balance and PSBR in the UK
smpl 1955 2050
graph ukcab_lf_exp2.line cab_uk_2 (x_uk_2-im_uk_2) psbr_uk_2
ukcab_lf_exp2.options linepat
ukcab_lf_exp2.setelem(1) lcolor(black) lwidth(2) lpat(1)
ukcab_lf_exp2.setelem(2) lcolor(black) lwidth(2) lpat(2)
ukcab_lf_exp2.setelem(3) lcolor(black) lwidth(2) lpat(3)
ukcab_lf_exp2.name(1) UK current account balance
ukcab_lf_exp2.name(2) UK trade balance
ukcab_lf_exp2.name(3) UK budget deficit
ukcab_lf_exp2.legend -display
ukcab_lf_exp2.options size(4,2) -inbox
	ukcab_lf_exp2.save(t=postscript) c:\Progs\ch6\sfc\graphs\ukcab_lf_exp2

' Current account, trade balance and PSBR in the US
smpl 1955 2050
graph uscab_lf_exp2.line cab_us_2 (x_us_2-im_us_2) psbr_us_2
uscab_lf_exp2.options linepat
uscab_lf_exp2.setelem(1) lcolor(black) lwidth(2) lpat(1)
uscab_lf_exp2.setelem(2) lcolor(black) lwidth(2) lpat(2)
uscab_lf_exp2.setelem(3) lcolor(black) lwidth(2) lpat(3)
uscab_lf_exp2.name(1) US current account balance
uscab_lf_exp2.name(2) US trade balance
uscab_lf_exp2.name(3) US budget deficit
uscab_lf_exp2.legend -display
uscab_lf_exp2.options size(4,2) -inbox
	uscab_lf_exp2.save(t=postscript) c:\Progs\ch6\sfc\graphs\uscab_lf_exp2

' Inflation relative to target
smpl 1955 2050
graph inf_lf_exp2.line pi_ds_uk_2 pi_ds_us_2 infl_uk infu_uk
inf_lf_exp2.options linepat
inf_lf_exp2.setelem(1) lcolor(black) lwidth(2) lpat(1)
inf_lf_exp2.setelem(2) lcolor(black) lwidth(2) lpat(3)
inf_lf_exp2.setelem(3) lcolor(black) lwidth(1) lpat(2)
inf_lf_exp2.setelem(4) lcolor(black) lwidth(1) lpat(2)
inf_lf_exp2.name(1) UK domestic price inflation
inf_lf_exp2.name(2) US domestic price inflation
inf_lf_exp2.name(3) Upper bound of target range
inf_lf_exp2.name(4) Lower bound of target range
inf_lf_exp2.legend -display
inf_lf_exp2.options size(4,2) -inbox
	inf_lf_exp2.save(t=postscript) c:\Progs\ch6\sfc\graphs\inf_lf_exp2

' Real GDP
smpl 1955 2050
graph yk_lf_exp2.line y_k_uk_2 y_k_us_2
yk_lf_exp2.options linepat
yk_lf_exp2.setelem(1) lcolor(black) lwidth(2) lpat(1)
yk_lf_exp2.setelem(2) lcolor(black) lwidth(2) lpat(2)
yk_lf_exp2.name(1) UK real GDP
yk_lf_exp2.name(2) US real GDP
yk_lf_exp2.legend -display
yk_lf_exp2.options size(4,2) -inbox
	yk_lf_exp2.save(t=postscript) c:\Progs\ch6\sfc\graphs\yk_lf_exp2

' Real consumption
smpl 1955 2050
graph ck_lf_exp2.line c_k_uk_2 c_k_us_2
ck_lf_exp2.options linepat
ck_lf_exp2.setelem(1) lcolor(black) lwidth(2) lpat(1)
ck_lf_exp2.setelem(2) lcolor(black) lwidth(2) lpat(2)
ck_lf_exp2.name(1) UK real consumption
ck_lf_exp2.name(2) US real consumption
ck_lf_exp2.legend -display
ck_lf_exp2.options size(4,2) -inbox
	ck_lf_exp2.save(t=postscript) c:\Progs\ch6\sfc\graphs\ck_lf_exp2

' Exchange rates
smpl 1955 2050
graph xr_lf_exp2.line xr_uk_2 rxr_uk_2
xr_lf_exp2.options linepat
xr_lf_exp2.setelem(1) lcolor(black) lwidth(2) lpat(1)
xr_lf_exp2.setelem(2) lcolor(black) lwidth(2) lpat(2)
xr_lf_exp2.name(1) US Dollars per Pound Sterling
xr_lf_exp2.name(2) UK real exchange rate
xr_lf_exp2.legend -display
xr_lf_exp2.options size(4,2) -inbox
	xr_lf_exp2.save(t=postscript) c:\Progs\ch6\sfc\graphs\xr_lf_exp2



' Experiment 3: Expansionary US tax cut
' ----------------------------------------------------------

smpl @all

' Store original value of shocked value to baseline
genr theta_us_0 = theta_us

' Decrease the tax rate in the US
smpl 1960 @last
theta_us = 0.19
smpl @all

' Select the third alternative Scenario 3

open_IT2.scenario(n, a="_3") "Scenario 3"

' Set simulation sample
smpl @first+2 @last

' Solve the model for the current sample
open_IT2.solve(m=50000,o=m,i=p)

' Store shocked value to scenario
genr theta_us_3 = theta_us
' and get back shocked value to baseline value
theta_us = theta_us_0

' Creates graphs

' Current account, trade balance and PSBR in the UK
smpl 1955 2050
graph ukcab_lf_exp3.line cab_uk_3 (x_uk_3-im_uk_3) psbr_uk_3
ukcab_lf_exp3.options linepat
ukcab_lf_exp3.setelem(1) lcolor(black) lwidth(2) lpat(1)
ukcab_lf_exp3.setelem(2) lcolor(black) lwidth(2) lpat(2)
ukcab_lf_exp3.setelem(3) lcolor(black) lwidth(2) lpat(3)
ukcab_lf_exp3.name(1) UK current account balance
ukcab_lf_exp3.name(2) UK trade balance
ukcab_lf_exp3.name(3) UK budget deficit
ukcab_lf_exp3.legend -display
ukcab_lf_exp3.options size(4,2) -inbox
	ukcab_lf_exp3.save(t=postscript) c:\Progs\ch6\sfc\graphs\ukcab_lf_exp3

' Current account, trade balance and PSBR in the US
smpl 1955 2050
graph uscab_lf_exp3.line cab_us_3 (x_us_3-im_us_3) psbr_us_3
uscab_lf_exp3.options linepat
uscab_lf_exp3.setelem(1) lcolor(black) lwidth(2) lpat(1)
uscab_lf_exp3.setelem(2) lcolor(black) lwidth(2) lpat(2)
uscab_lf_exp3.setelem(3) lcolor(black) lwidth(2) lpat(3)
uscab_lf_exp3.name(1) US current account balance
uscab_lf_exp3.name(2) US trade balance
uscab_lf_exp3.name(3) US budget deficit
uscab_lf_exp3.legend -display
uscab_lf_exp3.options size(4,2) -inbox
	uscab_lf_exp3.save(t=postscript) c:\Progs\ch6\sfc\graphs\uscab_lf_exp3

' Inflation relative to target
smpl 1955 2050
graph inf_lf_exp3.line pi_ds_uk_3 pi_ds_us_3 infl_uk infu_uk
inf_lf_exp3.options linepat
inf_lf_exp3.setelem(1) lcolor(black) lwidth(2) lpat(1)
inf_lf_exp3.setelem(2) lcolor(black) lwidth(2) lpat(3)
inf_lf_exp3.setelem(3) lcolor(black) lwidth(1) lpat(2)
inf_lf_exp3.setelem(4) lcolor(black) lwidth(1) lpat(2)
inf_lf_exp3.name(1) UK domestic price inflation
inf_lf_exp3.name(2) US domestic price inflation
inf_lf_exp3.name(3) Upper bound of target range
inf_lf_exp3.name(4) Lower bound of target range
inf_lf_exp3.legend -display
inf_lf_exp3.options size(4,2) -inbox
	inf_lf_exp3.save(t=postscript) c:\Progs\ch6\sfc\graphs\inf_lf_exp3

' Real GDP
smpl 1955 2050
graph yk_lf_exp3.line y_k_uk_3 y_k_us_3
yk_lf_exp3.options linepat
yk_lf_exp3.setelem(1) lcolor(black) lwidth(2) lpat(1)
yk_lf_exp3.setelem(2) lcolor(black) lwidth(2) lpat(2)
yk_lf_exp3.name(1) UK real GDP
yk_lf_exp3.name(2) US real GDP
yk_lf_exp3.legend -display
yk_lf_exp3.options size(4,2) -inbox
	yk_lf_exp3.save(t=postscript) c:\Progs\ch6\sfc\graphs\yk_lf_exp3

' Real consumption
smpl 1955 2050
graph ck_lf_exp3.line c_k_uk_3 c_k_us_3
ck_lf_exp3.options linepat
ck_lf_exp3.setelem(1) lcolor(black) lwidth(2) lpat(1)
ck_lf_exp3.setelem(2) lcolor(black) lwidth(2) lpat(2)
ck_lf_exp3.name(1) UK real consumption
ck_lf_exp3.name(2) US real consumption
ck_lf_exp3.legend -display
ck_lf_exp3.options size(4,2) -inbox
	ck_lf_exp3.save(t=postscript) c:\Progs\ch6\sfc\graphs\ck_lf_exp3

' Exchange rates
smpl 1955 2050
graph xr_lf_exp3.line xr_uk_3 rxr_uk_3
xr_lf_exp3.options linepat
xr_lf_exp3.setelem(1) lcolor(black) lwidth(2) lpat(1)
xr_lf_exp3.setelem(2) lcolor(black) lwidth(2) lpat(2)
xr_lf_exp3.name(1) US Dollars per Pound Sterling
xr_lf_exp3.name(2) UK real exchange rate
xr_lf_exp3.legend -display
xr_lf_exp3.options size(4,2) -inbox
	xr_lf_exp3.save(t=postscript) c:\Progs\ch6\sfc\graphs\xr_lf_exp3

